with Calendar;
with Text_Io;

package body Error_Report is
------------------------------------------------------------------------------
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.
--
--                  All Rights Reserved.
--
-- Permission to use, copy, modify, and distribute this software and its
-- documentation for any purpose and without fee is hereby granted,
-- provided that the above copyright notice(s) appear in all copies and that
-- both that copyright notice(s) and this permission notice appear in
-- supporting documentation, and that the name of Rational not be used in
-- advertising or publicity pertaining to distribution of the software
-- without specific, written prior permission.
--
-- Rational disclaims all warranties with regard to this software, including
-- all implied warranties of merchantability and fitness, in no event shall
-- Rational be liable for any special, indirect or consequential damages or
-- any damages whatsoever resulting from loss of use, data or profits, whether
-- in an action of contract, negligence or other tortious action, arising out
-- of or in connection with the use or performance of this software.
------------------------------------------------------------------------------

    Error_Called : Boolean := False;

    package Inte_Io is new Text_Io.Integer_Io (Integer);

--
    procedure There_Were_Errors (Yes : Boolean) is
------------------------------------------------------------------------------
-- Routines to set or to test the flag that records whether or not the Error
-- routine has ever been called; initially False.
------------------------------------------------------------------------------
    begin
        Error_Called := Yes;
    end There_Were_Errors;

    function There_Were_Errors return Boolean is
------------------------------------------------------------------------------
-- Routines to set or to test the flag that records whether or not the Error
-- routine has ever been called; initially False.
------------------------------------------------------------------------------
    begin
        return Error_Called;
    end There_Were_Errors;

--
    procedure Shoot (Preface : String; Msg : String) is
        use Calendar;

        Dur      : Day_Duration := Seconds (Clock);
        Hrs      : Natural := Natural (Dur / Duration (3600.0));
        Dur_Hrs  : Day_Duration :=
           Day_Duration (Day_Duration (Hrs) * Day_Duration (3600.0));
        Mins     : Natural := Natural ((Dur - Dur_Hrs) / Day_Duration (60.0));
        Dur_Mins : Day_Duration :=
           Day_Duration (Day_Duration (Mins) * Day_Duration (60.0));
        Secs     : Natural := Natural (Dur - Dur_Hrs - Dur_Mins);
    begin
        if Hrs < 10 then
            Text_Io.Put (Text_Io.Standard_Output, '0');
        end if;
        Inte_Io.Put (Text_Io.Standard_Output, Hrs, Width => 0);
        Text_Io.Put (Text_Io.Standard_Output, ':');
        if Mins < 10 then
            Text_Io.Put (Text_Io.Standard_Output, '0');
        end if;
        Inte_Io.Put (Text_Io.Standard_Output, Mins, Width => 0);
        Text_Io.Put (Text_Io.Standard_Output, ':');
        if Secs < 10 then
            Text_Io.Put (Text_Io.Standard_Output, '0');
        end if;
        Inte_Io.Put (Text_Io.Standard_Output, Secs, Width => 0);
        Text_Io.Put (Text_Io.Standard_Output, Preface);
        Text_Io.Put_Line (Text_Io.Standard_Output, Msg);
    end Shoot;

--
    procedure Error (Msg : String) is
    begin
        Error_Called := True;
        Shoot (" *** ", Msg);
    end Error;

--
    procedure Warning (Msg : String) is
    begin
        Shoot (" !!! ", Msg);
    end Warning;

--
    procedure Information (Msg : String) is
    begin
        Shoot (" --- ", Msg);
    end Information;

--
end Error_Report;
