with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_Fall is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_Fall - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----SEASONS  
------------------------------------------------------------------------------  

    Fall_28x32_Width  : constant := 28;  
    Fall_28x32_Height : constant := 32;  

    Fall_28x32_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#00#, 16#00#, 16#00#, 16#0E#, 16#00#, 16#00#,  
        16#00#, 16#1B#, 16#00#, 16#00#, 16#00#, 16#11#, 16#00#, 16#00#,  
        16#03#, 16#11#, 16#18#, 16#00#, 16#03#, 16#B5#, 16#B8#, 16#00#,  
        16#01#, 16#E0#, 16#F0#, 16#00#, 16#01#, 16#8A#, 16#30#, 16#00#,  
        16#00#, 16#A4#, 16#A0#, 16#00#, 16#01#, 16#80#, 16#30#, 16#00#,  
        16#03#, 16#4A#, 16#58#, 16#00#, 16#0F#, 16#24#, 16#9E#, 16#00#,  
        16#01#, 16#C0#, 16#70#, 16#00#, 16#00#, 16#4A#, 16#40#, 16#00#,  
        16#00#, 16#64#, 16#C0#, 16#00#, 16#00#, 16#44#, 16#40#, 16#00#,  
        16#00#, 16#7B#, 16#C0#, 16#00#, 16#00#, 16#EE#, 16#E0#, 16#00#,  
        16#01#, 16#84#, 16#30#, 16#00#, 16#00#, 16#04#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#32#, 16#BA#, 16#A5#, 16#20#,  
        16#4A#, 16#92#, 16#BD#, 16#A0#, 16#4A#, 16#92#, 16#BD#, 16#A0#,  
        16#7A#, 16#92#, 16#A5#, 16#60#, 16#4A#, 16#92#, 16#A5#, 16#60#,  
        16#4B#, 16#93#, 16#A5#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Fall_40x48_Width  : constant := 40;  
    Fall_40x48_Height : constant := 48;  

    Fall_40x48_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#38#, 16#00#, 16#00#, 16#00#, 16#00#, 16#28#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#28#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#C6#, 16#00#, 16#00#, 16#00#, 16#00#, 16#82#, 16#00#, 16#00#,  
        16#00#, 16#60#, 16#82#, 16#0C#, 16#00#, 16#00#, 16#60#, 16#92#,  
        16#0C#, 16#00#, 16#00#, 16#31#, 16#01#, 16#18#, 16#00#, 16#00#,  
        16#3E#, 16#00#, 16#F8#, 16#00#, 16#00#, 16#30#, 16#00#, 16#18#,  
        16#00#, 16#00#, 16#10#, 16#92#, 16#10#, 16#00#, 16#00#, 16#12#,  
        16#00#, 16#90#, 16#00#, 16#00#, 16#10#, 16#10#, 16#10#, 16#00#,  
        16#00#, 16#32#, 16#00#, 16#98#, 16#00#, 16#00#, 16#20#, 16#00#,  
        16#08#, 16#00#, 16#00#, 16#E0#, 16#28#, 16#0E#, 16#00#, 16#03#,  
        16#C4#, 16#10#, 16#47#, 16#00#, 16#00#, 16#38#, 16#82#, 16#38#,  
        16#00#, 16#00#, 16#0C#, 16#00#, 16#60#, 16#00#, 16#00#, 16#04#,  
        16#28#, 16#40#, 16#00#, 16#00#, 16#06#, 16#10#, 16#C0#, 16#00#,  
        16#00#, 16#04#, 16#00#, 16#40#, 16#00#, 16#00#, 16#04#, 16#00#,  
        16#40#, 16#00#, 16#00#, 16#04#, 16#C6#, 16#40#, 16#00#, 16#00#,  
        16#07#, 16#6D#, 16#C0#, 16#00#, 16#00#, 16#0C#, 16#10#, 16#60#,  
        16#00#, 16#00#, 16#38#, 16#10#, 16#38#, 16#00#, 16#00#, 16#00#,  
        16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#28#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#1C#, 16#8B#, 16#E8#, 16#A2#,  
        16#C8#, 16#22#, 16#88#, 16#88#, 16#B6#, 16#C8#, 16#22#, 16#88#,  
        16#88#, 16#B6#, 16#C8#, 16#22#, 16#88#, 16#88#, 16#B6#, 16#C8#,  
        16#22#, 16#88#, 16#88#, 16#AA#, 16#A8#, 16#22#, 16#88#, 16#88#,  
        16#AA#, 16#A8#, 16#3E#, 16#88#, 16#88#, 16#AA#, 16#A8#, 16#22#,  
        16#88#, 16#88#, 16#A2#, 16#A8#, 16#22#, 16#88#, 16#88#, 16#A2#,  
        16#98#, 16#22#, 16#88#, 16#88#, 16#A2#, 16#98#, 16#22#, 16#70#,  
        16#87#, 16#22#, 16#98#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Fall_56x64_Width  : constant := 56;  
    Fall_56x64_Height : constant := 64;  

    Fall_56x64_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1C#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#14#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#14#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#14#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#63#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#41#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#C1#,  
        16#81#, 16#80#, 16#00#, 16#00#, 16#00#, 16#C1#, 16#88#, 16#C1#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#63#, 16#00#, 16#63#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#7C#, 16#00#, 16#1F#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#60#, 16#00#, 16#03#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#20#, 16#49#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#24#, 16#00#, 16#12#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#,  
        16#00#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#,  
        16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#88#, 16#82#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#64#, 16#00#, 16#13#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#01#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#C0#, 16#14#, 16#01#, 16#C0#, 16#00#, 16#00#,  
        16#07#, 16#88#, 16#08#, 16#08#, 16#E0#, 16#00#, 16#00#, 16#00#,  
        16#70#, 16#80#, 16#87#, 16#00#, 16#00#, 16#00#, 16#00#, 16#18#,  
        16#00#, 16#0C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#14#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0C#, 16#08#, 16#18#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#09#, 16#00#, 16#48#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#08#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#09#, 16#E3#, 16#C8#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#0E#, 16#36#, 16#38#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#18#, 16#08#, 16#0C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#70#,  
        16#08#, 16#07#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#14#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#88#, 16#2F#, 16#F4#, 16#14#, 16#16#,  
        16#10#, 16#04#, 16#48#, 16#21#, 16#04#, 16#16#, 16#36#, 16#10#,  
        16#08#, 16#28#, 16#21#, 16#04#, 16#16#, 16#35#, 16#10#, 16#08#,  
        16#28#, 16#21#, 16#04#, 16#16#, 16#35#, 16#10#, 16#08#, 16#28#,  
        16#21#, 16#04#, 16#15#, 16#55#, 16#10#, 16#08#, 16#28#, 16#21#,  
        16#04#, 16#15#, 16#54#, 16#90#, 16#0F#, 16#E8#, 16#21#, 16#04#,  
        16#14#, 16#94#, 16#90#, 16#08#, 16#28#, 16#21#, 16#04#, 16#14#,  
        16#94#, 16#50#, 16#08#, 16#28#, 16#21#, 16#04#, 16#14#, 16#14#,  
        16#50#, 16#08#, 16#24#, 16#41#, 16#02#, 16#24#, 16#14#, 16#30#,  
        16#08#, 16#23#, 16#81#, 16#01#, 16#C4#, 16#14#, 16#30#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Fall_68x80_Width  : constant := 68;  
    Fall_68x80_Height : constant := 80;  

    Fall_68x80_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#E0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#A0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#A0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#A0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#08#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#06#, 16#06#, 16#0C#, 16#0C#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#0C#, 16#46#, 16#0C#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#18#, 16#03#,  
        16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#E0#,  
        16#00#, 16#F8#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#,  
        16#00#, 16#00#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#02#, 16#48#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#20#, 16#00#, 16#90#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#00#, 16#10#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#04#, 16#44#, 16#10#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#20#, 16#00#,  
        16#98#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#,  
        16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0E#,  
        16#00#, 16#A0#, 16#0E#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#3C#, 16#40#, 16#40#, 16#47#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#84#, 16#04#, 16#38#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#C0#, 16#00#, 16#60#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#A0#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#60#, 16#40#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#48#, 16#02#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#,  
        16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#4F#, 16#1E#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#71#, 16#B1#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#C0#, 16#40#, 16#60#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#80#, 16#40#, 16#38#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#A0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#0F#, 16#84#, 16#04#, 16#FF#, 16#90#, 16#12#, 16#02#, 16#60#,  
        16#40#, 16#10#, 16#44#, 16#04#, 16#08#, 16#10#, 16#13#, 16#06#,  
        16#50#, 16#40#, 16#20#, 16#24#, 16#04#, 16#08#, 16#10#, 16#13#,  
        16#06#, 16#50#, 16#40#, 16#20#, 16#24#, 16#04#, 16#08#, 16#10#,  
        16#13#, 16#06#, 16#48#, 16#40#, 16#20#, 16#24#, 16#04#, 16#08#,  
        16#10#, 16#12#, 16#8A#, 16#48#, 16#40#, 16#20#, 16#24#, 16#04#,  
        16#08#, 16#10#, 16#12#, 16#8A#, 16#44#, 16#40#, 16#20#, 16#24#,  
        16#04#, 16#08#, 16#10#, 16#12#, 16#8A#, 16#44#, 16#40#, 16#3F#,  
        16#E4#, 16#04#, 16#08#, 16#10#, 16#12#, 16#52#, 16#42#, 16#40#,  
        16#20#, 16#24#, 16#04#, 16#08#, 16#10#, 16#12#, 16#52#, 16#42#,  
        16#40#, 16#20#, 16#24#, 16#04#, 16#08#, 16#10#, 16#12#, 16#52#,  
        16#41#, 16#40#, 16#20#, 16#24#, 16#04#, 16#08#, 16#10#, 16#12#,  
        16#22#, 16#41#, 16#40#, 16#20#, 16#22#, 16#08#, 16#08#, 16#08#,  
        16#22#, 16#22#, 16#40#, 16#C0#, 16#20#, 16#21#, 16#F0#, 16#08#,  
        16#07#, 16#C2#, 16#22#, 16#40#, 16#C0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Fall_80x96_Width  : constant := 80;  
    Fall_80x96_Height : constant := 96;  

    Fall_80x96_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#30#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#04#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#0C#, 16#0C#, 16#18#, 16#18#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#0C#, 16#18#, 16#8C#, 16#18#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#30#, 16#06#,  
        16#30#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#,  
        16#C0#, 16#01#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#06#, 16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#04#, 16#90#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#40#, 16#01#,  
        16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#02#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#08#, 16#88#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#40#, 16#01#,  
        16#30#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#,  
        16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#1C#, 16#01#, 16#40#, 16#1C#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#78#, 16#80#, 16#80#, 16#8F#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#08#, 16#08#,  
        16#70#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#80#, 16#00#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#81#, 16#40#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#81#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#90#, 16#04#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#80#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#9E#, 16#3C#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#E3#, 16#63#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#80#, 16#80#,  
        16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#,  
        16#00#, 16#80#, 16#70#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#F0#,  
        16#40#, 16#47#, 16#FC#, 16#40#, 16#44#, 16#04#, 16#60#, 16#40#,  
        16#02#, 16#08#, 16#40#, 16#40#, 16#40#, 16#40#, 16#46#, 16#0C#,  
        16#50#, 16#40#, 16#04#, 16#04#, 16#40#, 16#40#, 16#40#, 16#40#,  
        16#46#, 16#0C#, 16#50#, 16#40#, 16#04#, 16#04#, 16#40#, 16#40#,  
        16#40#, 16#40#, 16#46#, 16#0C#, 16#48#, 16#40#, 16#04#, 16#04#,  
        16#40#, 16#40#, 16#40#, 16#40#, 16#45#, 16#14#, 16#48#, 16#40#,  
        16#04#, 16#04#, 16#40#, 16#40#, 16#40#, 16#40#, 16#45#, 16#14#,  
        16#44#, 16#40#, 16#04#, 16#04#, 16#40#, 16#40#, 16#40#, 16#40#,  
        16#45#, 16#14#, 16#44#, 16#40#, 16#07#, 16#FC#, 16#40#, 16#40#,  
        16#40#, 16#40#, 16#44#, 16#A4#, 16#42#, 16#40#, 16#04#, 16#04#,  
        16#40#, 16#40#, 16#40#, 16#40#, 16#44#, 16#A4#, 16#42#, 16#40#,  
        16#04#, 16#04#, 16#40#, 16#40#, 16#40#, 16#40#, 16#44#, 16#A4#,  
        16#41#, 16#40#, 16#04#, 16#04#, 16#40#, 16#40#, 16#40#, 16#40#,  
        16#44#, 16#44#, 16#41#, 16#40#, 16#04#, 16#04#, 16#20#, 16#80#,  
        16#40#, 16#20#, 16#84#, 16#44#, 16#40#, 16#C0#, 16#04#, 16#04#,  
        16#1F#, 16#00#, 16#40#, 16#1F#, 16#04#, 16#44#, 16#40#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

end Tile_Data_Fall;  

