with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_Mum is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_mum - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----FLOWERS  
------------------------------------------------------------------------------  

    Mum_28x32_Width  : constant := 28;  
    Mum_28x32_Height : constant := 32;  

    Mum_28x32_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#F8#, 16#20#, 16#00#,  
        16#02#, 16#04#, 16#DC#, 16#00#, 16#04#, 16#07#, 16#06#, 16#00#,  
        16#04#, 16#02#, 16#01#, 16#00#, 16#04#, 16#02#, 16#01#, 16#80#,  
        16#04#, 16#02#, 16#00#, 16#80#, 16#02#, 16#01#, 16#00#, 16#80#,  
        16#07#, 16#C1#, 16#00#, 16#80#, 16#18#, 16#63#, 16#00#, 16#80#,  
        16#30#, 16#17#, 16#87#, 16#80#, 16#20#, 16#1D#, 16#FC#, 16#40#,  
        16#20#, 16#30#, 16#40#, 16#40#, 16#20#, 16#36#, 16#40#, 16#40#,  
        16#20#, 16#32#, 16#C0#, 16#40#, 16#20#, 16#F9#, 16#80#, 16#40#,  
        16#21#, 16#8F#, 16#00#, 16#80#, 16#22#, 16#03#, 16#80#, 16#80#,  
        16#12#, 16#01#, 16#43#, 16#00#, 16#0C#, 16#03#, 16#3C#, 16#80#,  
        16#04#, 16#01#, 16#00#, 16#80#, 16#06#, 16#03#, 16#00#, 16#80#,  
        16#03#, 16#87#, 16#81#, 16#00#, 16#00#, 16#F8#, 16#FE#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#29#, 16#48#, 16#00#,  
        16#01#, 16#E9#, 16#78#, 16#00#, 16#01#, 16#29#, 16#48#, 16#00#,  
        16#01#, 16#29#, 16#48#, 16#00#, 16#01#, 16#29#, 16#48#, 16#00#,  
        16#01#, 16#26#, 16#48#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Mum_40x48_Width  : constant := 40;  
    Mum_40x48_Height : constant := 48;  

    Mum_40x48_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1C#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#27#, 16#C0#, 16#60#, 16#00#, 16#00#,  
        16#40#, 16#33#, 16#9C#, 16#00#, 16#00#, 16#40#, 16#1C#, 16#03#,  
        16#00#, 16#00#, 16#C0#, 16#14#, 16#00#, 16#80#, 16#01#, 16#00#,  
        16#08#, 16#00#, 16#80#, 16#01#, 16#00#, 16#08#, 16#00#, 16#60#,  
        16#00#, 16#80#, 16#08#, 16#00#, 16#20#, 16#00#, 16#80#, 16#08#,  
        16#00#, 16#10#, 16#00#, 16#80#, 16#04#, 16#00#, 16#10#, 16#00#,  
        16#80#, 16#04#, 16#00#, 16#10#, 16#00#, 16#78#, 16#04#, 16#00#,  
        16#10#, 16#03#, 16#C7#, 16#84#, 16#00#, 16#10#, 16#04#, 16#00#,  
        16#84#, 16#00#, 16#20#, 16#08#, 16#00#, 16#44#, 16#00#, 16#50#,  
        16#10#, 16#00#, 16#FE#, 16#03#, 16#90#, 16#10#, 16#00#, 16#FF#,  
        16#FC#, 16#08#, 16#10#, 16#01#, 16#81#, 16#C0#, 16#08#, 16#10#,  
        16#01#, 16#A9#, 16#80#, 16#08#, 16#10#, 16#03#, 16#85#, 16#80#,  
        16#08#, 16#10#, 16#01#, 16#A9#, 16#00#, 16#08#, 16#10#, 16#07#,  
        16#C3#, 16#00#, 16#08#, 16#10#, 16#18#, 16#FE#, 16#00#, 16#08#,  
        16#10#, 16#30#, 16#7E#, 16#00#, 16#10#, 16#10#, 16#40#, 16#0C#,  
        16#00#, 16#10#, 16#08#, 16#80#, 16#06#, 16#00#, 16#30#, 16#05#,  
        16#80#, 16#05#, 16#00#, 16#30#, 16#07#, 16#00#, 16#04#, 16#81#,  
        16#D0#, 16#01#, 16#00#, 16#04#, 16#7E#, 16#10#, 16#01#, 16#00#,  
        16#04#, 16#00#, 16#10#, 16#01#, 16#00#, 16#04#, 16#00#, 16#10#,  
        16#01#, 16#00#, 16#04#, 16#00#, 16#10#, 16#01#, 16#00#, 16#06#,  
        16#00#, 16#20#, 16#00#, 16#80#, 16#09#, 16#00#, 16#20#, 16#00#,  
        16#C0#, 16#08#, 16#C0#, 16#40#, 16#00#, 16#30#, 16#10#, 16#39#,  
        16#80#, 16#00#, 16#0F#, 16#E0#, 16#06#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#A2#, 16#88#, 16#00#,  
        16#00#, 16#0D#, 16#A2#, 16#D8#, 16#00#, 16#00#, 16#0D#, 16#A2#,  
        16#D8#, 16#00#, 16#00#, 16#0D#, 16#A2#, 16#D8#, 16#00#, 16#00#,  
        16#0A#, 16#A2#, 16#A8#, 16#00#, 16#00#, 16#0A#, 16#A2#, 16#A8#,  
        16#00#, 16#00#, 16#0A#, 16#A2#, 16#A8#, 16#00#, 16#00#, 16#08#,  
        16#9C#, 16#88#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Mum_56x64_Width  : constant := 56;  
    Mum_56x64_Height : constant := 64;  

    Mum_56x64_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1C#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#27#, 16#C0#, 16#60#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#33#, 16#9C#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#40#, 16#1C#, 16#03#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#C0#, 16#14#, 16#00#, 16#80#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#08#, 16#00#, 16#80#, 16#00#, 16#00#, 16#01#,  
        16#00#, 16#08#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#, 16#80#,  
        16#08#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#80#, 16#08#,  
        16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#80#, 16#04#, 16#00#,  
        16#10#, 16#00#, 16#00#, 16#00#, 16#80#, 16#04#, 16#00#, 16#10#,  
        16#00#, 16#00#, 16#00#, 16#78#, 16#04#, 16#00#, 16#10#, 16#00#,  
        16#00#, 16#03#, 16#C7#, 16#84#, 16#00#, 16#10#, 16#00#, 16#00#,  
        16#04#, 16#00#, 16#84#, 16#00#, 16#20#, 16#00#, 16#00#, 16#08#,  
        16#00#, 16#44#, 16#00#, 16#50#, 16#00#, 16#00#, 16#10#, 16#00#,  
        16#FE#, 16#03#, 16#90#, 16#00#, 16#00#, 16#10#, 16#00#, 16#FF#,  
        16#FC#, 16#08#, 16#00#, 16#00#, 16#10#, 16#01#, 16#81#, 16#C0#,  
        16#08#, 16#00#, 16#00#, 16#10#, 16#01#, 16#A9#, 16#80#, 16#08#,  
        16#00#, 16#00#, 16#10#, 16#03#, 16#85#, 16#80#, 16#08#, 16#00#,  
        16#00#, 16#10#, 16#01#, 16#A9#, 16#00#, 16#08#, 16#00#, 16#00#,  
        16#10#, 16#07#, 16#C3#, 16#00#, 16#08#, 16#00#, 16#00#, 16#10#,  
        16#18#, 16#FE#, 16#00#, 16#08#, 16#00#, 16#00#, 16#10#, 16#30#,  
        16#7E#, 16#00#, 16#10#, 16#00#, 16#00#, 16#10#, 16#40#, 16#0C#,  
        16#00#, 16#10#, 16#00#, 16#00#, 16#08#, 16#80#, 16#06#, 16#00#,  
        16#30#, 16#00#, 16#00#, 16#05#, 16#80#, 16#05#, 16#00#, 16#30#,  
        16#00#, 16#00#, 16#07#, 16#00#, 16#04#, 16#81#, 16#D0#, 16#00#,  
        16#00#, 16#01#, 16#00#, 16#04#, 16#7E#, 16#10#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#04#, 16#00#, 16#10#, 16#00#, 16#00#, 16#01#,  
        16#00#, 16#04#, 16#00#, 16#10#, 16#00#, 16#00#, 16#01#, 16#00#,  
        16#04#, 16#00#, 16#10#, 16#00#, 16#00#, 16#01#, 16#00#, 16#06#,  
        16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#80#, 16#09#, 16#00#,  
        16#20#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#08#, 16#C0#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#30#, 16#10#, 16#39#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#0C#, 16#60#, 16#06#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#82#, 16#82#, 16#82#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#C6#, 16#82#, 16#C6#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#C6#, 16#82#, 16#C6#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#C6#, 16#82#, 16#C6#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#AA#, 16#82#, 16#AA#, 16#00#, 16#00#, 16#00#, 16#00#, 16#AA#,  
        16#82#, 16#AA#, 16#00#, 16#00#, 16#00#, 16#00#, 16#AA#, 16#82#,  
        16#AA#, 16#00#, 16#00#, 16#00#, 16#00#, 16#92#, 16#82#, 16#92#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#92#, 16#82#, 16#92#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#82#, 16#44#, 16#82#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#82#, 16#38#, 16#82#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Mum_68x80_Width  : constant := 68;  
    Mum_68x80_Height : constant := 80;  

    Mum_68x80_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#70#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#9F#, 16#01#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#CE#,  
        16#70#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#,  
        16#70#, 16#0C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#,  
        16#00#, 16#50#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#04#, 16#00#, 16#20#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#00#, 16#20#, 16#01#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#02#, 16#00#, 16#20#, 16#00#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#20#, 16#00#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#10#, 16#00#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#10#,  
        16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#E0#,  
        16#10#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#,  
        16#1E#, 16#10#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#10#, 16#02#, 16#10#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#20#, 16#01#, 16#10#, 16#01#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#40#, 16#03#, 16#F8#, 16#0E#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#40#, 16#03#, 16#FF#, 16#F0#, 16#20#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#06#, 16#07#, 16#00#,  
        16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#06#, 16#A6#,  
        16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#0E#,  
        16#16#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#,  
        16#06#, 16#A4#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#40#, 16#1F#, 16#0C#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#40#, 16#63#, 16#F8#, 16#00#, 16#20#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#40#, 16#C1#, 16#F8#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#41#, 16#00#, 16#30#, 16#00#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#22#, 16#00#, 16#18#, 16#00#,  
        16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#16#, 16#00#, 16#14#,  
        16#00#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1C#, 16#00#,  
        16#12#, 16#07#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#,  
        16#00#, 16#11#, 16#F8#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#04#, 16#00#, 16#10#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#00#, 16#10#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#04#, 16#00#, 16#10#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#18#, 16#00#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#24#, 16#00#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#00#, 16#23#,  
        16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C0#,  
        16#40#, 16#E6#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#31#, 16#80#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#0E#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#40#, 16#24#, 16#02#, 16#40#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#60#, 16#64#, 16#02#, 16#60#, 16#60#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#60#, 16#64#, 16#02#, 16#60#,  
        16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#60#, 16#64#, 16#02#,  
        16#60#, 16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#50#, 16#A4#,  
        16#02#, 16#50#, 16#A0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#50#,  
        16#A4#, 16#02#, 16#50#, 16#A0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#50#, 16#A4#, 16#02#, 16#50#, 16#A0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#49#, 16#24#, 16#02#, 16#49#, 16#20#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#49#, 16#24#, 16#02#, 16#49#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#49#, 16#24#, 16#02#, 16#49#, 16#20#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#46#, 16#24#, 16#02#, 16#46#,  
        16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#46#, 16#22#, 16#04#,  
        16#46#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#46#, 16#21#,  
        16#F8#, 16#46#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Mum_80x96_Width  : constant := 80;  
    Mum_80x96_Height : constant := 96;  

    Mum_80x96_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#7C#, 16#06#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#03#, 16#39#,  
        16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#,  
        16#01#, 16#C0#, 16#30#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#0C#, 16#01#, 16#40#, 16#08#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#80#, 16#08#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#80#,  
        16#06#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#,  
        16#00#, 16#80#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#08#, 16#00#, 16#80#, 16#01#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#40#, 16#01#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#40#,  
        16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#,  
        16#80#, 16#40#, 16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#3C#, 16#78#, 16#40#, 16#01#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#40#, 16#08#, 16#40#, 16#02#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#, 16#04#, 16#40#,  
        16#05#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#,  
        16#0F#, 16#E0#, 16#39#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#0F#, 16#FF#, 16#C0#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#00#, 16#18#, 16#1C#, 16#00#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#1A#, 16#98#,  
        16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#,  
        16#38#, 16#58#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#1A#, 16#90#, 16#00#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#00#, 16#7C#, 16#30#, 16#00#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#01#, 16#8F#, 16#E0#,  
        16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#03#,  
        16#07#, 16#E0#, 16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#04#, 16#00#, 16#C0#, 16#01#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#88#, 16#00#, 16#60#, 16#03#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#58#, 16#00#, 16#50#,  
        16#03#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#70#,  
        16#00#, 16#48#, 16#1D#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#10#, 16#00#, 16#47#, 16#E1#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#40#, 16#01#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#40#,  
        16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#,  
        16#00#, 16#40#, 16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#10#, 16#00#, 16#60#, 16#02#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#90#, 16#02#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0C#, 16#00#, 16#8C#,  
        16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#,  
        16#01#, 16#03#, 16#98#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#C6#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#38#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#80#, 16#88#, 16#08#, 16#80#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#C1#, 16#88#, 16#08#, 16#C1#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C1#, 16#88#, 16#08#,  
        16#C1#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C1#,  
        16#88#, 16#08#, 16#C1#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#A2#, 16#88#, 16#08#, 16#A2#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#A2#, 16#88#, 16#08#, 16#A2#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#A2#, 16#88#, 16#08#,  
        16#A2#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#94#,  
        16#88#, 16#08#, 16#94#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#94#, 16#88#, 16#08#, 16#94#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#94#, 16#88#, 16#08#, 16#94#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#88#, 16#88#, 16#08#,  
        16#88#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#88#,  
        16#84#, 16#10#, 16#88#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#88#, 16#83#, 16#E0#, 16#88#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

end Tile_Data_Mum;  

