with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_Numbers is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_numbers - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----NUMBERS  
------------------------------------------------------------------------------  

    One_10x12_Width  : constant := 10;  
    One_10x12_Height : constant := 12;  

    One_10x12_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#0C#, 16#00#, 16#1C#, 16#00#, 16#3C#, 16#00#,  
        16#0C#, 16#00#, 16#0C#, 16#00#, 16#0C#, 16#00#, 16#0C#, 16#00#,  
        16#0C#, 16#00#, 16#3F#, 16#00#, 16#3F#, 16#00#, 16#00#, 16#00#);  

    One_14x16_Width  : constant := 14;  
    One_14x16_Height : constant := 16;  

    One_14x16_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#03#, 16#00#, 16#07#, 16#00#, 16#0F#, 16#00#,  
        16#1F#, 16#00#, 16#03#, 16#00#, 16#03#, 16#00#, 16#03#, 16#00#,  
        16#03#, 16#00#, 16#03#, 16#00#, 16#03#, 16#00#, 16#03#, 16#00#,  
        16#03#, 16#00#, 16#1F#, 16#E0#, 16#1F#, 16#E0#, 16#00#, 16#00#);  

    One_17x20_Width  : constant := 17;  
    One_17x20_Height : constant := 20;  

    One_17x20_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#01#, 16#80#, 16#00#, 16#03#, 16#80#,  
        16#00#, 16#07#, 16#80#, 16#00#, 16#0F#, 16#80#, 16#00#, 16#01#,  
        16#80#, 16#00#, 16#01#, 16#80#, 16#00#, 16#01#, 16#80#, 16#00#,  
        16#01#, 16#80#, 16#00#, 16#01#, 16#80#, 16#00#, 16#01#, 16#80#,  
        16#00#, 16#01#, 16#80#, 16#00#, 16#01#, 16#80#, 16#00#, 16#01#,  
        16#80#, 16#00#, 16#01#, 16#80#, 16#00#, 16#01#, 16#80#, 16#00#,  
        16#01#, 16#80#, 16#00#, 16#0F#, 16#F0#, 16#00#, 16#0F#, 16#F0#,  
        16#00#, 16#00#, 16#00#, 16#00#);  

    One_20x24_Width  : constant := 20;  
    One_20x24_Height : constant := 24;  

    One_20x24_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#E0#,  
        16#00#, 16#01#, 16#E0#, 16#00#, 16#03#, 16#E0#, 16#00#, 16#03#,  
        16#60#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#60#, 16#00#,  
        16#00#, 16#60#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#60#,  
        16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#,  
        16#60#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#60#, 16#00#,  
        16#00#, 16#60#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#60#,  
        16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#60#, 16#00#, 16#07#,  
        16#FE#, 16#00#, 16#07#, 16#FE#, 16#00#, 16#00#, 16#00#, 16#00#);  

    One_7x8_Width  : constant := 7;  
    One_7x8_Height : constant := 8;  

    One_7x8_Bits : constant U_Char_Array :=  
       (16#00#, 16#18#, 16#38#, 16#18#, 16#18#, 16#18#, 16#3C#, 16#00#);  

    Two_10x12_Width  : constant := 10;  
    Two_10x12_Height : constant := 12;  

    Two_10x12_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#3F#, 16#00#, 16#7F#, 16#80#, 16#61#, 16#80#,  
        16#63#, 16#80#, 16#07#, 16#00#, 16#0E#, 16#00#, 16#1C#, 16#00#,  
        16#38#, 16#00#, 16#7F#, 16#80#, 16#7F#, 16#80#, 16#00#, 16#00#);  

    Two_14x16_Width  : constant := 14;  
    Two_14x16_Height : constant := 16;  

    Two_14x16_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#0F#, 16#C0#, 16#1F#, 16#E0#, 16#38#, 16#70#,  
        16#30#, 16#30#, 16#00#, 16#30#, 16#00#, 16#70#, 16#00#, 16#E0#,  
        16#01#, 16#C0#, 16#03#, 16#80#, 16#07#, 16#00#, 16#0E#, 16#00#,  
        16#1C#, 16#00#, 16#3F#, 16#F0#, 16#3F#, 16#F0#, 16#00#, 16#00#);  

    Two_17x20_Width  : constant := 17;  
    Two_17x20_Height : constant := 20;  

    Two_17x20_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#0F#, 16#FC#, 16#00#, 16#1F#, 16#FE#,  
        16#00#, 16#38#, 16#07#, 16#00#, 16#30#, 16#03#, 16#00#, 16#30#,  
        16#03#, 16#00#, 16#00#, 16#07#, 16#00#, 16#00#, 16#0E#, 16#00#,  
        16#00#, 16#1C#, 16#00#, 16#00#, 16#38#, 16#00#, 16#00#, 16#70#,  
        16#00#, 16#00#, 16#E0#, 16#00#, 16#01#, 16#C0#, 16#00#, 16#03#,  
        16#80#, 16#00#, 16#07#, 16#00#, 16#00#, 16#0E#, 16#00#, 16#00#,  
        16#1C#, 16#00#, 16#00#, 16#3F#, 16#FF#, 16#00#, 16#3F#, 16#FF#,  
        16#00#, 16#00#, 16#00#, 16#00#);  

    Two_20x24_Width  : constant := 20;  
    Two_20x24_Height : constant := 24;  

    Two_20x24_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#0F#, 16#FF#, 16#00#, 16#1F#, 16#FF#,  
        16#80#, 16#3C#, 16#01#, 16#C0#, 16#38#, 16#00#, 16#C0#, 16#30#,  
        16#00#, 16#C0#, 16#00#, 16#00#, 16#C0#, 16#00#, 16#01#, 16#C0#,  
        16#00#, 16#03#, 16#80#, 16#00#, 16#07#, 16#00#, 16#00#, 16#0E#,  
        16#00#, 16#00#, 16#1C#, 16#00#, 16#00#, 16#38#, 16#00#, 16#00#,  
        16#70#, 16#00#, 16#00#, 16#E0#, 16#00#, 16#01#, 16#C0#, 16#00#,  
        16#03#, 16#80#, 16#00#, 16#07#, 16#00#, 16#00#, 16#0E#, 16#00#,  
        16#00#, 16#1C#, 16#00#, 16#00#, 16#38#, 16#00#, 16#00#, 16#3F#,  
        16#FF#, 16#C0#, 16#3F#, 16#FF#, 16#C0#, 16#00#, 16#00#, 16#00#);  

    Two_7x8_Width  : constant := 7;  
    Two_7x8_Height : constant := 8;  

    Two_7x8_Bits : constant U_Char_Array :=  
       (16#00#, 16#38#, 16#6C#, 16#0C#, 16#18#, 16#30#, 16#7C#, 16#00#);  

    Three_10x12_Width  : constant := 10;  
    Three_10x12_Height : constant := 12;  

    Three_10x12_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#3E#, 16#00#, 16#7F#, 16#00#, 16#63#, 16#00#,  
        16#03#, 16#00#, 16#1E#, 16#00#, 16#1E#, 16#00#, 16#03#, 16#00#,  
        16#63#, 16#00#, 16#7F#, 16#00#, 16#3E#, 16#00#, 16#00#, 16#00#);  

    Three_14x16_Width  : constant := 14;  
    Three_14x16_Height : constant := 16;  

    Three_14x16_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#0F#, 16#E0#, 16#1F#, 16#F0#, 16#30#, 16#18#,  
        16#30#, 16#18#, 16#00#, 16#18#, 16#00#, 16#30#, 16#03#, 16#E0#,  
        16#03#, 16#E0#, 16#00#, 16#30#, 16#00#, 16#18#, 16#30#, 16#18#,  
        16#30#, 16#18#, 16#1F#, 16#F0#, 16#0F#, 16#E0#, 16#00#, 16#00#);  

    Three_17x20_Width  : constant := 17;  
    Three_17x20_Height : constant := 20;  

    Three_17x20_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#0F#, 16#FC#, 16#00#, 16#1F#, 16#FE#,  
        16#00#, 16#38#, 16#07#, 16#00#, 16#30#, 16#03#, 16#00#, 16#30#,  
        16#03#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#, 16#03#, 16#00#,  
        16#00#, 16#06#, 16#00#, 16#01#, 16#FC#, 16#00#, 16#01#, 16#FC#,  
        16#00#, 16#00#, 16#06#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#,  
        16#03#, 16#00#, 16#30#, 16#03#, 16#00#, 16#30#, 16#03#, 16#00#,  
        16#38#, 16#07#, 16#00#, 16#1F#, 16#FE#, 16#00#, 16#0F#, 16#FC#,  
        16#00#, 16#00#, 16#00#, 16#00#);  

    Three_20x24_Width  : constant := 20;  
    Three_20x24_Height : constant := 24;  

    Three_20x24_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#0F#, 16#FF#, 16#00#, 16#1F#, 16#FF#,  
        16#80#, 16#38#, 16#01#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#30#,  
        16#00#, 16#C0#, 16#00#, 16#00#, 16#C0#, 16#00#, 16#00#, 16#C0#,  
        16#00#, 16#00#, 16#C0#, 16#00#, 16#00#, 16#C0#, 16#00#, 16#01#,  
        16#80#, 16#00#, 16#FF#, 16#00#, 16#00#, 16#FF#, 16#00#, 16#00#,  
        16#01#, 16#80#, 16#00#, 16#00#, 16#C0#, 16#00#, 16#00#, 16#C0#,  
        16#30#, 16#00#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#30#, 16#00#,  
        16#C0#, 16#30#, 16#00#, 16#C0#, 16#38#, 16#01#, 16#C0#, 16#1F#,  
        16#FF#, 16#80#, 16#0F#, 16#FF#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Three_7x8_Width  : constant := 7;  
    Three_7x8_Height : constant := 8;  

    Three_7x8_Bits : constant U_Char_Array :=  
       (16#00#, 16#38#, 16#6C#, 16#18#, 16#18#, 16#6C#, 16#38#, 16#00#);  

    Four_10x12_Width  : constant := 10;  
    Four_10x12_Height : constant := 12;  

    Four_10x12_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#03#, 16#00#, 16#07#, 16#00#, 16#0F#, 16#00#,  
        16#1F#, 16#00#, 16#3B#, 16#00#, 16#7F#, 16#80#, 16#7F#, 16#80#,  
        16#03#, 16#00#, 16#03#, 16#00#, 16#03#, 16#00#, 16#00#, 16#00#);  

    Four_14x16_Width  : constant := 14;  
    Four_14x16_Height : constant := 16;  

    Four_14x16_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#60#, 16#00#, 16#E0#, 16#01#, 16#E0#,  
        16#03#, 16#E0#, 16#07#, 16#60#, 16#0E#, 16#60#, 16#1C#, 16#60#,  
        16#38#, 16#60#, 16#3F#, 16#F8#, 16#3F#, 16#F8#, 16#00#, 16#60#,  
        16#00#, 16#60#, 16#00#, 16#60#, 16#00#, 16#60#, 16#00#, 16#00#);  

    Four_17x20_Width  : constant := 17;  
    Four_17x20_Height : constant := 20;  

    Four_17x20_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#0C#, 16#00#, 16#00#, 16#1C#,  
        16#00#, 16#00#, 16#3C#, 16#00#, 16#00#, 16#7C#, 16#00#, 16#00#,  
        16#EC#, 16#00#, 16#01#, 16#CC#, 16#00#, 16#03#, 16#8C#, 16#00#,  
        16#07#, 16#0C#, 16#00#, 16#0E#, 16#0C#, 16#00#, 16#1C#, 16#0C#,  
        16#00#, 16#3F#, 16#FF#, 16#00#, 16#3F#, 16#FF#, 16#00#, 16#00#,  
        16#0C#, 16#00#, 16#00#, 16#0C#, 16#00#, 16#00#, 16#0C#, 16#00#,  
        16#00#, 16#0C#, 16#00#, 16#00#, 16#0C#, 16#00#, 16#00#, 16#0C#,  
        16#00#, 16#00#, 16#00#, 16#00#);  

    Four_20x24_Width  : constant := 20;  
    Four_20x24_Height : constant := 24;  

    Four_20x24_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#, 16#07#,  
        16#00#, 16#00#, 16#0F#, 16#00#, 16#00#, 16#1F#, 16#00#, 16#00#,  
        16#3B#, 16#00#, 16#00#, 16#73#, 16#00#, 16#00#, 16#E3#, 16#00#,  
        16#01#, 16#C3#, 16#00#, 16#03#, 16#83#, 16#00#, 16#07#, 16#03#,  
        16#00#, 16#0E#, 16#03#, 16#00#, 16#1C#, 16#03#, 16#00#, 16#38#,  
        16#03#, 16#00#, 16#3F#, 16#FF#, 16#C0#, 16#3F#, 16#FF#, 16#C0#,  
        16#00#, 16#03#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#, 16#03#,  
        16#00#, 16#00#, 16#03#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#,  
        16#03#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Four_7x8_Width  : constant := 7;  
    Four_7x8_Height : constant := 8;  

    Four_7x8_Bits : constant U_Char_Array :=  
       (16#00#, 16#18#, 16#38#, 16#68#, 16#7C#, 16#18#, 16#18#, 16#00#);  

    Five_10x12_Width  : constant := 10;  
    Five_10x12_Height : constant := 12;  

    Five_10x12_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#7F#, 16#80#, 16#7F#, 16#80#, 16#60#, 16#00#,  
        16#60#, 16#00#, 16#7F#, 16#00#, 16#3F#, 16#80#, 16#01#, 16#80#,  
        16#61#, 16#80#, 16#7F#, 16#80#, 16#3F#, 16#00#, 16#00#, 16#00#);  

    Five_14x16_Width  : constant := 14;  
    Five_14x16_Height : constant := 16;  

    Five_14x16_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#3F#, 16#F8#, 16#3F#, 16#F8#, 16#30#, 16#00#,  
        16#30#, 16#00#, 16#30#, 16#00#, 16#30#, 16#00#, 16#3F#, 16#E0#,  
        16#1F#, 16#F0#, 16#00#, 16#38#, 16#00#, 16#18#, 16#30#, 16#18#,  
        16#38#, 16#38#, 16#1F#, 16#F0#, 16#0F#, 16#E0#, 16#00#, 16#00#);  

    Five_17x20_Width  : constant := 17;  
    Five_17x20_Height : constant := 20;  

    Five_17x20_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#3F#, 16#FF#, 16#00#, 16#3F#, 16#FF#,  
        16#00#, 16#30#, 16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#30#,  
        16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#30#, 16#00#, 16#00#,  
        16#30#, 16#00#, 16#00#, 16#3F#, 16#FC#, 16#00#, 16#1F#, 16#FE#,  
        16#00#, 16#00#, 16#06#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#,  
        16#03#, 16#00#, 16#30#, 16#03#, 16#00#, 16#30#, 16#03#, 16#00#,  
        16#38#, 16#07#, 16#00#, 16#1F#, 16#FE#, 16#00#, 16#0F#, 16#FC#,  
        16#00#, 16#00#, 16#00#, 16#00#);  

    Five_20x24_Width  : constant := 20;  
    Five_20x24_Height : constant := 24;  

    Five_20x24_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#3F#, 16#FF#, 16#C0#, 16#3F#, 16#FF#,  
        16#C0#, 16#30#, 16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#30#,  
        16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#30#, 16#00#, 16#00#,  
        16#30#, 16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#38#, 16#00#,  
        16#00#, 16#1F#, 16#FF#, 16#00#, 16#0F#, 16#FF#, 16#80#, 16#00#,  
        16#01#, 16#C0#, 16#00#, 16#00#, 16#C0#, 16#00#, 16#00#, 16#C0#,  
        16#00#, 16#00#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#30#, 16#00#,  
        16#C0#, 16#30#, 16#00#, 16#C0#, 16#38#, 16#01#, 16#C0#, 16#1F#,  
        16#FF#, 16#80#, 16#0F#, 16#FF#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Five_7x8_Width  : constant := 7;  
    Five_7x8_Height : constant := 8;  

    Five_7x8_Bits : constant U_Char_Array :=  
       (16#00#, 16#7C#, 16#60#, 16#60#, 16#18#, 16#4C#, 16#38#, 16#00#);  

    Six_10x12_Width  : constant := 10;  
    Six_10x12_Height : constant := 12;  

    Six_10x12_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#3F#, 16#00#, 16#7F#, 16#80#, 16#61#, 16#80#,  
        16#60#, 16#00#, 16#7F#, 16#00#, 16#7F#, 16#80#, 16#61#, 16#80#,  
        16#61#, 16#80#, 16#7F#, 16#80#, 16#3F#, 16#00#, 16#00#, 16#00#);  

    Six_14x16_Width  : constant := 14;  
    Six_14x16_Height : constant := 16;  

    Six_14x16_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#0F#, 16#E0#, 16#1F#, 16#F0#, 16#38#, 16#38#,  
        16#30#, 16#00#, 16#30#, 16#00#, 16#30#, 16#00#, 16#37#, 16#E0#,  
        16#3F#, 16#F0#, 16#38#, 16#38#, 16#30#, 16#18#, 16#30#, 16#18#,  
        16#38#, 16#38#, 16#1F#, 16#F0#, 16#0F#, 16#E0#, 16#00#, 16#00#);  

    Six_17x20_Width  : constant := 17;  
    Six_17x20_Height : constant := 20;  

    Six_17x20_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#0F#, 16#FC#, 16#00#, 16#1F#, 16#FE#,  
        16#00#, 16#38#, 16#03#, 16#00#, 16#30#, 16#03#, 16#00#, 16#30#,  
        16#03#, 16#00#, 16#30#, 16#00#, 16#00#, 16#30#, 16#00#, 16#00#,  
        16#30#, 16#00#, 16#00#, 16#37#, 16#FC#, 16#00#, 16#3F#, 16#FE#,  
        16#00#, 16#38#, 16#06#, 16#00#, 16#30#, 16#03#, 16#00#, 16#30#,  
        16#03#, 16#00#, 16#30#, 16#03#, 16#00#, 16#30#, 16#03#, 16#00#,  
        16#38#, 16#07#, 16#00#, 16#1F#, 16#FE#, 16#00#, 16#0F#, 16#FC#,  
        16#00#, 16#00#, 16#00#, 16#00#);  

    Six_20x24_Width  : constant := 20;  
    Six_20x24_Height : constant := 24;  

    Six_20x24_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#0F#, 16#FF#, 16#00#, 16#1F#, 16#FF#,  
        16#80#, 16#38#, 16#01#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#30#,  
        16#00#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#30#, 16#00#, 16#00#,  
        16#30#, 16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#30#, 16#00#,  
        16#00#, 16#37#, 16#FF#, 16#00#, 16#3F#, 16#FF#, 16#80#, 16#38#,  
        16#01#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#30#, 16#00#, 16#C0#,  
        16#30#, 16#00#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#30#, 16#00#,  
        16#C0#, 16#30#, 16#00#, 16#C0#, 16#38#, 16#01#, 16#C0#, 16#1F#,  
        16#FF#, 16#80#, 16#0F#, 16#FF#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Six_7x8_Width  : constant := 7;  
    Six_7x8_Height : constant := 8;  

    Six_7x8_Bits : constant U_Char_Array :=  
       (16#00#, 16#38#, 16#6C#, 16#40#, 16#78#, 16#6C#, 16#38#, 16#00#);  

    Seven_10x12_Width  : constant := 10;  
    Seven_10x12_Height : constant := 12;  

    Seven_10x12_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#7F#, 16#80#, 16#7F#, 16#80#, 16#01#, 16#80#,  
        16#03#, 16#80#, 16#07#, 16#00#, 16#0E#, 16#00#, 16#1C#, 16#00#,  
        16#38#, 16#00#, 16#70#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#);  

    Seven_14x16_Width  : constant := 14;  
    Seven_14x16_Height : constant := 16;  

    Seven_14x16_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#3F#, 16#F8#, 16#3F#, 16#F8#, 16#00#, 16#18#,  
        16#00#, 16#18#, 16#00#, 16#38#, 16#00#, 16#70#, 16#00#, 16#E0#,  
        16#01#, 16#C0#, 16#03#, 16#80#, 16#07#, 16#00#, 16#0E#, 16#00#,  
        16#1C#, 16#00#, 16#38#, 16#00#, 16#30#, 16#00#, 16#00#, 16#00#);  

    Seven_17x20_Width  : constant := 17;  
    Seven_17x20_Height : constant := 20;  

    Seven_17x20_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#3F#, 16#FF#, 16#00#, 16#3F#, 16#FF#,  
        16#00#, 16#00#, 16#03#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#,  
        16#03#, 16#00#, 16#00#, 16#07#, 16#00#, 16#00#, 16#0E#, 16#00#,  
        16#00#, 16#1C#, 16#00#, 16#00#, 16#38#, 16#00#, 16#00#, 16#70#,  
        16#00#, 16#00#, 16#E0#, 16#00#, 16#01#, 16#C0#, 16#00#, 16#03#,  
        16#80#, 16#00#, 16#07#, 16#00#, 16#00#, 16#0E#, 16#00#, 16#00#,  
        16#1C#, 16#00#, 16#00#, 16#38#, 16#00#, 16#00#, 16#30#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#);  

    Seven_20x24_Width  : constant := 20;  
    Seven_20x24_Height : constant := 24;  

    Seven_20x24_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#3F#, 16#FF#, 16#C0#, 16#3F#, 16#FF#,  
        16#C0#, 16#00#, 16#00#, 16#C0#, 16#00#, 16#00#, 16#C0#, 16#00#,  
        16#00#, 16#C0#, 16#00#, 16#00#, 16#C0#, 16#00#, 16#00#, 16#C0#,  
        16#00#, 16#01#, 16#C0#, 16#00#, 16#03#, 16#80#, 16#00#, 16#07#,  
        16#00#, 16#00#, 16#0E#, 16#00#, 16#00#, 16#1C#, 16#00#, 16#00#,  
        16#38#, 16#00#, 16#00#, 16#70#, 16#00#, 16#00#, 16#E0#, 16#00#,  
        16#01#, 16#C0#, 16#00#, 16#03#, 16#80#, 16#00#, 16#07#, 16#00#,  
        16#00#, 16#0E#, 16#00#, 16#00#, 16#1C#, 16#00#, 16#00#, 16#38#,  
        16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Seven_7x8_Width  : constant := 7;  
    Seven_7x8_Height : constant := 8;  

    Seven_7x8_Bits : constant U_Char_Array :=  
       (16#00#, 16#7C#, 16#7C#, 16#1C#, 16#38#, 16#70#, 16#60#, 16#00#);  

    Eight_10x12_Width  : constant := 10;  
    Eight_10x12_Height : constant := 12;  

    Eight_10x12_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#3F#, 16#00#, 16#7F#, 16#80#, 16#61#, 16#80#,  
        16#61#, 16#80#, 16#3F#, 16#00#, 16#3F#, 16#00#, 16#61#, 16#80#,  
        16#61#, 16#80#, 16#7F#, 16#80#, 16#3F#, 16#00#, 16#00#, 16#00#);  

    Eight_14x16_Width  : constant := 14;  
    Eight_14x16_Height : constant := 16;  

    Eight_14x16_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#0F#, 16#E0#, 16#1F#, 16#F0#, 16#38#, 16#38#,  
        16#30#, 16#18#, 16#30#, 16#18#, 16#18#, 16#30#, 16#0F#, 16#E0#,  
        16#0F#, 16#E0#, 16#18#, 16#30#, 16#30#, 16#18#, 16#30#, 16#18#,  
        16#38#, 16#38#, 16#1F#, 16#F0#, 16#0F#, 16#E0#, 16#00#, 16#00#);  

    Eight_17x20_Width  : constant := 17;  
    Eight_17x20_Height : constant := 20;  

    Eight_17x20_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#0F#, 16#FC#, 16#00#, 16#1F#, 16#FE#,  
        16#00#, 16#38#, 16#07#, 16#00#, 16#30#, 16#03#, 16#00#, 16#30#,  
        16#03#, 16#00#, 16#30#, 16#03#, 16#00#, 16#38#, 16#07#, 16#00#,  
        16#18#, 16#06#, 16#00#, 16#0F#, 16#FC#, 16#00#, 16#0F#, 16#FC#,  
        16#00#, 16#18#, 16#06#, 16#00#, 16#38#, 16#07#, 16#00#, 16#30#,  
        16#03#, 16#00#, 16#30#, 16#03#, 16#00#, 16#30#, 16#03#, 16#00#,  
        16#38#, 16#07#, 16#00#, 16#1F#, 16#FE#, 16#00#, 16#0F#, 16#FC#,  
        16#00#, 16#00#, 16#00#, 16#00#);  

    Eight_20x24_Width  : constant := 20;  
    Eight_20x24_Height : constant := 24;  

    Eight_20x24_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#0F#, 16#FF#, 16#00#, 16#1F#, 16#FF#,  
        16#80#, 16#38#, 16#01#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#30#,  
        16#00#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#30#, 16#00#, 16#C0#,  
        16#30#, 16#00#, 16#C0#, 16#38#, 16#01#, 16#C0#, 16#18#, 16#01#,  
        16#80#, 16#0F#, 16#FF#, 16#00#, 16#0F#, 16#FF#, 16#00#, 16#18#,  
        16#01#, 16#80#, 16#38#, 16#01#, 16#C0#, 16#30#, 16#00#, 16#C0#,  
        16#30#, 16#00#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#30#, 16#00#,  
        16#C0#, 16#30#, 16#00#, 16#C0#, 16#38#, 16#01#, 16#C0#, 16#1F#,  
        16#FF#, 16#80#, 16#0F#, 16#FF#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Eight_7x8_Width  : constant := 7;  
    Eight_7x8_Height : constant := 8;  

    Eight_7x8_Bits : constant U_Char_Array :=  
       (16#00#, 16#38#, 16#6C#, 16#38#, 16#6C#, 16#6C#, 16#38#, 16#00#);  

    Nine_10x12_Width  : constant := 10;  
    Nine_10x12_Height : constant := 12;  

    Nine_10x12_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#3F#, 16#00#, 16#7F#, 16#80#, 16#61#, 16#80#,  
        16#61#, 16#80#, 16#7F#, 16#80#, 16#3F#, 16#80#, 16#01#, 16#80#,  
        16#61#, 16#80#, 16#7F#, 16#80#, 16#3F#, 16#00#, 16#00#, 16#00#);  

    Nine_14x16_Width  : constant := 14;  
    Nine_14x16_Height : constant := 16;  

    Nine_14x16_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#0F#, 16#E0#, 16#1F#, 16#F0#, 16#38#, 16#38#,  
        16#30#, 16#18#, 16#30#, 16#18#, 16#38#, 16#38#, 16#1F#, 16#F8#,  
        16#0F#, 16#D8#, 16#00#, 16#18#, 16#00#, 16#18#, 16#30#, 16#18#,  
        16#38#, 16#38#, 16#1F#, 16#F0#, 16#0F#, 16#E0#, 16#00#, 16#00#);  

    Nine_17x20_Width  : constant := 17;  
    Nine_17x20_Height : constant := 20;  

    Nine_17x20_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#0F#, 16#FC#, 16#00#, 16#1F#, 16#FE#,  
        16#00#, 16#38#, 16#07#, 16#00#, 16#30#, 16#03#, 16#00#, 16#30#,  
        16#03#, 16#00#, 16#30#, 16#03#, 16#00#, 16#30#, 16#03#, 16#00#,  
        16#38#, 16#07#, 16#00#, 16#1F#, 16#FF#, 16#00#, 16#0F#, 16#FB#,  
        16#00#, 16#00#, 16#03#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#,  
        16#03#, 16#00#, 16#30#, 16#03#, 16#00#, 16#30#, 16#03#, 16#00#,  
        16#38#, 16#07#, 16#00#, 16#1F#, 16#FE#, 16#00#, 16#0F#, 16#FC#,  
        16#00#, 16#00#, 16#00#, 16#00#);  

    Nine_20x24_Width  : constant := 20;  
    Nine_20x24_Height : constant := 24;  

    Nine_20x24_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#0F#, 16#FF#, 16#00#, 16#1F#, 16#FF#,  
        16#80#, 16#38#, 16#01#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#30#,  
        16#00#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#30#, 16#00#, 16#C0#,  
        16#30#, 16#00#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#38#, 16#01#,  
        16#C0#, 16#1F#, 16#FF#, 16#C0#, 16#0F#, 16#FE#, 16#C0#, 16#00#,  
        16#00#, 16#C0#, 16#00#, 16#00#, 16#C0#, 16#00#, 16#00#, 16#C0#,  
        16#00#, 16#00#, 16#C0#, 16#30#, 16#00#, 16#C0#, 16#30#, 16#00#,  
        16#C0#, 16#30#, 16#00#, 16#C0#, 16#38#, 16#01#, 16#C0#, 16#1F#,  
        16#FF#, 16#80#, 16#0F#, 16#FF#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Nine_7x8_Width  : constant := 7;  
    Nine_7x8_Height : constant := 8;  

    Nine_7x8_Bits : constant U_Char_Array :=  
       (16#00#, 16#38#, 16#6C#, 16#3C#, 16#04#, 16#6C#, 16#38#, 16#00#);  

end Tile_Data_Numbers;  

