with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_Other is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_other - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----OTHER  
------------------------------------------------------------------------------  

    Bam_10x12_Width  : constant := 10;  
    Bam_10x12_Height : constant := 12;  

    Bam_10x12_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#3F#, 16#00#, 16#61#, 16#80#, 16#73#, 16#80#,  
        16#21#, 16#00#, 16#31#, 16#00#, 16#23#, 16#00#, 16#21#, 16#00#,  
        16#73#, 16#80#, 16#61#, 16#80#, 16#3F#, 16#00#, 16#00#, 16#00#);  

    Bam_14x16_Width  : constant := 14;  
    Bam_14x16_Height : constant := 16;  

    Bam_14x16_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#3F#, 16#F0#, 16#60#, 16#18#, 16#60#, 16#18#,  
        16#3B#, 16#70#, 16#10#, 16#20#, 16#10#, 16#20#, 16#10#, 16#20#,  
        16#10#, 16#20#, 16#10#, 16#20#, 16#10#, 16#20#, 16#3B#, 16#70#,  
        16#60#, 16#18#, 16#60#, 16#18#, 16#3F#, 16#F0#, 16#00#, 16#00#);  

    Bam_17x20_Width  : constant := 17;  
    Bam_17x20_Height : constant := 20;  

    Bam_17x20_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#3F#, 16#FE#, 16#00#, 16#60#, 16#03#,  
        16#00#, 16#60#, 16#03#, 16#00#, 16#3C#, 16#9E#, 16#00#, 16#10#,  
        16#04#, 16#00#, 16#10#, 16#04#, 16#00#, 16#10#, 16#04#, 16#00#,  
        16#10#, 16#04#, 16#00#, 16#10#, 16#04#, 16#00#, 16#10#, 16#04#,  
        16#00#, 16#10#, 16#04#, 16#00#, 16#10#, 16#04#, 16#00#, 16#10#,  
        16#04#, 16#00#, 16#10#, 16#04#, 16#00#, 16#3C#, 16#9E#, 16#00#,  
        16#60#, 16#03#, 16#00#, 16#60#, 16#03#, 16#00#, 16#3F#, 16#FE#,  
        16#00#, 16#00#, 16#00#, 16#00#);  

    Bam_20x24_Width  : constant := 20;  
    Bam_20x24_Height : constant := 24;  

    Bam_20x24_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#3F#, 16#FF#, 16#C0#, 16#60#, 16#00#,  
        16#60#, 16#60#, 16#00#, 16#60#, 16#60#, 16#00#, 16#60#, 16#3E#,  
        16#67#, 16#C0#, 16#10#, 16#00#, 16#80#, 16#10#, 16#00#, 16#80#,  
        16#10#, 16#00#, 16#80#, 16#10#, 16#00#, 16#80#, 16#10#, 16#00#,  
        16#80#, 16#10#, 16#00#, 16#80#, 16#10#, 16#00#, 16#80#, 16#10#,  
        16#00#, 16#80#, 16#10#, 16#00#, 16#80#, 16#10#, 16#00#, 16#80#,  
        16#10#, 16#00#, 16#80#, 16#10#, 16#00#, 16#80#, 16#3E#, 16#67#,  
        16#C0#, 16#60#, 16#00#, 16#60#, 16#60#, 16#00#, 16#60#, 16#60#,  
        16#00#, 16#60#, 16#3F#, 16#FF#, 16#C0#, 16#00#, 16#00#, 16#00#);  

    Bam_7x8_Width  : constant := 7;  
    Bam_7x8_Height : constant := 8;  

    Bam_7x8_Bits : constant U_Char_Array :=  
       (16#00#, 16#38#, 16#6C#, 16#28#, 16#28#, 16#6C#, 16#38#, 16#00#);  

    Crak_28x32_Width  : constant := 28;  
    Crak_28x32_Height : constant := 32;  

    Crak_28x32_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#06#, 16#00#, 16#00#, 16#03#, 16#06#, 16#00#, 16#00#,  
        16#03#, 16#FF#, 16#FC#, 16#00#, 16#03#, 16#FF#, 16#FC#, 16#00#,  
        16#00#, 16#10#, 16#40#, 16#00#, 16#01#, 16#FF#, 16#FC#, 16#00#,  
        16#01#, 16#7F#, 16#F4#, 16#00#, 16#01#, 16#90#, 16#4C#, 16#00#,  
        16#01#, 16#90#, 16#4C#, 16#00#, 16#01#, 16#90#, 16#4C#, 16#00#,  
        16#01#, 16#90#, 16#4C#, 16#00#, 16#01#, 16#90#, 16#4C#, 16#00#,  
        16#01#, 16#90#, 16#4C#, 16#00#, 16#01#, 16#7F#, 16#F4#, 16#00#,  
        16#09#, 16#FF#, 16#FD#, 16#00#, 16#1F#, 16#90#, 16#4F#, 16#80#,  
        16#09#, 16#90#, 16#4D#, 16#00#, 16#09#, 16#FF#, 16#FD#, 16#00#,  
        16#08#, 16#10#, 16#41#, 16#00#, 16#1F#, 16#FF#, 16#FF#, 16#80#,  
        16#08#, 16#10#, 16#41#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Crak_40x48_Width  : constant := 40;  
    Crak_40x48_Height : constant := 48;  

    Crak_40x48_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#7C#, 16#00#, 16#00#, 16#01#, 16#80#,  
        16#6C#, 16#00#, 16#00#, 16#01#, 16#80#, 16#6C#, 16#00#, 16#00#,  
        16#01#, 16#FF#, 16#FF#, 16#FF#, 16#80#, 16#01#, 16#FF#, 16#FF#,  
        16#FF#, 16#80#, 16#00#, 16#02#, 16#00#, 16#20#, 16#00#, 16#00#,  
        16#7F#, 16#FF#, 16#FF#, 16#00#, 16#00#, 16#7F#, 16#FF#, 16#FF#,  
        16#00#, 16#00#, 16#62#, 16#00#, 16#23#, 16#00#, 16#00#, 16#62#,  
        16#00#, 16#23#, 16#00#, 16#00#, 16#62#, 16#00#, 16#23#, 16#00#,  
        16#00#, 16#62#, 16#00#, 16#23#, 16#00#, 16#00#, 16#62#, 16#00#,  
        16#23#, 16#00#, 16#00#, 16#62#, 16#00#, 16#23#, 16#00#, 16#00#,  
        16#62#, 16#00#, 16#23#, 16#00#, 16#00#, 16#62#, 16#00#, 16#23#,  
        16#00#, 16#00#, 16#62#, 16#00#, 16#23#, 16#00#, 16#00#, 16#62#,  
        16#00#, 16#23#, 16#00#, 16#00#, 16#62#, 16#00#, 16#23#, 16#00#,  
        16#00#, 16#62#, 16#00#, 16#23#, 16#00#, 16#00#, 16#62#, 16#00#,  
        16#23#, 16#00#, 16#00#, 16#62#, 16#00#, 16#23#, 16#00#, 16#04#,  
        16#62#, 16#00#, 16#23#, 16#10#, 16#04#, 16#7F#, 16#FF#, 16#FF#,  
        16#10#, 16#1F#, 16#FF#, 16#FF#, 16#FF#, 16#FC#, 16#04#, 16#62#,  
        16#00#, 16#23#, 16#10#, 16#04#, 16#62#, 16#00#, 16#23#, 16#10#,  
        16#04#, 16#7F#, 16#FF#, 16#FF#, 16#10#, 16#04#, 16#02#, 16#00#,  
        16#20#, 16#10#, 16#04#, 16#02#, 16#00#, 16#20#, 16#10#, 16#1F#,  
        16#FF#, 16#FF#, 16#FF#, 16#FC#, 16#04#, 16#02#, 16#00#, 16#20#,  
        16#10#, 16#04#, 16#02#, 16#00#, 16#20#, 16#10#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Crak_56x64_Width  : constant := 56;  
    Crak_56x64_Height : constant := 64;  

    Crak_56x64_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#FE#, 16#00#, 16#00#, 16#00#, 16#00#, 16#30#, 16#00#,  
        16#C6#, 16#00#, 16#00#, 16#00#, 16#00#, 16#30#, 16#00#, 16#C6#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#30#, 16#00#, 16#C6#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#3F#, 16#FF#, 16#FF#, 16#FF#, 16#FE#,  
        16#00#, 16#00#, 16#3F#, 16#FF#, 16#FF#, 16#FF#, 16#FE#, 16#00#,  
        16#00#, 16#00#, 16#08#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#08#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#07#,  
        16#FF#, 16#FF#, 16#FF#, 16#F0#, 16#00#, 16#00#, 16#07#, 16#FF#,  
        16#FF#, 16#FF#, 16#F0#, 16#00#, 16#00#, 16#06#, 16#08#, 16#00#,  
        16#08#, 16#30#, 16#00#, 16#00#, 16#06#, 16#08#, 16#00#, 16#08#,  
        16#30#, 16#00#, 16#00#, 16#06#, 16#08#, 16#00#, 16#08#, 16#30#,  
        16#00#, 16#00#, 16#06#, 16#08#, 16#00#, 16#08#, 16#30#, 16#00#,  
        16#00#, 16#06#, 16#08#, 16#00#, 16#08#, 16#30#, 16#00#, 16#00#,  
        16#06#, 16#08#, 16#00#, 16#08#, 16#30#, 16#00#, 16#00#, 16#06#,  
        16#08#, 16#00#, 16#08#, 16#30#, 16#00#, 16#00#, 16#06#, 16#08#,  
        16#00#, 16#08#, 16#30#, 16#00#, 16#00#, 16#06#, 16#08#, 16#00#,  
        16#08#, 16#30#, 16#00#, 16#00#, 16#06#, 16#08#, 16#00#, 16#08#,  
        16#30#, 16#00#, 16#00#, 16#06#, 16#08#, 16#00#, 16#08#, 16#30#,  
        16#00#, 16#00#, 16#06#, 16#08#, 16#00#, 16#08#, 16#30#, 16#00#,  
        16#00#, 16#06#, 16#08#, 16#00#, 16#08#, 16#30#, 16#00#, 16#00#,  
        16#06#, 16#08#, 16#00#, 16#08#, 16#30#, 16#00#, 16#00#, 16#06#,  
        16#08#, 16#00#, 16#08#, 16#30#, 16#00#, 16#00#, 16#06#, 16#08#,  
        16#00#, 16#08#, 16#30#, 16#00#, 16#00#, 16#06#, 16#08#, 16#00#,  
        16#08#, 16#30#, 16#00#, 16#00#, 16#06#, 16#08#, 16#00#, 16#08#,  
        16#30#, 16#00#, 16#00#, 16#06#, 16#08#, 16#00#, 16#08#, 16#30#,  
        16#00#, 16#00#, 16#06#, 16#08#, 16#00#, 16#08#, 16#30#, 16#00#,  
        16#01#, 16#07#, 16#FF#, 16#FF#, 16#FF#, 16#F0#, 16#40#, 16#01#,  
        16#07#, 16#FF#, 16#FF#, 16#FF#, 16#F0#, 16#40#, 16#07#, 16#FE#,  
        16#08#, 16#00#, 16#08#, 16#3F#, 16#F0#, 16#01#, 16#06#, 16#08#,  
        16#00#, 16#08#, 16#30#, 16#40#, 16#01#, 16#06#, 16#08#, 16#00#,  
        16#08#, 16#30#, 16#40#, 16#01#, 16#06#, 16#08#, 16#00#, 16#08#,  
        16#30#, 16#40#, 16#01#, 16#07#, 16#FF#, 16#FF#, 16#FF#, 16#F0#,  
        16#40#, 16#01#, 16#00#, 16#08#, 16#00#, 16#08#, 16#00#, 16#40#,  
        16#01#, 16#00#, 16#08#, 16#00#, 16#08#, 16#00#, 16#40#, 16#07#,  
        16#FF#, 16#FF#, 16#FF#, 16#FF#, 16#FF#, 16#F0#, 16#01#, 16#00#,  
        16#08#, 16#00#, 16#08#, 16#00#, 16#40#, 16#01#, 16#00#, 16#08#,  
        16#00#, 16#08#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Crak_68x80_Width  : constant := 68;  
    Crak_68x80_Height : constant := 80;  

    Crak_68x80_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#07#, 16#F8#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#0C#, 16#00#, 16#06#, 16#18#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0C#, 16#00#, 16#06#, 16#18#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#0C#, 16#00#, 16#06#, 16#18#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#0C#, 16#00#, 16#06#, 16#18#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#FF#, 16#FF#,  
        16#FF#, 16#FF#, 16#FF#, 16#80#, 16#00#, 16#00#, 16#0F#, 16#FF#,  
        16#FF#, 16#FF#, 16#FF#, 16#FF#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#80#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#80#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#08#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#FF#, 16#FF#, 16#FF#, 16#FF#, 16#F8#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#FF#, 16#FF#, 16#FF#, 16#FF#,  
        16#F8#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#80#, 16#00#,  
        16#08#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#80#,  
        16#00#, 16#08#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C0#,  
        16#80#, 16#00#, 16#08#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#C0#, 16#80#, 16#00#, 16#08#, 16#18#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#C0#, 16#80#, 16#00#, 16#08#, 16#18#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#C0#, 16#80#, 16#00#, 16#08#, 16#18#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#C0#, 16#80#, 16#00#, 16#08#, 16#18#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#80#, 16#00#, 16#08#,  
        16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#80#, 16#00#,  
        16#08#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#80#,  
        16#00#, 16#08#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C0#,  
        16#80#, 16#00#, 16#08#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#C0#, 16#80#, 16#00#, 16#08#, 16#18#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#C0#, 16#80#, 16#00#, 16#08#, 16#18#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#C0#, 16#80#, 16#00#, 16#08#, 16#18#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#C0#, 16#80#, 16#00#, 16#08#, 16#18#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#80#, 16#00#, 16#08#,  
        16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#80#, 16#00#,  
        16#08#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#80#,  
        16#00#, 16#08#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C0#,  
        16#80#, 16#00#, 16#08#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#C0#, 16#80#, 16#00#, 16#08#, 16#18#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#C0#, 16#80#, 16#00#, 16#08#, 16#18#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#C0#, 16#80#, 16#00#, 16#08#, 16#18#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#C0#, 16#80#, 16#00#, 16#08#, 16#18#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#80#, 16#00#, 16#08#,  
        16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#80#, 16#00#,  
        16#08#, 16#18#, 16#00#, 16#00#, 16#00#, 16#80#, 16#FF#, 16#FF#,  
        16#FF#, 16#FF#, 16#F8#, 16#10#, 16#00#, 16#00#, 16#80#, 16#FF#,  
        16#FF#, 16#FF#, 16#FF#, 16#F8#, 16#10#, 16#00#, 16#00#, 16#80#,  
        16#C0#, 16#80#, 16#00#, 16#08#, 16#18#, 16#10#, 16#00#, 16#03#,  
        16#FF#, 16#C0#, 16#80#, 16#00#, 16#08#, 16#1F#, 16#FC#, 16#00#,  
        16#00#, 16#80#, 16#C0#, 16#80#, 16#00#, 16#08#, 16#18#, 16#10#,  
        16#00#, 16#00#, 16#80#, 16#C0#, 16#80#, 16#00#, 16#08#, 16#18#,  
        16#10#, 16#00#, 16#00#, 16#80#, 16#C0#, 16#80#, 16#00#, 16#08#,  
        16#18#, 16#10#, 16#00#, 16#00#, 16#80#, 16#C0#, 16#80#, 16#00#,  
        16#08#, 16#18#, 16#10#, 16#00#, 16#00#, 16#80#, 16#FF#, 16#FF#,  
        16#FF#, 16#FF#, 16#F8#, 16#10#, 16#00#, 16#00#, 16#80#, 16#00#,  
        16#80#, 16#00#, 16#08#, 16#00#, 16#10#, 16#00#, 16#00#, 16#80#,  
        16#00#, 16#80#, 16#00#, 16#08#, 16#00#, 16#10#, 16#00#, 16#03#,  
        16#FF#, 16#FF#, 16#FF#, 16#FF#, 16#FF#, 16#FF#, 16#FC#, 16#00#,  
        16#00#, 16#80#, 16#00#, 16#80#, 16#00#, 16#08#, 16#00#, 16#10#,  
        16#00#, 16#00#, 16#80#, 16#00#, 16#80#, 16#00#, 16#08#, 16#00#,  
        16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Crak_80x96_Width  : constant := 80;  
    Crak_80x96_Height : constant := 96;  

    Crak_80x96_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#3F#, 16#C0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#60#, 16#00#, 16#30#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#60#, 16#00#,  
        16#30#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#60#, 16#00#, 16#30#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#60#, 16#00#, 16#30#, 16#C0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#7F#, 16#FF#, 16#FF#, 16#FF#,  
        16#FF#, 16#FC#, 16#00#, 16#00#, 16#00#, 16#00#, 16#7F#, 16#FF#,  
        16#FF#, 16#FF#, 16#FF#, 16#FC#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#00#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#FF#,  
        16#FF#, 16#FF#, 16#FF#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#07#, 16#FF#, 16#FF#, 16#FF#, 16#FF#, 16#C0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#,  
        16#40#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#04#,  
        16#00#, 16#00#, 16#40#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#,  
        16#40#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#04#,  
        16#00#, 16#00#, 16#40#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#,  
        16#40#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#04#,  
        16#00#, 16#00#, 16#40#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#,  
        16#40#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#04#,  
        16#00#, 16#00#, 16#40#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#,  
        16#40#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#04#,  
        16#00#, 16#00#, 16#40#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#,  
        16#40#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#04#,  
        16#00#, 16#00#, 16#40#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#04#, 16#07#, 16#FF#, 16#FF#, 16#FF#,  
        16#FF#, 16#C0#, 16#80#, 16#00#, 16#00#, 16#04#, 16#07#, 16#FF#,  
        16#FF#, 16#FF#, 16#FF#, 16#C0#, 16#80#, 16#00#, 16#00#, 16#04#,  
        16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#, 16#80#, 16#00#,  
        16#00#, 16#1F#, 16#FE#, 16#04#, 16#00#, 16#00#, 16#40#, 16#FF#,  
        16#E0#, 16#00#, 16#00#, 16#04#, 16#06#, 16#04#, 16#00#, 16#00#,  
        16#40#, 16#C0#, 16#80#, 16#00#, 16#00#, 16#04#, 16#06#, 16#04#,  
        16#00#, 16#00#, 16#40#, 16#C0#, 16#80#, 16#00#, 16#00#, 16#04#,  
        16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#, 16#80#, 16#00#,  
        16#00#, 16#04#, 16#06#, 16#04#, 16#00#, 16#00#, 16#40#, 16#C0#,  
        16#80#, 16#00#, 16#00#, 16#04#, 16#07#, 16#FF#, 16#FF#, 16#FF#,  
        16#FF#, 16#C0#, 16#80#, 16#00#, 16#00#, 16#04#, 16#00#, 16#04#,  
        16#00#, 16#00#, 16#40#, 16#00#, 16#80#, 16#00#, 16#00#, 16#04#,  
        16#00#, 16#04#, 16#00#, 16#00#, 16#40#, 16#00#, 16#80#, 16#00#,  
        16#00#, 16#1F#, 16#FF#, 16#FF#, 16#FF#, 16#FF#, 16#FF#, 16#FF#,  
        16#E0#, 16#00#, 16#00#, 16#04#, 16#00#, 16#04#, 16#00#, 16#00#,  
        16#40#, 16#00#, 16#80#, 16#00#, 16#00#, 16#04#, 16#00#, 16#04#,  
        16#00#, 16#00#, 16#40#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Dot_10x12_Width  : constant := 10;  
    Dot_10x12_Height : constant := 12;  

    Dot_10x12_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#0C#, 16#00#, 16#3F#, 16#00#, 16#6D#, 16#80#,  
        16#3F#, 16#00#, 16#73#, 16#80#, 16#73#, 16#80#, 16#3F#, 16#00#,  
        16#6D#, 16#80#, 16#3F#, 16#00#, 16#0C#, 16#00#, 16#00#, 16#00#);  

    Dot_14x16_Width  : constant := 14;  
    Dot_14x16_Height : constant := 16;  

    Dot_14x16_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#03#, 16#00#, 16#3B#, 16#70#, 16#3F#, 16#F0#,  
        16#77#, 16#B8#, 16#3B#, 16#70#, 16#5F#, 16#E8#, 16#7C#, 16#F8#,  
        16#7C#, 16#F8#, 16#5F#, 16#E8#, 16#3B#, 16#70#, 16#77#, 16#B8#,  
        16#3F#, 16#F0#, 16#3B#, 16#70#, 16#03#, 16#00#, 16#00#, 16#00#);  

    Dot_17x20_Width  : constant := 17;  
    Dot_17x20_Height : constant := 20;  

    Dot_17x20_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#7F#, 16#0B#, 16#E8#, 16#7F#, 16#1D#, 16#DC#,  
        16#7F#, 16#2F#, 16#7A#, 16#7F#, 16#77#, 16#77#, 16#7F#, 16#3F#,  
        16#FE#, 16#7F#, 16#5F#, 16#FD#, 16#7F#, 16#7C#, 16#1F#, 16#7F#,  
        16#7D#, 16#DF#, 16#7F#, 16#0D#, 16#D8#, 16#7F#, 16#7D#, 16#DF#,  
        16#7F#, 16#7C#, 16#1F#, 16#7F#, 16#5F#, 16#FD#, 16#7F#, 16#3F#,  
        16#FE#, 16#7F#, 16#7F#, 16#7F#, 16#7F#, 16#77#, 16#77#, 16#7F#,  
        16#2F#, 16#FA#, 16#7F#, 16#1D#, 16#DC#, 16#7F#, 16#0B#, 16#E8#,  
        16#7F#, 16#00#, 16#00#, 16#7F#);  

    Dot_20x24_Width  : constant := 20;  
    Dot_20x24_Height : constant := 24;  

    Dot_20x24_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#0F#, 16#03#, 16#FC#, 16#0F#, 16#0D#, 16#FB#,  
        16#0F#, 16#0E#, 16#F7#, 16#0F#, 16#37#, 16#6E#, 16#CF#, 16#7B#,  
        16#9D#, 16#EF#, 16#6D#, 16#FB#, 16#6F#, 16#78#, 16#01#, 16#EF#,  
        16#3B#, 16#FD#, 16#CF#, 16#5B#, 16#FD#, 16#AF#, 16#6B#, 16#0D#,  
        16#6F#, 16#73#, 16#6C#, 16#EF#, 16#73#, 16#6C#, 16#EF#, 16#6B#,  
        16#0D#, 16#6F#, 16#5B#, 16#FD#, 16#AF#, 16#3B#, 16#FD#, 16#CF#,  
        16#78#, 16#01#, 16#EF#, 16#6D#, 16#FB#, 16#6F#, 16#7B#, 16#9D#,  
        16#EF#, 16#37#, 16#6E#, 16#CF#, 16#0E#, 16#F7#, 16#0F#, 16#0D#,  
        16#FB#, 16#0F#, 16#03#, 16#FC#, 16#0F#, 16#00#, 16#00#, 16#0F#);  

    Dot_7x8_Width  : constant := 7;  
    Dot_7x8_Height : constant := 8;  

    Dot_7x8_Bits : constant U_Char_Array :=  
       (16#00#, 16#54#, 16#38#, 16#6C#, 16#38#, 16#54#, 16#00#, 16#00#);  

end Tile_Data_Other;  

