with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_Plum is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_plum - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----FLOWERS  
------------------------------------------------------------------------------  

    Plum_28x32_Width  : constant := 28;  
    Plum_28x32_Height : constant := 32;  

    Plum_28x32_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#07#, 16#00#, 16#00#, 16#00#, 16#18#, 16#C0#, 16#00#,  
        16#00#, 16#10#, 16#40#, 16#00#, 16#00#, 16#20#, 16#20#, 16#00#,  
        16#00#, 16#20#, 16#20#, 16#00#, 16#00#, 16#E0#, 16#38#, 16#00#,  
        16#03#, 16#10#, 16#46#, 16#00#, 16#02#, 16#02#, 16#02#, 16#00#,  
        16#04#, 16#07#, 16#01#, 16#00#, 16#04#, 16#0D#, 16#81#, 16#00#,  
        16#04#, 16#07#, 16#01#, 16#00#, 16#02#, 16#02#, 16#02#, 16#00#,  
        16#03#, 16#10#, 16#46#, 16#00#, 16#00#, 16#E0#, 16#38#, 16#00#,  
        16#00#, 16#20#, 16#20#, 16#00#, 16#00#, 16#20#, 16#20#, 16#00#,  
        16#00#, 16#10#, 16#40#, 16#00#, 16#00#, 16#18#, 16#C0#, 16#00#,  
        16#00#, 16#07#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#0E#, 16#42#, 16#52#, 16#00#,  
        16#09#, 16#42#, 16#5E#, 16#00#, 16#09#, 16#42#, 16#52#, 16#00#,  
        16#0E#, 16#42#, 16#52#, 16#00#, 16#08#, 16#42#, 16#52#, 16#00#,  
        16#08#, 16#79#, 16#92#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Plum_40x48_Width  : constant := 40;  
    Plum_40x48_Height : constant := 48;  

    Plum_40x48_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#7C#, 16#00#, 16#00#, 16#00#, 16#01#, 16#83#, 16#00#, 16#00#,  
        16#00#, 16#02#, 16#00#, 16#80#, 16#00#, 16#00#, 16#02#, 16#00#,  
        16#80#, 16#00#, 16#00#, 16#04#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#04#, 16#00#, 16#40#, 16#00#, 16#00#, 16#04#, 16#00#, 16#40#,  
        16#00#, 16#00#, 16#3E#, 16#00#, 16#F8#, 16#00#, 16#00#, 16#C4#,  
        16#00#, 16#46#, 16#00#, 16#01#, 16#00#, 16#00#, 16#01#, 16#00#,  
        16#01#, 16#00#, 16#10#, 16#01#, 16#00#, 16#02#, 16#00#, 16#38#,  
        16#00#, 16#80#, 16#02#, 16#00#, 16#54#, 16#00#, 16#80#, 16#02#,  
        16#00#, 16#FE#, 16#00#, 16#80#, 16#02#, 16#00#, 16#54#, 16#00#,  
        16#80#, 16#02#, 16#00#, 16#38#, 16#00#, 16#80#, 16#01#, 16#00#,  
        16#10#, 16#01#, 16#00#, 16#01#, 16#00#, 16#00#, 16#01#, 16#00#,  
        16#00#, 16#C4#, 16#00#, 16#46#, 16#00#, 16#00#, 16#3E#, 16#00#,  
        16#F8#, 16#00#, 16#00#, 16#04#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#04#, 16#00#, 16#40#, 16#00#, 16#00#, 16#04#, 16#00#, 16#40#,  
        16#00#, 16#00#, 16#02#, 16#00#, 16#80#, 16#00#, 16#00#, 16#02#,  
        16#00#, 16#80#, 16#00#, 16#00#, 16#01#, 16#83#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#7C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#F2#, 16#08#, 16#A2#, 16#00#,  
        16#00#, 16#8A#, 16#08#, 16#B6#, 16#00#, 16#00#, 16#8A#, 16#08#,  
        16#B6#, 16#00#, 16#00#, 16#8A#, 16#08#, 16#AA#, 16#00#, 16#00#,  
        16#F2#, 16#08#, 16#AA#, 16#00#, 16#00#, 16#82#, 16#08#, 16#A2#,  
        16#00#, 16#00#, 16#82#, 16#08#, 16#A2#, 16#00#, 16#00#, 16#83#,  
        16#E7#, 16#22#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Plum_56x64_Width  : constant := 56;  
    Plum_56x64_Height : constant := 64;  

    Plum_56x64_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#FE#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#01#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#20#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#10#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#10#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#20#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#,  
        16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#01#, 16#E0#, 16#00#, 16#0F#,  
        16#00#, 16#00#, 16#00#, 16#06#, 16#10#, 16#00#, 16#10#, 16#C0#,  
        16#00#, 16#00#, 16#08#, 16#08#, 16#00#, 16#20#, 16#20#, 16#00#,  
        16#00#, 16#10#, 16#04#, 16#00#, 16#40#, 16#10#, 16#00#, 16#00#,  
        16#20#, 16#02#, 16#00#, 16#80#, 16#08#, 16#00#, 16#00#, 16#20#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#00#, 16#40#, 16#00#,  
        16#10#, 16#00#, 16#04#, 16#00#, 16#00#, 16#40#, 16#00#, 16#38#,  
        16#00#, 16#04#, 16#00#, 16#00#, 16#40#, 16#00#, 16#54#, 16#00#,  
        16#04#, 16#00#, 16#00#, 16#40#, 16#00#, 16#FE#, 16#00#, 16#04#,  
        16#00#, 16#00#, 16#40#, 16#00#, 16#54#, 16#00#, 16#04#, 16#00#,  
        16#00#, 16#40#, 16#00#, 16#38#, 16#00#, 16#04#, 16#00#, 16#00#,  
        16#40#, 16#00#, 16#10#, 16#00#, 16#04#, 16#00#, 16#00#, 16#20#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#00#, 16#20#, 16#02#,  
        16#00#, 16#80#, 16#08#, 16#00#, 16#00#, 16#10#, 16#04#, 16#00#,  
        16#40#, 16#10#, 16#00#, 16#00#, 16#08#, 16#08#, 16#00#, 16#20#,  
        16#20#, 16#00#, 16#00#, 16#06#, 16#10#, 16#00#, 16#10#, 16#C0#,  
        16#00#, 16#00#, 16#01#, 16#E0#, 16#00#, 16#0F#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#20#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#20#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#,  
        16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#,  
        16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#20#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#03#, 16#01#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#FE#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#07#, 16#C4#, 16#04#, 16#14#, 16#10#,  
        16#00#, 16#00#, 16#04#, 16#24#, 16#04#, 16#16#, 16#30#, 16#00#,  
        16#00#, 16#04#, 16#14#, 16#04#, 16#16#, 16#30#, 16#00#, 16#00#,  
        16#04#, 16#14#, 16#04#, 16#16#, 16#30#, 16#00#, 16#00#, 16#04#,  
        16#14#, 16#04#, 16#15#, 16#50#, 16#00#, 16#00#, 16#04#, 16#24#,  
        16#04#, 16#15#, 16#50#, 16#00#, 16#00#, 16#07#, 16#C4#, 16#04#,  
        16#15#, 16#50#, 16#00#, 16#00#, 16#04#, 16#04#, 16#04#, 16#14#,  
        16#90#, 16#00#, 16#00#, 16#04#, 16#04#, 16#04#, 16#14#, 16#90#,  
        16#00#, 16#00#, 16#04#, 16#04#, 16#02#, 16#24#, 16#10#, 16#00#,  
        16#00#, 16#04#, 16#07#, 16#F1#, 16#C4#, 16#10#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Plum_68x80_Width  : constant := 68;  
    Plum_68x80_Height : constant := 80;  

    Plum_68x80_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#F0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#18#, 16#0C#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#02#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#,  
        16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#80#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#80#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#00#, 16#00#, 16#78#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#30#, 16#80#, 16#00#,  
        16#86#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#40#,  
        16#01#, 16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#,  
        16#20#, 16#02#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#00#, 16#10#, 16#04#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#02#, 16#00#, 16#00#, 16#80#, 16#00#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#02#, 16#00#, 16#01#, 16#C0#, 16#00#, 16#20#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#02#, 16#A0#, 16#00#,  
        16#20#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#07#, 16#F0#,  
        16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#02#,  
        16#A0#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#,  
        16#01#, 16#C0#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#00#, 16#00#, 16#80#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#00#, 16#10#, 16#04#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#80#, 16#20#, 16#02#, 16#00#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#40#, 16#01#, 16#01#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#30#, 16#80#, 16#00#,  
        16#86#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#00#,  
        16#00#, 16#78#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#01#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#02#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#18#,  
        16#0C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#07#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#07#, 16#F0#, 16#80#, 16#20#, 16#4C#, 16#30#, 16#00#,  
        16#00#, 16#00#, 16#04#, 16#08#, 16#80#, 16#20#, 16#4C#, 16#30#,  
        16#00#, 16#00#, 16#00#, 16#04#, 16#04#, 16#80#, 16#20#, 16#4C#,  
        16#30#, 16#00#, 16#00#, 16#00#, 16#04#, 16#04#, 16#80#, 16#20#,  
        16#4A#, 16#50#, 16#00#, 16#00#, 16#00#, 16#04#, 16#04#, 16#80#,  
        16#20#, 16#4A#, 16#50#, 16#00#, 16#00#, 16#00#, 16#04#, 16#04#,  
        16#80#, 16#20#, 16#4A#, 16#50#, 16#00#, 16#00#, 16#00#, 16#04#,  
        16#08#, 16#80#, 16#20#, 16#49#, 16#90#, 16#00#, 16#00#, 16#00#,  
        16#07#, 16#F0#, 16#80#, 16#20#, 16#49#, 16#90#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#00#, 16#80#, 16#20#, 16#49#, 16#90#, 16#00#,  
        16#00#, 16#00#, 16#04#, 16#00#, 16#80#, 16#20#, 16#48#, 16#10#,  
        16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#80#, 16#20#, 16#48#,  
        16#10#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#80#, 16#10#,  
        16#88#, 16#10#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#FF#,  
        16#0F#, 16#08#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Plum_80x96_Width  : constant := 80;  
    Plum_80x96_Height : constant := 96;  

    Plum_80x96_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#E0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#30#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#40#, 16#04#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#80#, 16#02#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#01#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#00#, 16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#02#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1E#,  
        16#00#, 16#00#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#61#, 16#00#, 16#01#, 16#0C#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#80#, 16#80#, 16#02#, 16#02#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#40#, 16#04#,  
        16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#,  
        16#20#, 16#08#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#04#, 16#00#, 16#01#, 16#00#, 16#00#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#03#, 16#80#,  
        16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#,  
        16#05#, 16#40#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#04#, 16#00#, 16#0F#, 16#E0#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#04#, 16#00#, 16#05#, 16#40#, 16#00#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#03#, 16#80#,  
        16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#,  
        16#01#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#02#, 16#00#, 16#20#, 16#08#, 16#00#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#40#, 16#04#,  
        16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#,  
        16#80#, 16#02#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#61#, 16#00#, 16#01#, 16#0C#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#1E#, 16#00#, 16#00#, 16#F0#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#00#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#02#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#01#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#01#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#80#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#40#, 16#04#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#30#, 16#18#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#E0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#3F#, 16#82#, 16#00#, 16#40#, 16#8C#, 16#18#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#42#, 16#00#, 16#40#, 16#8C#, 16#18#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#22#, 16#00#, 16#40#,  
        16#8C#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#22#,  
        16#00#, 16#40#, 16#8A#, 16#28#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#20#, 16#22#, 16#00#, 16#40#, 16#8A#, 16#28#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#22#, 16#00#, 16#40#, 16#8A#, 16#28#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#42#, 16#00#, 16#40#,  
        16#89#, 16#48#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3F#, 16#82#,  
        16#00#, 16#40#, 16#89#, 16#48#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#20#, 16#02#, 16#00#, 16#40#, 16#89#, 16#48#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#02#, 16#00#, 16#40#, 16#88#, 16#88#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#02#, 16#00#, 16#40#,  
        16#88#, 16#88#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#02#,  
        16#00#, 16#21#, 16#08#, 16#88#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#20#, 16#03#, 16#FC#, 16#1E#, 16#08#, 16#88#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

end Tile_Data_Plum;  

