with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_Wdragon is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_wdragon - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----DRAGONS  
------------------------------------------------------------------------------  

    Wdragon_28x32_Width  : constant := 28;  
    Wdragon_28x32_Height : constant := 32;  

    Wdragon_28x32_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#82#, 16#A0#,  
        16#00#, 16#03#, 16#82#, 16#A0#, 16#00#, 16#05#, 16#02#, 16#A0#,  
        16#60#, 16#05#, 16#01#, 16#40#, 16#1F#, 16#FD#, 16#01#, 16#40#,  
        16#1C#, 16#10#, 16#81#, 16#40#, 16#37#, 16#00#, 16#C0#, 16#00#,  
        16#40#, 16#09#, 16#60#, 16#00#, 16#07#, 16#00#, 16#20#, 16#00#,  
        16#00#, 16#21#, 16#90#, 16#00#, 16#40#, 16#03#, 16#D8#, 16#00#,  
        16#00#, 16#01#, 16#24#, 16#00#, 16#07#, 16#09#, 16#74#, 16#00#,  
        16#24#, 16#01#, 16#F6#, 16#00#, 16#14#, 16#2C#, 16#F2#, 16#00#,  
        16#04#, 16#44#, 16#12#, 16#00#, 16#02#, 16#03#, 16#02#, 16#00#,  
        16#02#, 16#00#, 16#89#, 16#00#, 16#71#, 16#88#, 16#C1#, 16#80#,  
        16#0C#, 16#40#, 16#20#, 16#80#, 16#03#, 16#E0#, 16#22#, 16#80#,  
        16#00#, 16#10#, 16#18#, 16#C0#, 16#00#, 16#0C#, 16#48#, 16#40#,  
        16#00#, 16#02#, 16#04#, 16#60#, 16#00#, 16#03#, 16#06#, 16#60#,  
        16#00#, 16#00#, 16#8E#, 16#60#, 16#00#, 16#00#, 16#DF#, 16#20#,  
        16#00#, 16#00#, 16#21#, 16#A0#, 16#00#, 16#00#, 16#38#, 16#C0#,  
        16#00#, 16#00#, 16#07#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Wdragon_40x48_Width  : constant := 40;  
    Wdragon_40x48_Height : constant := 48;  

    Wdragon_40x48_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#0A#, 16#00#, 16#44#, 16#00#,  
        16#00#, 16#14#, 16#00#, 16#54#, 16#00#, 16#00#, 16#14#, 16#00#,  
        16#54#, 16#00#, 16#00#, 16#14#, 16#00#, 16#54#, 16#60#, 16#00#,  
        16#26#, 16#00#, 16#28#, 16#10#, 16#60#, 16#22#, 16#00#, 16#28#,  
        16#4F#, 16#9C#, 16#22#, 16#00#, 16#28#, 16#03#, 16#03#, 16#F1#,  
        16#00#, 16#00#, 16#3C#, 16#00#, 16#01#, 16#00#, 16#00#, 16#41#,  
        16#A0#, 16#31#, 16#80#, 16#00#, 16#04#, 16#00#, 16#6A#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#4A#, 16#60#, 16#00#, 16#01#, 16#63#,  
        16#40#, 16#20#, 16#00#, 16#00#, 16#02#, 16#00#, 16#30#, 16#00#,  
        16#40#, 16#82#, 16#07#, 16#18#, 16#00#, 16#20#, 16#00#, 16#47#,  
        16#C8#, 16#00#, 16#00#, 16#00#, 16#08#, 16#24#, 16#00#, 16#00#,  
        16#00#, 16#28#, 16#E2#, 16#00#, 16#00#, 16#E0#, 16#26#, 16#F2#,  
        16#00#, 16#11#, 16#00#, 16#09#, 16#F3#, 16#00#, 16#49#, 16#00#,  
        16#05#, 16#F3#, 16#00#, 16#49#, 16#02#, 16#20#, 16#71#, 16#00#,  
        16#00#, 16#86#, 16#30#, 16#19#, 16#00#, 16#00#, 16#80#, 16#0C#,  
        16#01#, 16#00#, 16#00#, 16#83#, 16#02#, 16#01#, 16#80#, 16#00#,  
        16#40#, 16#01#, 16#90#, 16#80#, 16#00#, 16#20#, 16#04#, 16#94#,  
        16#40#, 16#7E#, 16#30#, 16#30#, 16#40#, 16#60#, 16#41#, 16#C8#,  
        16#00#, 16#40#, 16#20#, 16#00#, 16#EC#, 16#00#, 16#22#, 16#20#,  
        16#00#, 16#3F#, 16#02#, 16#23#, 16#30#, 16#00#, 16#00#, 16#80#,  
        16#18#, 16#28#, 16#00#, 16#00#, 16#42#, 16#48#, 16#08#, 16#00#,  
        16#00#, 16#70#, 16#04#, 16#08#, 16#00#, 16#00#, 16#08#, 16#86#,  
        16#24#, 16#00#, 16#00#, 16#08#, 16#02#, 16#6C#, 16#00#, 16#00#,  
        16#06#, 16#21#, 16#1E#, 16#00#, 16#00#, 16#01#, 16#00#, 16#B2#,  
        16#00#, 16#00#, 16#01#, 16#80#, 16#C6#, 16#00#, 16#00#, 16#00#,  
        16#C7#, 16#26#, 16#00#, 16#00#, 16#00#, 16#23#, 16#F2#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#52#, 16#00#, 16#00#, 16#00#, 16#18#,  
        16#1A#, 16#00#, 16#00#, 16#00#, 16#06#, 16#04#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#F8#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Wdragon_56x64_Width  : constant := 56;  
    Wdragon_56x64_Height : constant := 64;  

    Wdragon_56x64_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#05#, 16#00#, 16#01#, 16#04#, 16#00#, 16#00#, 16#00#,  
        16#05#, 16#00#, 16#01#, 16#24#, 16#00#, 16#00#, 16#00#, 16#0A#,  
        16#00#, 16#01#, 16#24#, 16#00#, 16#00#, 16#00#, 16#1A#, 16#00#,  
        16#01#, 16#24#, 16#00#, 16#00#, 16#00#, 16#12#, 16#00#, 16#01#,  
        16#54#, 16#00#, 16#00#, 16#00#, 16#12#, 16#00#, 16#01#, 16#54#,  
        16#60#, 16#00#, 16#00#, 16#12#, 16#00#, 16#01#, 16#54#, 16#18#,  
        16#00#, 16#00#, 16#21#, 16#00#, 16#00#, 16#88#, 16#08#, 16#00#,  
        16#00#, 16#21#, 16#00#, 16#00#, 16#88#, 16#07#, 16#79#, 16#E0#,  
        16#31#, 16#00#, 16#00#, 16#88#, 16#00#, 16#E6#, 16#1F#, 16#E0#,  
        16#80#, 16#00#, 16#88#, 16#3F#, 16#00#, 16#40#, 16#00#, 16#80#,  
        16#00#, 16#00#, 16#4C#, 16#00#, 16#00#, 16#00#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#32#, 16#00#, 16#60#, 16#60#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#00#, 16#40#, 16#10#, 16#00#, 16#00#, 16#04#,  
        16#00#, 16#01#, 16#89#, 16#18#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#00#, 16#80#, 16#88#, 16#00#, 16#00#, 16#00#, 16#34#, 16#0C#,  
        16#00#, 16#84#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#,  
        16#06#, 16#00#, 16#00#, 16#40#, 16#10#, 16#08#, 16#23#, 16#83#,  
        16#00#, 16#00#, 16#00#, 16#10#, 16#08#, 16#46#, 16#E1#, 16#00#,  
        16#00#, 16#10#, 16#00#, 16#20#, 16#82#, 16#1C#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#04#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#2A#, 16#02#, 16#40#, 16#00#, 16#40#, 16#0F#,  
        16#00#, 16#2A#, 16#3E#, 16#20#, 16#00#, 16#00#, 16#34#, 16#00#,  
        16#23#, 16#37#, 16#20#, 16#00#, 16#08#, 16#20#, 16#00#, 16#00#,  
        16#A7#, 16#10#, 16#00#, 16#00#, 16#20#, 16#00#, 16#08#, 16#BF#,  
        16#10#, 16#00#, 16#22#, 16#20#, 16#02#, 16#22#, 16#7F#, 16#50#,  
        16#00#, 16#30#, 16#30#, 16#08#, 16#18#, 16#1F#, 16#58#, 16#00#,  
        16#00#, 16#10#, 16#10#, 16#0C#, 16#07#, 16#08#, 16#00#, 16#00#,  
        16#08#, 16#00#, 16#02#, 16#81#, 16#08#, 16#00#, 16#00#, 16#09#,  
        16#08#, 16#03#, 16#01#, 16#08#, 16#00#, 16#41#, 16#08#, 16#0C#,  
        16#01#, 16#80#, 16#0C#, 16#00#, 16#00#, 16#04#, 16#00#, 16#00#,  
        16#40#, 16#04#, 16#00#, 16#20#, 16#42#, 16#00#, 16#23#, 16#22#,  
        16#02#, 16#00#, 16#00#, 16#02#, 16#00#, 16#02#, 16#21#, 16#42#,  
        16#00#, 16#7F#, 16#81#, 16#80#, 16#20#, 16#10#, 16#03#, 16#00#,  
        16#40#, 16#70#, 16#40#, 16#10#, 16#10#, 16#01#, 16#00#, 16#00#,  
        16#1E#, 16#E0#, 16#00#, 16#08#, 16#00#, 16#80#, 16#00#, 16#03#,  
        16#20#, 16#00#, 16#04#, 16#30#, 16#80#, 16#00#, 16#00#, 16#1C#,  
        16#01#, 16#06#, 16#08#, 16#C0#, 16#00#, 16#00#, 16#03#, 16#00#,  
        16#03#, 16#00#, 16#A0#, 16#00#, 16#00#, 16#01#, 16#00#, 16#11#,  
        16#00#, 16#A0#, 16#00#, 16#00#, 16#00#, 16#C1#, 16#01#, 16#A0#,  
        16#10#, 16#00#, 16#00#, 16#00#, 16#60#, 16#80#, 16#40#, 16#10#,  
        16#00#, 16#00#, 16#00#, 16#10#, 16#40#, 16#41#, 16#08#, 16#00#,  
        16#00#, 16#00#, 16#08#, 16#40#, 16#62#, 16#38#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#00#, 16#30#, 16#B4#, 16#00#, 16#00#, 16#00#,  
        16#03#, 16#08#, 16#98#, 16#3C#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#80#, 16#05#, 16#D4#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#,  
        16#0E#, 16#14#, 16#00#, 16#00#, 16#00#, 16#00#, 16#62#, 16#1A#,  
        16#0C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#30#, 16#71#, 16#8C#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#38#, 16#C2#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#04#, 16#0B#, 16#C2#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#04#, 16#40#, 16#62#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#10#, 16#24#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#C0#, 16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#68#, 16#7C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1F#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Wdragon_68x80_Width  : constant := 68;  
    Wdragon_68x80_Height : constant := 80;  

    Wdragon_68x80_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#00#,  
        16#00#, 16#44#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0A#,  
        16#00#, 16#00#, 16#44#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#12#, 16#00#, 16#00#, 16#44#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#22#, 16#00#, 16#00#, 16#44#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#24#, 16#00#, 16#00#, 16#44#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#44#, 16#00#, 16#00#, 16#2A#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#44#, 16#00#, 16#00#, 16#2A#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#44#, 16#00#, 16#00#,  
        16#2A#, 16#80#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#44#, 16#00#,  
        16#00#, 16#2A#, 16#80#, 16#18#, 16#00#, 16#00#, 16#00#, 16#84#,  
        16#00#, 16#00#, 16#11#, 16#00#, 16#04#, 16#00#, 16#00#, 16#00#,  
        16#84#, 16#00#, 16#00#, 16#11#, 16#00#, 16#03#, 16#00#, 16#00#,  
        16#00#, 16#82#, 16#00#, 16#00#, 16#11#, 16#00#, 16#00#, 16#9E#,  
        16#3E#, 16#00#, 16#C2#, 16#00#, 16#00#, 16#11#, 16#00#, 16#00#,  
        16#79#, 16#C5#, 16#FF#, 16#82#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#03#, 16#C0#, 16#00#, 16#00#, 16#01#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#1E#, 16#00#, 16#04#, 16#00#, 16#01#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#E0#, 16#00#, 16#01#, 16#00#, 16#00#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#80#, 16#0C#, 16#40#, 16#01#, 16#00#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#02#,  
        16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#02#, 16#41#, 16#00#,  
        16#0C#, 16#A0#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#00#, 16#04#, 16#02#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#40#, 16#00#, 16#01#, 16#08#, 16#00#, 16#00#, 16#00#,  
        16#80#, 16#04#, 16#80#, 16#E0#, 16#01#, 16#04#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#00#, 16#80#, 16#00#, 16#06#, 16#00#,  
        16#00#, 16#00#, 16#C0#, 16#04#, 16#00#, 16#81#, 16#00#, 16#03#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#83#, 16#0F#,  
        16#00#, 16#80#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#, 16#02#,  
        16#39#, 16#E8#, 16#80#, 16#00#, 16#00#, 16#08#, 16#00#, 16#02#,  
        16#04#, 16#08#, 16#1C#, 16#40#, 16#00#, 16#00#, 16#08#, 16#00#,  
        16#04#, 16#00#, 16#04#, 16#04#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#08#, 16#00#, 16#24#, 16#02#, 16#20#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#00#, 16#A4#, 16#02#, 16#10#, 16#00#,  
        16#00#, 16#20#, 16#00#, 16#C0#, 16#00#, 16#84#, 16#3F#, 16#08#,  
        16#00#, 16#00#, 16#00#, 16#04#, 16#80#, 16#00#, 16#84#, 16#63#,  
        16#08#, 16#00#, 16#00#, 16#04#, 16#08#, 16#00#, 16#00#, 16#02#,  
        16#47#, 16#08#, 16#00#, 16#00#, 16#08#, 16#48#, 16#00#, 16#00#,  
        16#21#, 16#4F#, 16#04#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#,  
        16#01#, 16#00#, 16#FF#, 16#04#, 16#00#, 16#00#, 16#10#, 16#88#,  
        16#00#, 16#10#, 16#84#, 16#7F#, 16#24#, 16#00#, 16#00#, 16#10#,  
        16#04#, 16#00#, 16#80#, 16#60#, 16#3F#, 16#24#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#01#, 16#00#, 16#30#, 16#07#, 16#04#, 16#00#,  
        16#00#, 16#00#, 16#02#, 16#08#, 16#00#, 16#09#, 16#01#, 16#44#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#80#, 16#0C#, 16#01#,  
        16#04#, 16#00#, 16#00#, 16#00#, 16#01#, 16#10#, 16#00#, 16#06#,  
        16#00#, 16#04#, 16#00#, 16#00#, 16#40#, 16#41#, 16#00#, 16#40#,  
        16#03#, 16#00#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#,  
        16#20#, 16#00#, 16#C0#, 16#01#, 16#00#, 16#00#, 16#20#, 16#10#,  
        16#40#, 16#00#, 16#80#, 16#42#, 16#01#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#40#, 16#00#, 16#86#, 16#61#, 16#00#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#00#, 16#04#, 16#20#, 16#30#, 16#80#,  
        16#00#, 16#7F#, 16#C0#, 16#10#, 16#00#, 16#80#, 16#10#, 16#00#,  
        16#40#, 16#00#, 16#C0#, 16#3C#, 16#08#, 16#00#, 16#42#, 16#08#,  
        16#00#, 16#40#, 16#00#, 16#00#, 16#07#, 16#84#, 16#00#, 16#00#,  
        16#08#, 16#02#, 16#20#, 16#00#, 16#00#, 16#00#, 16#7E#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#80#, 16#00#, 16#04#, 16#00#, 16#30#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#40#, 16#02#, 16#02#, 16#00#, 16#18#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#38#, 16#00#, 16#03#, 16#00#, 16#2C#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#01#, 16#00#, 16#24#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#00#, 16#10#, 16#90#,  
        16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#02#, 16#00#,  
        16#D0#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#81#,  
        16#00#, 16#20#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#40#, 16#80#, 16#20#, 16#41#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#20#, 16#88#, 16#30#, 16#87#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#10#, 16#00#, 16#18#, 16#04#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#0C#, 16#16#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#08#, 16#44#, 16#06#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#03#,  
        16#72#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#,  
        16#01#, 16#02#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#C2#, 16#03#, 16#81#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#40#, 16#06#, 16#40#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#38#, 16#20#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#18#, 16#1C#, 16#30#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#07#, 16#F8#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#08#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#20#,  
        16#0C#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#08#, 16#04#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#C0#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#07#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#1E#, 16#38#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#C0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Wdragon_80x96_Width  : constant := 80;  
    Wdragon_80x96_Height : constant := 96;  

    Wdragon_80x96_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0C#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#14#, 16#00#, 16#00#, 16#04#, 16#44#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#28#, 16#00#, 16#00#, 16#04#, 16#44#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#44#, 16#00#, 16#00#,  
        16#04#, 16#44#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#48#,  
        16#00#, 16#00#, 16#04#, 16#44#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#88#, 16#00#, 16#00#, 16#04#, 16#44#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#90#, 16#00#, 16#00#, 16#02#, 16#A8#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#10#, 16#00#, 16#00#,  
        16#02#, 16#A8#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#08#,  
        16#00#, 16#00#, 16#02#, 16#A8#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#08#, 16#00#, 16#00#, 16#02#, 16#A8#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#08#, 16#00#, 16#00#, 16#01#, 16#10#,  
        16#60#, 16#00#, 16#00#, 16#00#, 16#01#, 16#08#, 16#00#, 16#00#,  
        16#01#, 16#10#, 16#18#, 16#00#, 16#00#, 16#00#, 16#02#, 16#08#,  
        16#00#, 16#00#, 16#01#, 16#10#, 16#06#, 16#00#, 16#00#, 16#00#,  
        16#02#, 16#08#, 16#00#, 16#00#, 16#01#, 16#10#, 16#01#, 16#E0#,  
        16#00#, 16#00#, 16#02#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#10#, 16#00#, 16#00#, 16#02#, 16#04#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#07#, 16#C3#, 16#C0#, 16#03#, 16#04#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1E#, 16#3C#, 16#3F#,  
        16#FE#, 16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#E0#,  
        16#00#, 16#00#, 16#06#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#06#, 16#00#, 16#00#, 16#00#, 16#04#, 16#02#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#38#, 16#00#, 16#03#, 16#70#, 16#00#, 16#03#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#00#, 16#04#, 16#00#,  
        16#00#, 16#00#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#88#, 16#00#, 16#04#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#40#, 16#00#, 16#02#, 16#10#, 16#00#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#09#, 16#20#, 16#10#, 16#00#, 16#E0#, 16#80#,  
        16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#80#,  
        16#66#, 16#18#, 16#10#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#,  
        16#42#, 16#00#, 16#00#, 16#04#, 16#18#, 16#00#, 16#00#, 16#00#,  
        16#80#, 16#00#, 16#0C#, 16#00#, 16#00#, 16#02#, 16#08#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#90#, 16#07#, 16#00#, 16#02#,  
        16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#08#,  
        16#00#, 16#00#, 16#06#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#09#,  
        16#00#, 16#08#, 16#00#, 16#00#, 16#03#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#11#, 16#00#, 16#08#, 16#00#, 16#30#, 16#10#, 16#80#,  
        16#00#, 16#00#, 16#01#, 16#00#, 16#00#, 16#00#, 16#10#, 16#FE#,  
        16#10#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#31#, 16#88#, 16#40#, 16#00#, 16#00#, 16#04#, 16#00#,  
        16#00#, 16#20#, 16#20#, 16#10#, 16#4C#, 16#20#, 16#00#, 16#00#,  
        16#08#, 16#00#, 16#00#, 16#C0#, 16#00#, 16#10#, 16#24#, 16#10#,  
        16#00#, 16#00#, 16#08#, 16#00#, 16#01#, 16#00#, 16#80#, 16#90#,  
        16#32#, 16#10#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#01#,  
        16#01#, 16#90#, 16#18#, 16#08#, 16#00#, 16#00#, 16#10#, 16#00#,  
        16#00#, 16#00#, 16#02#, 16#08#, 16#04#, 16#04#, 16#00#, 16#00#,  
        16#10#, 16#01#, 16#18#, 16#00#, 16#02#, 16#08#, 16#3E#, 16#04#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#10#, 16#04#, 16#02#, 16#08#,  
        16#63#, 16#04#, 16#00#, 16#00#, 16#02#, 16#02#, 16#10#, 16#00#,  
        16#00#, 16#04#, 16#4F#, 16#04#, 16#00#, 16#00#, 16#04#, 16#22#,  
        16#00#, 16#00#, 16#00#, 16#82#, 16#9F#, 16#02#, 16#00#, 16#00#,  
        16#04#, 16#22#, 16#00#, 16#01#, 16#08#, 16#01#, 16#BF#, 16#02#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#, 16#04#, 16#00#,  
        16#BF#, 16#02#, 16#00#, 16#00#, 16#08#, 16#22#, 16#00#, 16#00#,  
        16#82#, 16#08#, 16#7F#, 16#12#, 16#00#, 16#00#, 16#08#, 16#01#,  
        16#02#, 16#08#, 16#01#, 16#80#, 16#3F#, 16#12#, 16#00#, 16#00#,  
        16#04#, 16#01#, 16#00#, 16#10#, 16#80#, 16#C4#, 16#0F#, 16#02#,  
        16#00#, 16#00#, 16#04#, 16#00#, 16#81#, 16#00#, 16#00#, 16#22#,  
        16#01#, 16#22#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#, 16#08#,  
        16#00#, 16#30#, 16#01#, 16#62#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#41#, 16#00#, 16#00#, 16#08#, 16#00#, 16#41#, 16#00#, 16#00#,  
        16#80#, 16#00#, 16#21#, 16#00#, 16#00#, 16#0C#, 16#00#, 16#01#,  
        16#00#, 16#00#, 16#40#, 16#10#, 16#21#, 16#06#, 16#00#, 16#06#,  
        16#20#, 16#01#, 16#80#, 16#00#, 16#00#, 16#00#, 16#10#, 16#02#,  
        16#00#, 16#01#, 16#88#, 16#00#, 16#40#, 16#00#, 16#28#, 16#1C#,  
        16#08#, 16#00#, 16#02#, 16#00#, 16#82#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#08#, 16#00#, 16#02#, 16#00#, 16#C1#, 16#00#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#02#, 16#0C#,  
        16#60#, 16#08#, 16#20#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#,  
        16#00#, 16#08#, 16#20#, 16#10#, 16#10#, 16#00#, 16#7F#, 16#F0#,  
        16#01#, 16#02#, 16#02#, 16#00#, 16#30#, 16#00#, 16#10#, 16#00#,  
        16#C0#, 16#0F#, 16#01#, 16#00#, 16#03#, 16#08#, 16#10#, 16#20#,  
        16#10#, 16#00#, 16#00#, 16#01#, 16#E0#, 16#C0#, 16#00#, 16#08#,  
        16#18#, 16#21#, 16#08#, 16#00#, 16#00#, 16#00#, 16#3C#, 16#41#,  
        16#01#, 16#00#, 16#08#, 16#40#, 16#04#, 16#00#, 16#00#, 16#00#,  
        16#03#, 16#E0#, 16#00#, 16#00#, 16#04#, 16#01#, 16#04#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#1C#, 16#00#, 16#00#, 16#04#, 16#00#,  
        16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#04#,  
        16#02#, 16#02#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#40#, 16#00#, 16#03#, 16#00#, 16#09#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#C0#, 16#04#, 16#01#, 16#00#, 16#08#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#78#, 16#00#, 16#10#, 16#88#,  
        16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#,  
        16#10#, 16#D8#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#04#, 16#04#, 16#00#, 16#60#, 16#10#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#06#, 16#02#, 16#00#, 16#10#, 16#00#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#01#, 16#00#, 16#10#,  
        16#10#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#81#,  
        16#00#, 16#18#, 16#60#, 16#60#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#40#, 16#08#, 16#0C#, 16#00#, 16#D0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#04#, 16#00#, 16#90#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#03#,  
        16#02#, 16#D0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0C#,  
        16#08#, 16#21#, 16#04#, 16#D0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#06#, 16#00#, 16#01#, 16#9C#, 16#50#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#00#, 16#80#, 16#50#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#86#, 16#00#,  
        16#60#, 16#30#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#C0#, 16#00#, 16#F0#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#40#, 16#03#, 16#98#, 16#08#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#60#, 16#1C#, 16#0C#, 16#08#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#30#, 16#0E#,  
        16#0C#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#10#, 16#03#, 16#FE#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0C#, 16#00#, 16#03#, 16#08#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#10#, 16#01#, 16#08#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#10#,  
        16#01#, 16#90#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#85#, 16#00#, 16#90#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#C0#, 16#40#, 16#90#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#30#, 16#00#, 16#F0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0E#,  
        16#00#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#FF#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

end Tile_Data_Wdragon;  

