with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_Winter is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_winter - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----SEASONS  
------------------------------------------------------------------------------  

    Winter_28x32_Width  : constant := 28;  
    Winter_28x32_Height : constant := 32;  

    Winter_28x32_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#03#, 16#F3#, 16#F8#, 16#00#, 16#06#, 16#7F#, 16#CC#, 16#00#,  
        16#0C#, 16#2E#, 16#86#, 16#00#, 16#1A#, 16#3B#, 16#8B#, 16#00#,  
        16#11#, 16#4A#, 16#51#, 16#00#, 16#10#, 16#4A#, 16#41#, 16#00#,  
        16#19#, 16#FF#, 16#F3#, 16#00#, 16#16#, 16#6E#, 16#CD#, 16#00#,  
        16#1E#, 16#FF#, 16#EF#, 16#00#, 16#0B#, 16#4B#, 16#5A#, 16#00#,  
        16#0B#, 16#5B#, 16#5A#, 16#00#, 16#1E#, 16#FF#, 16#EF#, 16#00#,  
        16#16#, 16#6E#, 16#CD#, 16#00#, 16#19#, 16#FF#, 16#F7#, 16#00#,  
        16#10#, 16#4A#, 16#41#, 16#00#, 16#11#, 16#4A#, 16#51#, 16#00#,  
        16#1A#, 16#3B#, 16#89#, 16#00#, 16#0C#, 16#2E#, 16#87#, 16#00#,  
        16#06#, 16#6E#, 16#C6#, 16#00#, 16#03#, 16#FB#, 16#FC#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#4B#, 16#A5#, 16#DD#, 16#C0#,  
        16#49#, 16#34#, 16#91#, 16#20#, 16#49#, 16#34#, 16#99#, 16#20#,  
        16#79#, 16#2C#, 16#91#, 16#C0#, 16#79#, 16#2C#, 16#91#, 16#20#,  
        16#4B#, 16#A4#, 16#9D#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Winter_40x48_Width  : constant := 40;  
    Winter_40x48_Height : constant := 48;  

    Winter_40x48_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#7F#, 16#C7#, 16#FC#, 16#00#, 16#00#, 16#80#,  
        16#28#, 16#02#, 16#00#, 16#01#, 16#00#, 16#7C#, 16#01#, 16#00#,  
        16#02#, 16#81#, 16#AB#, 16#02#, 16#80#, 16#04#, 16#42#, 16#44#,  
        16#84#, 16#40#, 16#04#, 16#22#, 16#44#, 16#88#, 16#40#, 16#04#,  
        16#14#, 16#44#, 16#50#, 16#40#, 16#04#, 16#04#, 16#44#, 16#40#,  
        16#40#, 16#04#, 16#1F#, 16#7D#, 16#F0#, 16#40#, 16#04#, 16#65#,  
        16#C7#, 16#4C#, 16#40#, 16#04#, 16#86#, 16#6C#, 16#C2#, 16#40#,  
        16#04#, 16#82#, 16#6C#, 16#82#, 16#40#, 16#05#, 16#7F#, 16#D7#,  
        16#FD#, 16#40#, 16#03#, 16#85#, 16#BB#, 16#43#, 16#80#, 16#01#,  
        16#04#, 16#6C#, 16#41#, 16#00#, 16#03#, 16#85#, 16#BB#, 16#43#,  
        16#80#, 16#05#, 16#7F#, 16#D7#, 16#FD#, 16#40#, 16#04#, 16#82#,  
        16#6C#, 16#82#, 16#40#, 16#04#, 16#86#, 16#6C#, 16#C2#, 16#40#,  
        16#04#, 16#65#, 16#C7#, 16#4C#, 16#40#, 16#04#, 16#1F#, 16#7D#,  
        16#F0#, 16#40#, 16#04#, 16#04#, 16#44#, 16#40#, 16#40#, 16#04#,  
        16#14#, 16#44#, 16#50#, 16#40#, 16#04#, 16#22#, 16#44#, 16#88#,  
        16#40#, 16#04#, 16#42#, 16#44#, 16#84#, 16#40#, 16#02#, 16#81#,  
        16#AB#, 16#02#, 16#80#, 16#01#, 16#00#, 16#7C#, 16#01#, 16#00#,  
        16#00#, 16#80#, 16#28#, 16#02#, 16#00#, 16#00#, 16#7F#, 16#C7#,  
        16#FC#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#22#, 16#FA#, 16#2F#, 16#BE#, 16#F0#, 16#22#, 16#23#, 16#22#,  
        16#20#, 16#88#, 16#2A#, 16#23#, 16#22#, 16#20#, 16#88#, 16#2A#,  
        16#22#, 16#A2#, 16#38#, 16#F0#, 16#36#, 16#22#, 16#A2#, 16#20#,  
        16#90#, 16#36#, 16#22#, 16#62#, 16#20#, 16#88#, 16#22#, 16#FA#,  
        16#62#, 16#3E#, 16#88#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Winter_56x64_Width  : constant := 56;  
    Winter_56x64_Height : constant := 64;  

    Winter_56x64_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#3F#, 16#E3#, 16#FE#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#40#, 16#14#, 16#01#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#80#, 16#3E#, 16#00#, 16#80#, 16#00#, 16#00#, 16#01#, 16#40#,  
        16#D5#, 16#81#, 16#40#, 16#00#, 16#00#, 16#02#, 16#21#, 16#22#,  
        16#42#, 16#20#, 16#00#, 16#00#, 16#02#, 16#11#, 16#22#, 16#44#,  
        16#20#, 16#00#, 16#00#, 16#02#, 16#0A#, 16#22#, 16#28#, 16#20#,  
        16#00#, 16#00#, 16#02#, 16#02#, 16#22#, 16#20#, 16#20#, 16#00#,  
        16#00#, 16#02#, 16#0F#, 16#BE#, 16#F8#, 16#20#, 16#00#, 16#00#,  
        16#02#, 16#32#, 16#E3#, 16#A6#, 16#20#, 16#00#, 16#00#, 16#02#,  
        16#43#, 16#36#, 16#61#, 16#20#, 16#00#, 16#00#, 16#02#, 16#41#,  
        16#36#, 16#41#, 16#20#, 16#00#, 16#00#, 16#02#, 16#BF#, 16#EB#,  
        16#FE#, 16#A0#, 16#00#, 16#00#, 16#01#, 16#C2#, 16#DD#, 16#A1#,  
        16#C0#, 16#00#, 16#00#, 16#00#, 16#82#, 16#36#, 16#20#, 16#80#,  
        16#00#, 16#00#, 16#01#, 16#C2#, 16#DD#, 16#A1#, 16#C0#, 16#00#,  
        16#00#, 16#02#, 16#BF#, 16#EB#, 16#FE#, 16#A0#, 16#00#, 16#00#,  
        16#02#, 16#41#, 16#36#, 16#41#, 16#20#, 16#00#, 16#00#, 16#02#,  
        16#43#, 16#36#, 16#61#, 16#20#, 16#00#, 16#00#, 16#02#, 16#32#,  
        16#E3#, 16#A6#, 16#20#, 16#00#, 16#00#, 16#02#, 16#0F#, 16#BE#,  
        16#F8#, 16#20#, 16#00#, 16#00#, 16#02#, 16#02#, 16#22#, 16#20#,  
        16#20#, 16#00#, 16#00#, 16#02#, 16#0A#, 16#22#, 16#28#, 16#20#,  
        16#00#, 16#00#, 16#02#, 16#11#, 16#22#, 16#44#, 16#20#, 16#00#,  
        16#00#, 16#02#, 16#21#, 16#22#, 16#42#, 16#20#, 16#00#, 16#00#,  
        16#01#, 16#40#, 16#D5#, 16#81#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#80#, 16#3E#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#40#,  
        16#14#, 16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3F#, 16#E3#,  
        16#FE#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#08#, 16#2F#, 16#EC#, 16#2F#, 16#EF#, 16#EF#,  
        16#80#, 16#08#, 16#21#, 16#0C#, 16#21#, 16#08#, 16#08#, 16#40#,  
        16#09#, 16#21#, 16#0A#, 16#21#, 16#08#, 16#08#, 16#20#, 16#09#,  
        16#21#, 16#0A#, 16#21#, 16#08#, 16#08#, 16#20#, 16#09#, 16#21#,  
        16#09#, 16#21#, 16#08#, 16#08#, 16#20#, 16#0A#, 16#A1#, 16#09#,  
        16#21#, 16#0F#, 16#08#, 16#40#, 16#0A#, 16#A1#, 16#08#, 16#A1#,  
        16#08#, 16#0F#, 16#80#, 16#0E#, 16#61#, 16#08#, 16#A1#, 16#08#,  
        16#08#, 16#40#, 16#0C#, 16#61#, 16#08#, 16#61#, 16#08#, 16#08#,  
        16#40#, 16#0C#, 16#61#, 16#08#, 16#61#, 16#08#, 16#08#, 16#20#,  
        16#08#, 16#2F#, 16#E8#, 16#61#, 16#0F#, 16#E8#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Winter_68x80_Width  : constant := 68;  
    Winter_68x80_Height : constant := 80;  

    Winter_68x80_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#FF#,  
        16#8F#, 16#F8#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#00#, 16#50#, 16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#02#, 16#00#, 16#F8#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#05#, 16#03#, 16#56#, 16#05#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#08#, 16#84#, 16#89#, 16#08#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#44#, 16#89#, 16#10#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#28#, 16#88#, 16#A0#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#08#, 16#88#,  
        16#80#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#3E#,  
        16#FB#, 16#E0#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#,  
        16#CB#, 16#8E#, 16#98#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#09#, 16#0C#, 16#D9#, 16#84#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#09#, 16#04#, 16#D9#, 16#04#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0A#, 16#FF#, 16#AF#, 16#FA#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#07#, 16#0B#, 16#76#, 16#87#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#08#, 16#D8#, 16#82#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#0B#, 16#76#,  
        16#87#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0A#, 16#FF#,  
        16#AF#, 16#FA#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#09#,  
        16#04#, 16#D9#, 16#04#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#09#, 16#0C#, 16#D9#, 16#84#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#08#, 16#CB#, 16#8E#, 16#98#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#08#, 16#3E#, 16#FB#, 16#E0#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#08#, 16#88#, 16#80#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#28#, 16#88#, 16#A0#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#44#, 16#89#,  
        16#10#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#84#,  
        16#89#, 16#08#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#05#,  
        16#03#, 16#56#, 16#05#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#02#, 16#00#, 16#F8#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#00#, 16#50#, 16#04#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#FF#, 16#8F#, 16#F8#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#22#, 16#27#, 16#FC#, 16#C0#, 16#9F#, 16#F3#, 16#FE#, 16#7F#,  
        16#00#, 16#22#, 16#20#, 16#40#, 16#C0#, 16#81#, 16#02#, 16#00#,  
        16#40#, 16#80#, 16#22#, 16#20#, 16#40#, 16#A0#, 16#81#, 16#02#,  
        16#00#, 16#40#, 16#40#, 16#22#, 16#20#, 16#40#, 16#A0#, 16#81#,  
        16#02#, 16#00#, 16#40#, 16#40#, 16#22#, 16#20#, 16#40#, 16#90#,  
        16#81#, 16#02#, 16#00#, 16#40#, 16#40#, 16#15#, 16#40#, 16#40#,  
        16#90#, 16#81#, 16#02#, 16#00#, 16#40#, 16#40#, 16#15#, 16#40#,  
        16#40#, 16#88#, 16#81#, 16#03#, 16#E0#, 16#40#, 16#80#, 16#15#,  
        16#40#, 16#40#, 16#88#, 16#81#, 16#02#, 16#00#, 16#7F#, 16#00#,  
        16#15#, 16#40#, 16#40#, 16#84#, 16#81#, 16#02#, 16#00#, 16#41#,  
        16#00#, 16#08#, 16#80#, 16#40#, 16#84#, 16#81#, 16#02#, 16#00#,  
        16#40#, 16#80#, 16#08#, 16#80#, 16#40#, 16#82#, 16#81#, 16#02#,  
        16#00#, 16#40#, 16#80#, 16#08#, 16#80#, 16#40#, 16#82#, 16#81#,  
        16#02#, 16#00#, 16#40#, 16#40#, 16#08#, 16#87#, 16#FC#, 16#81#,  
        16#81#, 16#03#, 16#FE#, 16#40#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Winter_80x96_Width  : constant := 80;  
    Winter_80x96_Height : constant := 96;  

    Winter_80x96_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#FE#, 16#3F#, 16#E0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#04#, 16#01#, 16#40#, 16#10#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#03#, 16#E0#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#14#,  
        16#0D#, 16#58#, 16#14#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#22#, 16#12#, 16#24#, 16#22#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#21#, 16#12#, 16#24#, 16#42#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#A2#, 16#22#,  
        16#82#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#,  
        16#22#, 16#22#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#20#, 16#FB#, 16#EF#, 16#82#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#23#, 16#2E#, 16#3A#, 16#62#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#24#, 16#33#, 16#66#,  
        16#12#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#24#,  
        16#13#, 16#64#, 16#12#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#2B#, 16#FE#, 16#BF#, 16#EA#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#1C#, 16#2D#, 16#DA#, 16#1C#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#23#, 16#62#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1C#,  
        16#2D#, 16#DA#, 16#1C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#2B#, 16#FE#, 16#BF#, 16#EA#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#24#, 16#13#, 16#64#, 16#12#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#24#, 16#33#, 16#66#,  
        16#12#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#23#,  
        16#2E#, 16#3A#, 16#62#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#20#, 16#FB#, 16#EF#, 16#82#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#20#, 16#22#, 16#22#, 16#02#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#A2#, 16#22#,  
        16#82#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#21#,  
        16#12#, 16#24#, 16#42#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#22#, 16#12#, 16#24#, 16#22#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#14#, 16#0D#, 16#58#, 16#14#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#03#, 16#E0#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#,  
        16#01#, 16#40#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#FE#, 16#3F#, 16#E0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#44#,  
        16#7F#, 16#C6#, 16#04#, 16#7F#, 16#C7#, 16#FC#, 16#7F#, 16#00#,  
        16#04#, 16#44#, 16#04#, 16#06#, 16#04#, 16#04#, 16#04#, 16#00#,  
        16#40#, 16#80#, 16#04#, 16#44#, 16#04#, 16#05#, 16#04#, 16#04#,  
        16#04#, 16#00#, 16#40#, 16#40#, 16#04#, 16#44#, 16#04#, 16#05#,  
        16#04#, 16#04#, 16#04#, 16#00#, 16#40#, 16#40#, 16#04#, 16#44#,  
        16#04#, 16#04#, 16#84#, 16#04#, 16#04#, 16#00#, 16#40#, 16#40#,  
        16#02#, 16#A8#, 16#04#, 16#04#, 16#84#, 16#04#, 16#04#, 16#00#,  
        16#40#, 16#40#, 16#02#, 16#A8#, 16#04#, 16#04#, 16#44#, 16#04#,  
        16#07#, 16#C0#, 16#40#, 16#80#, 16#02#, 16#A8#, 16#04#, 16#04#,  
        16#44#, 16#04#, 16#04#, 16#00#, 16#7F#, 16#00#, 16#02#, 16#A8#,  
        16#04#, 16#04#, 16#24#, 16#04#, 16#04#, 16#00#, 16#41#, 16#00#,  
        16#01#, 16#10#, 16#04#, 16#04#, 16#24#, 16#04#, 16#04#, 16#00#,  
        16#40#, 16#80#, 16#01#, 16#10#, 16#04#, 16#04#, 16#14#, 16#04#,  
        16#04#, 16#00#, 16#40#, 16#80#, 16#01#, 16#10#, 16#04#, 16#04#,  
        16#14#, 16#04#, 16#04#, 16#00#, 16#40#, 16#40#, 16#01#, 16#10#,  
        16#7F#, 16#C4#, 16#0C#, 16#04#, 16#07#, 16#FC#, 16#40#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  
end Tile_Data_Winter;  

