with Main;  
use Main;  

with Tile_Data_Spring;  
use Tile_Data_Spring;  
with Tile_Data_Summer;  
use Tile_Data_Summer;  
with Tile_Data_Fall;  
use Tile_Data_Fall;  
with Tile_Data_Winter;  
use Tile_Data_Winter;  

with Tile_Data_Bamboo;  
use Tile_Data_Bamboo;  
with Tile_Data_Mum;  
use Tile_Data_Mum;  
with Tile_Data_Orchid;  
use Tile_Data_Orchid;  
with Tile_Data_Plum;  
use Tile_Data_Plum;  

with Tile_Data_Gdragon;  
use Tile_Data_Gdragon;  
with Tile_Data_Rdragon;  
use Tile_Data_Rdragon;  
with Tile_Data_Wdragon;  
use Tile_Data_Wdragon;  

with Tile_Data_East;  
use Tile_Data_East;  
with Tile_Data_West;  
use Tile_Data_West;  
with Tile_Data_North;  
use Tile_Data_North;  
with Tile_Data_South;  
use Tile_Data_South;  

with Tile_Data_Numbers;  
use Tile_Data_Numbers;  
with Tile_Data_Other;  
use Tile_Data_Other;  

with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Image;  
use Xlbt_Image;  
with Xlbt_Visual;  
use Xlbt_Visual;  

with Xlbp_Image;  
use Xlbp_Image;  

package body Tile_Bits is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_bits - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- XImage initialization structures  
------------------------------------------------------------------------------  

    procedure Setup_Images (Size : Natural) is  

        procedure New_Image (Image  : out X_Image;  
                             Data   :     U_Char_Array;  
                             Width  :     U_Short;  
                             Height :     U_Short) is  
            Img : X_Image;  
        begin  
            Img := X_Create_Image  
                      (Display              => Dpy,  
                       Visual               => None_X_Visual,  
                       Depth                => 1,  
                       Format               => X_Y_Bitmap,  
                       Offset               => 0,  
                       Data                 => Data,  
                       Width                => Width,  
                       Height               => Height,  
                       Bitmap_Pad           => 8,  
                       Image_Bytes_Per_Line => (Width + 7) / 8);  
            -- Img.Format := Z_Pixmap;  
            -- Img.Byte_Order       := Lsb_First;  
            -- Img.Bitmap_Bit_Order := Lsb_First;  
            Image := Img;  
        end New_Image;  

    begin  

----Get rid of any existing images.  

        X_Destroy_Image (Spring);  
        X_Destroy_Image (Summer);  
        X_Destroy_Image (Fall);  
        X_Destroy_Image (Winter);  
        X_Destroy_Image (Bamboo);  
        X_Destroy_Image (Mum);  
        X_Destroy_Image (Orchid);  
        X_Destroy_Image (Plum);  
        X_Destroy_Image (Gdragon);  
        X_Destroy_Image (Rdragon);  
        X_Destroy_Image (Wdragon);  
        X_Destroy_Image (East);  
        X_Destroy_Image (West);  
        X_Destroy_Image (North);  
        X_Destroy_Image (South);  
        X_Destroy_Image (One);  
        X_Destroy_Image (Two);  
        X_Destroy_Image (Three);  
        X_Destroy_Image (Four);  
        X_Destroy_Image (Five);  
        X_Destroy_Image (Six);  
        X_Destroy_Image (Seven);  
        X_Destroy_Image (Eight);  
        X_Destroy_Image (Nine);  
        X_Destroy_Image (Bam);  
        X_Destroy_Image (Crak);  
        X_Destroy_Image (Dot);  

----Create a new set of images of the appropriate size.  

        case Size is  
            when 0 =>  
                New_Image (Spring, Spring_28x32_Bits,  
                           Spring_28x32_Width, Spring_28x32_Height);  
                New_Image (Summer, Summer_28x32_Bits,  
                           Summer_28x32_Width, Summer_28x32_Height);  
                New_Image (Fall, Fall_28x32_Bits,  
                           Fall_28x32_Width, Fall_28x32_Height);  
                New_Image (Winter, Winter_28x32_Bits,  
                           Winter_28x32_Width, Winter_28x32_Height);  
                New_Image (Bamboo, Bamboo_28x32_Bits,  
                           Bamboo_28x32_Width, Bamboo_28x32_Height);  
                New_Image (Mum, Mum_28x32_Bits,  
                           Mum_28x32_Width, Mum_28x32_Height);  
                New_Image (Orchid, Orchid_28x32_Bits,  
                           Orchid_28x32_Width, Orchid_28x32_Height);  
                New_Image (Plum, Plum_28x32_Bits,  
                           Plum_28x32_Width, Plum_28x32_Height);  
                New_Image (Gdragon, Gdragon_28x32_Bits,  
                           Gdragon_28x32_Width, Gdragon_28x32_Height);  
                New_Image (Rdragon, Rdragon_28x32_Bits,  
                           Rdragon_28x32_Width, Rdragon_28x32_Height);  
                New_Image (Wdragon, Wdragon_28x32_Bits,  
                           Wdragon_28x32_Width, Wdragon_28x32_Height);  
                New_Image (East, East_28x32_Bits,  
                           East_28x32_Width, East_28x32_Height);  
                New_Image (West, West_28x32_Bits,  
                           West_28x32_Width, West_28x32_Height);  
                New_Image (North, North_28x32_Bits,  
                           North_28x32_Width, North_28x32_Height);  
                New_Image (South, South_28x32_Bits,  
                           South_28x32_Width, South_28x32_Height);  
                New_Image (One, One_7x8_Bits, One_7x8_Width, One_7x8_Height);  
                New_Image (Two, Two_7x8_Bits, Two_7x8_Width, Two_7x8_Height);  
                New_Image (Three, Three_7x8_Bits,  
                           Three_7x8_Width, Three_7x8_Height);  
                New_Image (Four, Four_7x8_Bits,  
                           Four_7x8_Width, Four_7x8_Height);  
                New_Image (Five, Five_7x8_Bits,  
                           Five_7x8_Width, Five_7x8_Height);  
                New_Image (Six, Six_7x8_Bits, Six_7x8_Width, Six_7x8_Height);  
                New_Image (Seven, Seven_7x8_Bits,  
                           Seven_7x8_Width, Seven_7x8_Height);  
                New_Image (Eight, Eight_7x8_Bits,  
                           Eight_7x8_Width, Eight_7x8_Height);  
                New_Image (Nine, Nine_7x8_Bits,  
                           Nine_7x8_Width, Nine_7x8_Height);  
                New_Image (Bam, Bam_7x8_Bits, Bam_7x8_Width, Bam_7x8_Height);  
                New_Image (Crak, Crak_28x32_Bits,  
                           Crak_28x32_Width, Crak_28x32_Height);  
                New_Image (Dot, Dot_7x8_Bits, Dot_7x8_Width, Dot_7x8_Height);  

            when 1 =>  
                New_Image (Spring, Spring_40x48_Bits,  
                           Spring_40x48_Width, Spring_40x48_Height);  
                New_Image (Summer, Summer_40x48_Bits,  
                           Summer_40x48_Width, Summer_40x48_Height);  
                New_Image (Fall, Fall_40x48_Bits,  
                           Fall_40x48_Width, Fall_40x48_Height);  
                New_Image (Winter, Winter_40x48_Bits,  
                           Winter_40x48_Width, Winter_40x48_Height);  
                New_Image (Bamboo, Bamboo_40x48_Bits,  
                           Bamboo_40x48_Width, Bamboo_40x48_Height);  
                New_Image (Mum, Mum_40x48_Bits,  
                           Mum_40x48_Width, Mum_40x48_Height);  
                New_Image (Orchid, Orchid_40x48_Bits,  
                           Orchid_40x48_Width, Orchid_40x48_Height);  
                New_Image (Plum, Plum_40x48_Bits,  
                           Plum_40x48_Width, Plum_40x48_Height);  
                New_Image (Gdragon, Gdragon_40x48_Bits,  
                           Gdragon_40x48_Width, Gdragon_40x48_Height);  
                New_Image (Rdragon, Rdragon_40x48_Bits,  
                           Rdragon_40x48_Width, Rdragon_40x48_Height);  
                New_Image (Wdragon, Wdragon_40x48_Bits,  
                           Wdragon_40x48_Width, Wdragon_40x48_Height);  
                New_Image (East, East_40x48_Bits,  
                           East_40x48_Width, East_40x48_Height);  
                New_Image (West, West_40x48_Bits,  
                           West_40x48_Width, West_40x48_Height);  
                New_Image (North, North_40x48_Bits,  
                           North_40x48_Width, North_40x48_Height);  
                New_Image (South, South_40x48_Bits,  
                           South_40x48_Width, South_40x48_Height);  
                New_Image (One, One_10x12_Bits,  
                           One_10x12_Width, One_10x12_Height);  
                New_Image (Two, Two_10x12_Bits,  
                           Two_10x12_Width, Two_10x12_Height);  
                New_Image (Three, Three_10x12_Bits,  
                           Three_10x12_Width, Three_10x12_Height);  
                New_Image (Four, Four_10x12_Bits,  
                           Four_10x12_Width, Four_10x12_Height);  
                New_Image (Five, Five_10x12_Bits,  
                           Five_10x12_Width, Five_10x12_Height);  
                New_Image (Six, Six_10x12_Bits,  
                           Six_10x12_Width, Six_10x12_Height);  
                New_Image (Seven, Seven_10x12_Bits,  
                           Seven_10x12_Width, Seven_10x12_Height);  
                New_Image (Eight, Eight_10x12_Bits,  
                           Eight_10x12_Width, Eight_10x12_Height);  
                New_Image (Nine, Nine_10x12_Bits,  
                           Nine_10x12_Width, Nine_10x12_Height);  
                New_Image (Bam, Bam_10x12_Bits,  
                           Bam_10x12_Width, Bam_10x12_Height);  
                New_Image (Crak, Crak_40x48_Bits,  
                           Crak_40x48_Width, Crak_40x48_Height);  
                New_Image (Dot, Dot_10x12_Bits,  
                           Dot_10x12_Width, Dot_10x12_Height);  

            when 2 =>  
                New_Image (Spring, Spring_56x64_Bits,  
                           Spring_56x64_Width, Spring_56x64_Height);  
                New_Image (Summer, Summer_56x64_Bits,  
                           Summer_56x64_Width, Summer_56x64_Height);  
                New_Image (Fall, Fall_56x64_Bits,  
                           Fall_56x64_Width, Fall_56x64_Height);  
                New_Image (Winter, Winter_56x64_Bits,  
                           Winter_56x64_Width, Winter_56x64_Height);  
                New_Image (Bamboo, Bamboo_56x64_Bits,  
                           Bamboo_56x64_Width, Bamboo_56x64_Height);  
                New_Image (Mum, Mum_56x64_Bits,  
                           Mum_56x64_Width, Mum_56x64_Height);  
                New_Image (Orchid, Orchid_56x64_Bits,  
                           Orchid_56x64_Width, Orchid_56x64_Height);  
                New_Image (Plum, Plum_56x64_Bits,  
                           Plum_56x64_Width, Plum_56x64_Height);  
                New_Image (Gdragon, Gdragon_56x64_Bits,  
                           Gdragon_56x64_Width, Gdragon_56x64_Height);  
                New_Image (Rdragon, Rdragon_56x64_Bits,  
                           Rdragon_56x64_Width, Rdragon_56x64_Height);  
                New_Image (Wdragon, Wdragon_56x64_Bits,  
                           Wdragon_56x64_Width, Wdragon_56x64_Height);  
                New_Image (East, East_56x64_Bits,  
                           East_56x64_Width, East_56x64_Height);  
                New_Image (West, West_56x64_Bits,  
                           West_56x64_Width, West_56x64_Height);  
                New_Image (North, North_56x64_Bits,  
                           North_56x64_Width, North_56x64_Height);  
                New_Image (South, South_56x64_Bits,  
                           South_56x64_Width, South_56x64_Height);  
                New_Image (One, One_14x16_Bits,  
                           One_14x16_Width, One_14x16_Height);  
                New_Image (Two, Two_14x16_Bits,  
                           Two_14x16_Width, Two_14x16_Height);  
                New_Image (Three, Three_14x16_Bits,  
                           Three_14x16_Width, Three_14x16_Height);  
                New_Image (Four, Four_14x16_Bits,  
                           Four_14x16_Width, Four_14x16_Height);  
                New_Image (Five, Five_14x16_Bits,  
                           Five_14x16_Width, Five_14x16_Height);  
                New_Image (Six, Six_14x16_Bits,  
                           Six_14x16_Width, Six_14x16_Height);  
                New_Image (Seven, Seven_14x16_Bits,  
                           Seven_14x16_Width, Seven_14x16_Height);  
                New_Image (Eight, Eight_14x16_Bits,  
                           Eight_14x16_Width, Eight_14x16_Height);  
                New_Image (Nine, Nine_14x16_Bits,  
                           Nine_14x16_Width, Nine_14x16_Height);  
                New_Image (Bam, Bam_14x16_Bits,  
                           Bam_14x16_Width, Bam_14x16_Height);  
                New_Image (Crak, Crak_56x64_Bits,  
                           Crak_56x64_Width, Crak_56x64_Height);  
                New_Image (Dot, Dot_14x16_Bits,  
                           Dot_14x16_Width, Dot_14x16_Height);  

            when 3 =>  
                New_Image (Spring, Spring_68x80_Bits,  
                           Spring_68x80_Width, Spring_68x80_Height);  
                New_Image (Summer, Summer_68x80_Bits,  
                           Summer_68x80_Width, Summer_68x80_Height);  
                New_Image (Fall, Fall_68x80_Bits,  
                           Fall_68x80_Width, Fall_68x80_Height);  
                New_Image (Winter, Winter_68x80_Bits,  
                           Winter_68x80_Width, Winter_68x80_Height);  
                New_Image (Bamboo, Bamboo_68x80_Bits,  
                           Bamboo_68x80_Width, Bamboo_68x80_Height);  
                New_Image (Mum, Mum_68x80_Bits,  
                           Mum_68x80_Width, Mum_68x80_Height);  
                New_Image (Orchid, Orchid_68x80_Bits,  
                           Orchid_68x80_Width, Orchid_68x80_Height);  
                New_Image (Plum, Plum_68x80_Bits,  
                           Plum_68x80_Width, Plum_68x80_Height);  
                New_Image (Gdragon, Gdragon_68x80_Bits,  
                           Gdragon_68x80_Width, Gdragon_68x80_Height);  
                New_Image (Rdragon, Rdragon_68x80_Bits,  
                           Rdragon_68x80_Width, Rdragon_68x80_Height);  
                New_Image (Wdragon, Wdragon_68x80_Bits,  
                           Wdragon_68x80_Width, Wdragon_68x80_Height);  
                New_Image (East, East_68x80_Bits,  
                           East_68x80_Width, East_68x80_Height);  
                New_Image (West, West_68x80_Bits,  
                           West_68x80_Width, West_68x80_Height);  
                New_Image (North, North_68x80_Bits,  
                           North_68x80_Width, North_68x80_Height);  
                New_Image (South, South_68x80_Bits,  
                           South_68x80_Width, South_68x80_Height);  
                New_Image (One, One_17x20_Bits,  
                           One_17x20_Width, One_17x20_Height);  
                New_Image (Two, Two_17x20_Bits,  
                           Two_17x20_Width, Two_17x20_Height);  
                New_Image (Three, Three_17x20_Bits,  
                           Three_17x20_Width, Three_17x20_Height);  
                New_Image (Four, Four_17x20_Bits,  
                           Four_17x20_Width, Four_17x20_Height);  
                New_Image (Five, Five_17x20_Bits,  
                           Five_17x20_Width, Five_17x20_Height);  
                New_Image (Six, Six_17x20_Bits,  
                           Six_17x20_Width, Six_17x20_Height);  
                New_Image (Seven, Seven_17x20_Bits,  
                           Seven_17x20_Width, Seven_17x20_Height);  
                New_Image (Eight, Eight_17x20_Bits,  
                           Eight_17x20_Width, Eight_17x20_Height);  
                New_Image (Nine, Nine_17x20_Bits,  
                           Nine_17x20_Width, Nine_17x20_Height);  
                New_Image (Bam, Bam_17x20_Bits,  
                           Bam_17x20_Width, Bam_17x20_Height);  
                New_Image (Crak, Crak_68x80_Bits,  
                           Crak_68x80_Width, Crak_68x80_Height);  
                New_Image (Dot, Dot_17x20_Bits,  
                           Dot_17x20_Width, Dot_17x20_Height);  

            when 4 =>  
                New_Image (Spring, Spring_80x96_Bits,  
                           Spring_80x96_Width, Spring_80x96_Height);  
                New_Image (Summer, Summer_80x96_Bits,  
                           Summer_80x96_Width, Summer_80x96_Height);  
                New_Image (Fall, Fall_80x96_Bits,  
                           Fall_80x96_Width, Fall_80x96_Height);  
                New_Image (Winter, Winter_80x96_Bits,  
                           Winter_80x96_Width, Winter_80x96_Height);  
                New_Image (Bamboo, Bamboo_80x96_Bits,  
                           Bamboo_80x96_Width, Bamboo_80x96_Height);  
                New_Image (Mum, Mum_80x96_Bits,  
                           Mum_80x96_Width, Mum_80x96_Height);  
                New_Image (Orchid, Orchid_80x96_Bits,  
                           Orchid_80x96_Width, Orchid_80x96_Height);  
                New_Image (Plum, Plum_80x96_Bits,  
                           Plum_80x96_Width, Plum_80x96_Height);  
                New_Image (Gdragon, Gdragon_80x96_Bits,  
                           Gdragon_80x96_Width, Gdragon_80x96_Height);  
                New_Image (Rdragon, Rdragon_80x96_Bits,  
                           Rdragon_80x96_Width, Rdragon_80x96_Height);  
                New_Image (Wdragon, Wdragon_80x96_Bits,  
                           Wdragon_80x96_Width, Wdragon_80x96_Height);  
                New_Image (East, East_80x96_Bits,  
                           East_80x96_Width, East_80x96_Height);  
                New_Image (West, West_80x96_Bits,  
                           West_80x96_Width, West_80x96_Height);  
                New_Image (North, North_80x96_Bits,  
                           North_80x96_Width, North_80x96_Height);  
                New_Image (South, South_80x96_Bits,  
                           South_80x96_Width, South_80x96_Height);  
                New_Image (One, One_20x24_Bits,  
                           One_20x24_Width, One_20x24_Height);  
                New_Image (Two, Two_20x24_Bits,  
                           Two_20x24_Width, Two_20x24_Height);  
                New_Image (Three, Three_20x24_Bits,  
                           Three_20x24_Width, Three_20x24_Height);  
                New_Image (Four, Four_20x24_Bits,  
                           Four_20x24_Width, Four_20x24_Height);  
                New_Image (Five, Five_20x24_Bits,  
                           Five_20x24_Width, Five_20x24_Height);  
                New_Image (Six, Six_20x24_Bits,  
                           Six_20x24_Width, Six_20x24_Height);  
                New_Image (Seven, Seven_20x24_Bits,  
                           Seven_20x24_Width, Seven_20x24_Height);  
                New_Image (Eight, Eight_20x24_Bits,  
                           Eight_20x24_Width, Eight_20x24_Height);  
                New_Image (Nine, Nine_20x24_Bits,  
                           Nine_20x24_Width, Nine_20x24_Height);  
                New_Image (Bam, Bam_20x24_Bits,  
                           Bam_20x24_Width, Bam_20x24_Height);  
                New_Image (Crak, Crak_80x96_Bits,  
                           Crak_80x96_Width, Crak_80x96_Height);  
                New_Image (Dot, Dot_20x24_Bits,  
                           Dot_20x24_Width, Dot_20x24_Height);  

            when others =>  
                raise Constraint_Error;  
        end case;  

    end Setup_Images;  

--
end Tile_Bits;  

