with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Event;  
use Xlbt_Event;  
with Xlbt_Host;  
use Xlbt_Host;  
with Xlbt_String;  
use Xlbt_String;  

with Xlbmt_Network_Types;  
use Xlbmt_Network_Types;  

package Xlbmp_Internal is  
------------------------------------------------------------------------------  
-- X Library Machine Dependent Internal Support  
--  
-- Xlbmp_Internal - Very low-level routines for internal X Library support  
-- These should not be called by anyone other than X Library routines.  Not  
-- even Extensions should have cause to use these.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  
--
    function Internal_X_Events_Queued (Display : X_Display;  
                                       Mode    : X_Event_Queuing) return S_Long;  
------------------------------------------------------------------------------  
-- Read some new events; maybe flush before doing so.  All public event  
-- queries are based upon this routine.  Assumes that the display is locked.  
------------------------------------------------------------------------------  

    procedure Internal_X_Flush_Display (Display : X_Display);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to flush  
--  
-- Flush any buffered messages.  They are sent immediately.  If writing to the  
-- network would block then we attempt to read incoming events in between write  
-- attempts.  
------------------------------------------------------------------------------  

    procedure Internal_X_Read_Events (Display : X_Display);  
------------------------------------------------------------------------------  
-- Don't return until we read and queue up at least one new event.  Just like  
-- Pending_Events except that we don't return a count and we don't return  
-- until we've read at least one.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Routines for establishing the initial connection or terminating it.  
------------------------------------------------------------------------------  

    procedure Internal_X_Connect_Display  
                 (Display_Name  :        X_String;  
                  Expanded_Name : out    X_String_Pointer;  
                  Display_Num   : out    U_Char;  
                  Screen_Num    : out    X_Screen_Number;  
                  Family        : out    X_Host_Protocol_Family;  
                  Server_Addr   : out    X_Network_Host_Address;  
                  Connection    : out    X_Network_Connection;  
                  Status        : out    X_Status;  
                  Error         : in out X_Error_String);  
------------------------------------------------------------------------------  
--  Display_Name    - Specifies what machine/screen/number to connect to  
--  Expanded_Name   - Receives the full specification of the connection made  
--                      or None_X_String_Pointer on errors  
--  Display_Num     - Receives the display number of the connection  
--  Screen_Num      - Receives the screen number of the connection  
--  Family          - Receives the network family type of the connection  
--  Server_Addr     - Receives information about the address of the server  
--  Connection      - Receives the actual connection data values  
--  Status          - Receives an indication of the success/failure  
--  Error           - Receives a string indicating any failure  
--  
-- Attempts to connect to server, given display name. Returns network connection  
-- ID.  Status = Failed if connection fails.  The expanded display name  
-- of the form hostname:number.screen ("::" if DECnet) is returned in a result  
-- parameter. The screen number to use is also returned.  
-- The Display_Name may be of the following format:  
--  
--     [hostname] : [:] displaynumber [.screennumber]  
--  
-- The second colon indicates a DECnet style name.  No hostname is interpreted  
-- as the most efficient local connection to a server on the same machine.  
-- This is usually:  
--  
--     -  shared memory  
--     -  local stream  
--     -  UNIX domain socket  
--     -  TCP to local host  
------------------------------------------------------------------------------  

    procedure Internal_X_Disconnect_Display  
                 (Connection : in out X_Network_Connection);  
------------------------------------------------------------------------------  
-- Called to break the network connection to the server.  
------------------------------------------------------------------------------  

end Xlbmp_Internal;  

