with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Color;  
use Xlbt_Color;  
with Xlbt_Cursor_Font;  
use Xlbt_Cursor_Font;  

package Xlbp_Cursor is  
------------------------------------------------------------------------------  
-- X Library Cursors  
--  
-- Xlbp_Cursor - Creating, changing, and destroying cursors.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    function X_Create_Font_Cursor (Display : X_Display;  
                                   Cursor  : X_Cursor_Font) return X_Cursor;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Cursor  - Specifies which of the standard cursors to use.  
--  
-- Returns a cursor structure describing one of the standard cursor shapes.  
------------------------------------------------------------------------------  

    function X_Create_Glyph_Cursor (Display     : X_Display;  
                                    Source_Font : X_Font;  
                                    Mask_Font   : X_Font;  
                                    Source_Char : U_Short;  
                                    Mask_Char   : U_Short;  
                                    Foreground  : X_Color;  
                                    Background  : X_Color) return X_Cursor;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Source_Font - Specifies the font for the source glyph.  
--  Mask_Font   - Specifies the font for the mask glyph.  You may use  
--                None_X_Font.  
--  Source_Char - Specifies the source character glyph.  
--  Mask_Char   - Specifies the mask character glyph.  
--  Foreground  - Specifies the foreground color.  
--  Background  - Specifies the background color.  
--  
-- Creates a cursor based upon two glyphs.  
------------------------------------------------------------------------------  

    function X_Create_Pixmap_Cursor (Display    : X_Display;  
                                     Source     : X_Pixmap;  
                                     Mask       : X_Pixmap;  
                                     Foreground : X_Color;  
                                     Background : X_Color;  
                                     X          : U_Short;  
                                     Y          : U_Short) return X_Cursor;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Source      - Specifies the source pixmap.  
--  Mask        - Specifies the mask pixmap.  
--  Foreground  - Specifies the foreground color.  
--  Background  - Specifies the background color.  
--  X           - Specifies the X coordinate of the hotspot.  
--  Y           - Specifies the Y coordinate of the hotspot.  
--  
-- Creates a cursor using pixmaps.  
------------------------------------------------------------------------------  

    procedure X_Define_Cursor (Display : X_Display;  
                               Window  : X_Window;  
                               Cursor  : X_Cursor);  
------------------------------------------------------------------------------  
--  Display     - Specifies the cursor to use.  
--  Window      - Specifies the window to use.  
--  Cursor      - Specifies the cursor to use.  
--  
-- Defines/sets the cursor to use within a given window.  
------------------------------------------------------------------------------  

    procedure X_Free_Cursor (Display :        X_Display;  
                             Cursor  : in out X_Cursor);  
------------------------------------------------------------------------------  
--  Display     - Specifies the cursor to use.  
--  Cursor      - Specifies the cursor to free.  
--  
-- Deletes the cursor ID and frees the cursor storage if no other resource  
-- references it.  
------------------------------------------------------------------------------  

    procedure X_Query_Best_Cursor (Display     :     X_Display;  
                                   Drawable    :     X_Drawable;  
                                   Width       :     U_Short;  
                                   Height      :     U_Short;  
                                   Best_Width  : out U_Short;  
                                   Best_Height : out U_Short;  
                                   Status      : out X_Status);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Drawable    - Specifies (indirectly) the screen being queried.  
--  Width       - Specifies the width, in pixels, which we would like.  
--  Height      - Specifies the height, in pixels, which we would like.  
--  Best_Width  - Receives the best or closest width, in pixels, to Width.  
--  Best_Height - Receives the best or closest heigh, in pixels, to Height.  
--  Status      - Receives Successful or Failed.  
--  
-- Queries the server to discover the best or closest size to some desired  
-- size of cursor.  The Best_Width/Height returned indicates the size most  
-- supported by the server that is the best or closest fit to the desired size.  
------------------------------------------------------------------------------  

    procedure X_Recolor_Cursor (Display    : X_Display;  
                                Cursor     : X_Cursor;  
                                Foreground : X_Color;  
                                Background : X_Color);  
------------------------------------------------------------------------------  
--  Display     - Specifies the cursor to use.  
--  Cursor      - Specifies the cursor to recolor.  
--  Foreground  - Specifies the foreground color.  
--  Background  - Specifies the background color.  
--  
-- Recolors the specified cursor with the specified colors.  
------------------------------------------------------------------------------  

    procedure X_Undefine_Cursor (Display : X_Display;  
                                 Window  : X_Window);  
------------------------------------------------------------------------------  
--  Display     - Specifies the cursor to use.  
--  Window      - Specifies the window to use.  
--  
-- Undefines/resets the cursor for use within a window.  
------------------------------------------------------------------------------  

end Xlbp_Cursor;  

