with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Display2;  
use Xlbt_Display2;  
with Xlbt_Error;  
use Xlbt_Error;  
with Xlbt_Extension;  
use Xlbt_Extension;  
with Xlbt_Event;  
use Xlbt_Event;  
with Xlbt_Font;  
use Xlbt_Font;  
with Xlbt_Request;  
use Xlbt_Request;  
with Xlbt_String;  
use Xlbt_String;  
with Xlbt_Visual;  
use Xlbt_Visual;  

with Xlbp_Proc_Var;  
use Xlbp_Proc_Var;  

package Xlbp_Extension is  
------------------------------------------------------------------------------  
-- X Library Extensions  
--  
-- Xlbp_Extension - Used to establish and control extensions to the X Library  
-- and to the X protocol.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Extension Query Routines  
------------------------------------------------------------------------------  

    procedure X_Query_Extension (Display      :     X_Display;  
                                 Name         :     X_String;  
                                 Major_Opcode : out X_Request_Code;  
                                 First_Event  : out X_Event_Code;  
                                 First_Error  : out X_Error_Code;  
                                 Present      : out Boolean;  
                                 Status       : out X_Status);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Name        - Specifies the name of the extension.  Upper/Lower case does  
--                matter.  
--  Major_Opcode- Receives the major opcode for the extension if it has one;  
--                otherwise 0.  
--  First_Event - Receives the base event code for the extension if it has one;  
--                otherwise None_X_Event_Code.  
--  First_Error - Receives the base error code for the extension if it has one;  
--                otherwise None_X_Error_Code.  
--  Status      - Receives Successful if the extension exists; Failed if not.  
--  
-- Determines whether or not the named extension is present in the X server.  
------------------------------------------------------------------------------  

    function X_List_Extensions (Display : X_Display)  
                               return X_String_Pointer_List;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  
-- Returns an empty array if there are no extensions; otherwise returns an  
-- array of extension names.  Use the routine Free_X_Extension_List to  
-- release the storage used by the list when it is no longer needed.  
------------------------------------------------------------------------------  

    procedure Free_X_Extension_List (List : in out X_String_Pointer_List)  
        renames Free_X_String_Pointer_List;  
------------------------------------------------------------------------------  
--  List    - Specifies the list that is to be freed  
--  
-- The X_List_Extensions function returns a list of extension names.  Call  
-- this routine to free that list when it is no longer needed.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Extension Establishment Routines  
------------------------------------------------------------------------------  

    function X_Init_Extension (Display : X_Display;  
                               Name    : X_String) return X_Ext_Codes;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Name    - Specifies the name of the extension.  
--  
-- Calls X_Query_Extension to see if the extension exists.  If it does then it  
-- allocates storage for maintaining the extension information on the  
-- connection; it chains this onto the extension list for the X_Display; and it  
-- returns the info the stub implementor will need to access the extension.  
------------------------------------------------------------------------------  

    function X_Add_Extension (Display : X_Display) return X_Ext_Codes;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use  
--  
-- Allocates and initializes the X_Ext_Codes structure for a local X Library  
-- extension that does not require special server support.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Extension Procedural Hooks - DISPLAYs  
------------------------------------------------------------------------------  

    function X_E_Set_Close_Display  
                (Display   : X_Display;  
                 Extension : X_Extension_Number;  
                 Proc      : Proc_Var_X_Close_Display_Extension.Pv)  
                return Proc_Var_X_Close_Display_Extension.Pv;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display involved.  
--  Extension   - Specifies the extension number.  
--  Proc        - Specifies the routine to call.  
--  
-- Hooks in the routine to be called when a display connection is closed.  
-- Returns the previously hooked routine.  
------------------------------------------------------------------------------  

------------------------------------------------------------------------------  
-- Extension Procedural Hooks - FONTs  
------------------------------------------------------------------------------  

    function X_E_Set_Create_Font  
                (Display   : X_Display;  
                 Extension : X_Extension_Number;  
                 Proc      : Proc_Var_X_Display_Font_Extension.Pv)  
                return Proc_Var_X_Display_Font_Extension.Pv;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display involved.  
--  Extension   - Specifies the extension number.  
--  Proc        - Specifies the routine to call.  
--  
-- Hooks in a routine to call when a Font is created.  
-- Returns the previously hooked routine.  
------------------------------------------------------------------------------  

    function X_E_Set_Free_Font  
                (Display   : X_Display;  
                 Extension : X_Extension_Number;  
                 Proc      : Proc_Var_X_Display_Font_Extension.Pv)  
                return Proc_Var_X_Display_Font_Extension.Pv;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display involved.  
--  Extension   - Specifies the extension number.  
--  Proc        - Specifies the routine to call.  
--  
-- Hooks in a routine to call when a Font is freed.  
-- Returns the previously hooked routine.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Extension Procedural Hooks - GCs  
------------------------------------------------------------------------------  

    function X_E_Set_Copy_Gc  
                (Display   : X_Display;  
                 Extension : X_Extension_Number;  
                 Proc      : Proc_Var_X_Display_Gc_Extension.Pv)  
                return Proc_Var_X_Display_Gc_Extension.Pv;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display involved.  
--  Extension   - Specifies the extension number.  
--  Proc        - Specifies the routine to call.  
--  
-- Hooks in the routine to call when a GC is copied.  
-- Returns the previously hooked routine.  
------------------------------------------------------------------------------  

    function X_E_Set_Create_Gc (Display   : X_Display;  
                                Extension : X_Extension_Number;  
                                Proc      : Proc_Var_X_Display_Gc_Extension.Pv)  
                               return Proc_Var_X_Display_Gc_Extension.Pv;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display involved.  
--  Extension   - Specifies the extension number.  
--  Proc        - Specifies the routine to call.  
--  
-- Hooks in the routine to call when a GC is created.  
-- Returns the previously hooked routine.  
------------------------------------------------------------------------------  

    function X_E_Set_Flush_Gc (Display   : X_Display;  
                               Extension : X_Extension_Number;  
                               Proc      : Proc_Var_X_Display_Gc_Extension.Pv)  
                              return Proc_Var_X_Display_Gc_Extension.Pv;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display involved.  
--  Extension   - Specifies the extension number.  
--  Proc        - Specifies the routine to call.  
--  
-- Hooks in a routine to call when a GC is flushed.  
-- Returns the previously hooked routine.  
------------------------------------------------------------------------------  

    function X_E_Set_Free_Gc (Display   : X_Display;  
                              Extension : X_Extension_Number;  
                              Proc      : Proc_Var_X_Display_Gc_Extension.Pv)  
                             return Proc_Var_X_Display_Gc_Extension.Pv;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display involved.  
--  Extension   - Specifies the extension number.  
--  Proc        - Specifies the routine to call.  
--  
-- Hooks in the routine to call when a GC is freed.  
-- Returns the previously hooked routine.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Extension Procedural Hooks - EVENTs  
------------------------------------------------------------------------------  

    function X_E_Set_Wire_To_Event (Display : X_Display;  
                                    Event   : X_Event_Code;  
                                    Proc    : Proc_Var_X_Wire_Event.Pv)  
                                   return Proc_Var_X_Wire_Event.Pv;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display involved.  
--  Event        - Specifies the event code to be modified.  
--  Proc        - Specifies the routine to call.  
--  
-- Hooks in a Wire-to-Event translation routine.  
-- Returns the previously hooked routine.  
------------------------------------------------------------------------------  

    function X_E_Set_Event_To_Wire (Display : X_Display;  
                                    Event   : X_Event_Code;  
                                    Proc    : Proc_Var_X_Event_Wire.Pv)  
                                   return Proc_Var_X_Event_Wire.Pv;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display involved.  
--  Event        - Specifies the event code to be modified.  
--  Proc        - Specifies the routine to call.  
--  
-- Hooks in an Event-to-Wire translation routine.  
-- Returns the previously hooked routine.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Extension Procedural Hooks - ERRORs  
------------------------------------------------------------------------------  

    function X_E_Set_Error (Display   : X_Display;  
                            Extension : X_Extension_Number;  
                            Proc      : Proc_Var_X_Error_Extension.Pv)  
                           return Proc_Var_X_Error_Extension.Pv;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display involved.  
--  Extension   - Specifies the extension number.  
--  Proc        - Specifies the routine to call.  
--  
-- Hooks in routine to call when an X protocol error occurs.  
-- Returns the previously hooked routine.  
------------------------------------------------------------------------------  

    function X_E_Set_Error_String  
                (Display   : X_Display;  
                 Extension : X_Extension_Number;  
                 Proc      : Proc_Var_X_Error_String_Extension.Pv)  
                return Proc_Var_X_Error_String_Extension.Pv;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display involved.  
--  Extension   - Specifies the extension number.  
--  Proc        - Specifies the routine to call.  
--  
-- Hooks in routine to call when an error needs to be turned into a string.  
-- Returns the previously hooked routine.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Extension Data Hooks  
------------------------------------------------------------------------------  

    procedure X_Add_To_Extension_List  
                 (Structure : X_Display;         -- X_Display  
                  Ext_Data  : X_Ext_Data);  
    procedure X_Add_To_Extension_List  
                 (Structure : X_Font_Struct;     -- X_Font_Struct  
                  Ext_Data  : X_Ext_Data);  
    procedure X_Add_To_Extension_List  
                 (Structure : X_Gc;              -- X_Gc  
                  Ext_Data  : X_Ext_Data);  
    procedure X_Add_To_Extension_List  
                 (Structure : X_Screen;          -- X_Screen  
                  Ext_Data  : X_Ext_Data);  
    procedure X_Add_To_Extension_List  
                 (Structure : X_Screen_Format;  -- X_Screen_Format  
                  Ext_Data  : X_Ext_Data);  
    procedure X_Add_To_Extension_List  
                 (Structure : X_Visual;          -- X_Visual  
                  Ext_Data  : X_Ext_Data);  
------------------------------------------------------------------------------  
--  Structure - Specifies the structure to receive the Ext_Data.  
--  Ext_Data  - Specifies the X_Ext_Data to hang upon the Structure.  
--  
-- Set the X_Extension_Number within the X_Ext_Data before calling this  
-- routine to add the X_Ext_Data to the structure.  
------------------------------------------------------------------------------  

    function X_Find_On_Extension_List  
                (Structure : X_Display;                      -- X_Display  
                 Extension : X_Extension_Number) return X_Ext_Data;  
    function X_Find_On_Extension_List  
                (Structure : X_Font_Struct;                  -- X_Font_Struct  
                 Extension : X_Extension_Number) return X_Ext_Data;  
    function X_Find_On_Extension_List  
                (Structure : X_Gc;                           -- X_Gc  
                 Extension : X_Extension_Number) return X_Ext_Data;  
    function X_Find_On_Extension_List  
                (Structure : X_Screen;                       -- X_Screen  
                 Extension : X_Extension_Number) return X_Ext_Data;  
    function X_Find_On_Extension_List  
                (Structure : X_Screen_Format;   -- X_Screen_Format  
                 Extension : X_Extension_Number) return X_Ext_Data;  
    function X_Find_On_Extension_List  
                (Structure : X_Visual;                       -- X_Visual  
                 Extension : X_Extension_Number) return X_Ext_Data;  
------------------------------------------------------------------------------  
--  Structure - Specifies the structure to examine for the Ext_Data.  
--  Extension - Specifies the extension number to look for.  
--  
-- Returns None_X_Ext_Data or else the X_Ext_Data on the Structure that has the  
-- specified extension number.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Special X Library Control Routines  
------------------------------------------------------------------------------  

    function X_Alloc_Id (Display : X_Display) return X_Id;  
------------------------------------------------------------------------------  
-- Display  - Specifies the display to use.  
--  
-- Request and allocate an X_Id for a new resource on a given display.  
------------------------------------------------------------------------------  

--
end Xlbp_Extension;  

