with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Font;  
use Xlbt_Font;  
with Xlbt_String;  
use Xlbt_String;  

package Xlbp_Font_Names is  
------------------------------------------------------------------------------  
-- X Library Font Names  
--  
-- Xlbp_Font_Names - What fonts are there and where do they live?  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    function X_Get_Font_Path (Display : X_Display) return X_String_Pointer_List;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  
-- Returns the current font search path for the X server.  
-- Use Free_X_String_Pointer_List to free up the storage associated with the  
-- list.  
------------------------------------------------------------------------------  

    procedure X_Set_Font_Path (Display     : X_Display;  
                               Directories : X_String_Pointer_Array);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Directories - Specifies the search path to be set.  
--  
-- Sets the directory path used by the X server when looking for a font.  
-- Setting the path to empty (null array) restores the default path.  
-- There is only one path per server and the names in the path are machine  
-- dependent (server machine dependent).  
------------------------------------------------------------------------------  

    function X_List_Fonts_With_Info  
                (Display       : X_Display;  
                 Pattern       : X_String;  
                 Maximum_Names : U_Short) return X_Fonts_With_Info;  
------------------------------------------------------------------------------  
--  Display       - Specifies the display to use.  
--  Pattern       - Specifies the pattern string to use.  (Note: Unlike in the  
--                  C world; the string should not be null terminated.)  
--  Maximum_Names - Specifies the maximum number of names to be returned.  
--  
-- Returns a list of names and a corresponding list of font information.  
-- These fonts have names that match the pattern.  The list of names is  
-- limited in size to the number specified.  Call X_Free_Font_Info  
-- in order to free the storage associated with the various fonts when done.  
------------------------------------------------------------------------------  

    procedure X_Free_Font_Info (Info : in out X_Fonts_With_Info);  
------------------------------------------------------------------------------  
--  Info    - A value returned by X_List_Fonts_With_Info.  
--  
-- This is called to free up the information returned by  
-- X_List_Fonts_With_Info.  
------------------------------------------------------------------------------  

    function X_List_Fonts  
                (Display       : X_Display;  
                 Pattern       : X_String;  
                 Maximum_Names : U_Short) return X_String_Pointer_List;  
------------------------------------------------------------------------------  
--  Display       - Specifies the display to use.  
--  Pattern       - Specifies the pattern string to use.  (Note: Unlike in the  
--                  C world; the string should not be null terminated.)  
--  Maximum_Names - Specifies the maximum number of names to be returned.  
--  
-- Returns a list of names of available fonts.  These fonts have names that  
-- match the pattern.  The list of names is limited in size to the number  
-- specified.  Call Free_X_String_Pointer_List to free up the list.  
------------------------------------------------------------------------------  

end Xlbp_Font_Names;  

