with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Graphics;  
use Xlbt_Graphics;  

package Xlbp_Graphics is  
------------------------------------------------------------------------------  
-- X Library Graphics  
--  
-- Xlbp_Graphics - Draw various types of things on a window  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    procedure X_Clear_Area (Display   : X_Display;  
                            Window    : X_Window;  
                            X         : S_Short;  
                            Y         : S_Short;  
                            Width     : U_Short;  
                            Height    : U_Short;  
                            Exposures : Boolean);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Window      - Specifies the window to use.  
--  X           - Specifies the X coordinate of the rectangle to clear.  
--  Y           - Specifies the Y coordinate of the rectangle to clear.  
--  Width       - Specifies the width, in pixels, of the rectangle.  
--  Height      - Specifies the height, in pixels, of the rectangle.  
--  Exposures   - Specifies True if exposure events should be generated.  
--  
-- Clears a rectangle within a window and optionally generates exposure  
-- event(s) for that rectangle.  X/Y/Width/Height of 0 means "clear the entire  
-- window".  
------------------------------------------------------------------------------  

    procedure X_Clear_Window (Display : X_Display;  
                              Window  : X_Window);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Window      - Specifies the window to use.  
--  
-- Clears the entire window.  
-- Equivalent to X_Clear_Area(Display,Window,0,0,0,0,False);  
------------------------------------------------------------------------------  

    procedure X_Copy_Area (Display       : X_Display;  
                           Source        : X_Drawable;  
                           Destination   : X_Drawable;  
                           Gc            : X_Gc;  
                           Source_X      : S_Short;  
                           Source_Y      : S_Short;  
                           Width         : U_Short;  
                           Height        : U_Short;  
                           Destination_X : S_Short;  
                           Destination_Y : S_Short);  
------------------------------------------------------------------------------  
--  Display       - Specifies the display to use.  
--  Source        - Specifies the source of the copy.  
--  Destination   - Specifies the destination for the copy.  
--  Gc            - Specifies the graphics context for the copy.  
--  Source_X      - Specifies the X coordinate within the source rectangle.  
--  Source_Y      - Specifies the Y coordinate within the source rectangle.  
--  Width         - Specifies the width, in pixels, of the rectangle.  
--  Height        - Specifies the height, in pixels, of the rectangle.  
--  Destination_X - Specifies the X coordinate within the destination rectangle.  
--  Destination_Y - Specifies the Y coordinate within the destination rectangle.  
--  
-- Copies the Source rectangle to the Destination drawable.  The two rectangles  
-- must have the same root and depth.  
------------------------------------------------------------------------------  

    procedure X_Copy_Plane (Display       : X_Display;  
                            Source        : X_Drawable;  
                            Destination   : X_Drawable;  
                            Gc            : X_Gc;  
                            Source_X      : S_Short;  
                            Source_Y      : S_Short;  
                            Width         : U_Short;  
                            Height        : U_Short;  
                            Destination_X : S_Short;  
                            Destination_Y : S_Short;  
                            Bit_Plane     : X_Plane_Mask);  
------------------------------------------------------------------------------  
--  Display       - Specifies the display to use.  
--  Source        - Specifies the source of the copy.  
--  Destination   - Specifies the destination for the copy.  
--  Gc            - Specifies the graphics context for the copy.  
--  Source_X      - Specifies the X coordinate of the source rectangle.  
--  Source_Y      - Specifies the Y coordinate of the source rectangle.  
--  Width         - Specifies the width, in pixels, of the rectangle.  
--  Height        - Specifies the height, in pixels, of the rectangle.  
--  Destination_X - Specifies the X coordinate of the destination rectangle.  
--  Destination_Y - Specifies the Y coordinate of the destination rectangle.  
--  Bit_Plane     - Specifies the plane (only one) involved in the copy.  
--  
-- Copies the Source rectangle to the Destination drawable.  The two rectangles  
-- must have the same root and depth.  
------------------------------------------------------------------------------  

    procedure X_Draw_Arcs (Display  : X_Display;  
                           Drawable : X_Drawable;  
                           Gc       : X_Gc;  
                           Arcs     : X_Arc_Array);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Drawable - Specifies the drawable (Window, Pixmap) to use.  
--  Gc       - Specifies the graphics context to use.  
--  Arcs     - Specifies the arcs to draw.  
--  
-- Draws a series of arcs on the drawable.  
------------------------------------------------------------------------------  

    procedure X_Draw_Arc (Display  : X_Display;  
                          Drawable : X_Drawable;  
                          Gc       : X_Gc;  
                          X        : S_Short;  
                          Y        : S_Short;  
                          Width    : U_Short;  
                          Height   : U_Short;  
                          Angle1   : S_Short;  
                          Angle2   : S_Short);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Drawable - Specifies the drawable (Window, Pixmap) to use.  
--  Gc       - Specifies the graphics context to use.  
--  X        - Specifies the X coordinate of the arc.  
--  Y        - Specifies the Y coordinate of the arc.  
--  Width    - Specifies the width, in pixels, of the arc.  
--  Height   - Specifies the height, in pixels, of the arc.  
--  Angle1   - Specifies the start of the arc relative to the 3-o-clock  
--             position from the center in units of degrees*64.  
--  Angle2   - Specifies the path and extent of the arc relative to the start  
--             of the arc, in units of degree*64.  
--  
-- Draws a single arc on the drawable.  
------------------------------------------------------------------------------  

    procedure X_Fill_Arcs (Display  : X_Display;  
                           Drawable : X_Drawable;  
                           Gc       : X_Gc;  
                           Arcs     : X_Arc_Array);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Drawable - Specifies the drawable (Window, Pixmap) to use.  
--  Gc       - Specifies the graphics context to use.  
--  Arcs     - Specifies the arcs to draw.  
--  
-- Fills a series of arcs on the drawable.  
------------------------------------------------------------------------------  

    procedure X_Fill_Arc (Display  : X_Display;  
                          Drawable : X_Drawable;  
                          Gc       : X_Gc;  
                          X        : S_Short;  
                          Y        : S_Short;  
                          Width    : U_Short;  
                          Height   : U_Short;  
                          Angle1   : S_Short;  
                          Angle2   : S_Short);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Drawable - Specifies the drawable (Window, Pixmap) to use.  
--  Gc       - Specifies the graphics context to use.  
--  X        - Specifies the X coordinate of the arc.  
--  Y        - Specifies the Y coordinate of the arc.  
--  Width    - Specifies the width, in pixels, of the arc.  
--  Height   - Specifies the height, in pixels, of the arc.  
--  Angle1   - Specifies the start of the arc relative to the 3-o-clock  
--             position from the center in units of degrees*64.  
--  Angle2   - Specifies the path and extent of the arc relative to the start  
--             of the arc, in units of degree*64.  
--  
-- Fills a single arc on the drawable.  
------------------------------------------------------------------------------  

    procedure X_Draw_Points (Display  : X_Display;  
                             Drawable : X_Drawable;  
                             Gc       : X_Gc;  
                             Points   : X_Point_Array;  
                             Mode     : X_Coordinate_Mode);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Drawable - Specifies the drawable (Window, Pixmap) to use.  
--  Gc       - Specifies the graphics context to use.  
--  Points   - Specifies an array of points to be drawn.  
--  Mode     - Specifies the coordinate mode; Coord_Mode_Origin (absolute) or  
--             Coord_Mode_Previous (relative).  
--  
-- Draws a series of points on the drawable.  
------------------------------------------------------------------------------  

    procedure X_Draw_Point (Display  : X_Display;  
                            Drawable : X_Drawable;  
                            Gc       : X_Gc;  
                            X        : S_Short;  
                            Y        : S_Short);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Drawable - Specifies the drawable (Window, Pixmap) to use.  
--  Gc       - Specifies the graphics context to use.  
--  X        - Specifies the X coordinate of the point.  
--  Y        - Specifies the Y coordinate of the point.  
--  
-- Draws a point on the drawable.  
------------------------------------------------------------------------------  

    procedure X_Fill_Polygon (Display  : X_Display;  
                              Drawable : X_Drawable;  
                              Gc       : X_Gc;  
                              Points   : X_Point_Array;  
                              Shape    : X_Polygon_Shape;  
                              Mode     : X_Coordinate_Mode);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Drawable - Specifies the drawable (Window, Pixmap) to use.  
--  Gc       - Specifies the graphics context to use.  
--  Points   - Specifies an array of points to be drawn.  
--  Shape    - Specifies a "shape" value that helps the server to optimize  
--             the drawing time.  
--  Mode     - Specifies the coordinate mode; Coord_Mode_Origin (absolute) or  
--             Coord_Mode_Previous (relative).  
--  
-- Draws a series of points on the drawable.  
------------------------------------------------------------------------------  

    procedure X_Draw_Lines (Display  : X_Display;  
                            Drawable : X_Drawable;  
                            Gc       : X_Gc;  
                            Points   : X_Point_Array;  
                            Mode     : X_Coordinate_Mode);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Drawable - Specifies the drawable (Window, Pixmap) to use.  
--  Gc       - Specifies the graphics context to use.  
--  Points   - Specifies an array of points indicating the lines to be drawn.  
--  Mode     - Specifies the coordinate mode; Coord_Mode_Origin (absolute) or  
--             Coord_Mode_Previous (relative).  
--  
-- Draws a series of connected lines on the drawable.  
------------------------------------------------------------------------------  

    procedure X_Draw_Line (Display  : X_Display;  
                           Drawable : X_Drawable;  
                           Gc       : X_Gc;  
                           X1       : S_Short;  
                           Y1       : S_Short;  
                           X2       : S_Short;  
                           Y2       : S_Short);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Drawable - Specifies the drawable (Window, Pixmap) to use.  
--  Gc       - Specifies the graphics context to use.  
--  X1       - Specifies the initial point of the line.  
--  Y1       - Specifies the initial point of the line.  
--  X2       - Specifies the final point of the line.  
--  Y2       - Specifies the final point of the line.  
--  
-- Draws a single line on the drawable.  
------------------------------------------------------------------------------  

    procedure X_Draw_Segments (Display  : X_Display;  
                               Drawable : X_Drawable;  
                               Gc       : X_Gc;  
                               Segments : X_Segment_Array);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Drawable - Specifies the drawable (Window, Pixmap) to use.  
--  Gc       - Specifies the graphics context to use.  
--  Segments - Specifies the line segments to draw.  
--  
-- Draws a series of line segments on the drawable.  
------------------------------------------------------------------------------  

    procedure X_Draw_Rectangles (Display    : X_Display;  
                                 Drawable   : X_Drawable;  
                                 Gc         : X_Gc;  
                                 Rectangles : X_Rectangle_Array);  
------------------------------------------------------------------------------  
--  Display    - Specifies the display to use.  
--  Drawable   - Specifies the drawable (Window, Pixmap) to use.  
--  Gc         - Specifies the graphics context to use.  
--  Rectangles - Specifies the rectangles to be drawn.  
--  
-- Draws a series of rectangles on a drawable.  
------------------------------------------------------------------------------  

    procedure X_Draw_Rectangle (Display  : X_Display;  
                                Drawable : X_Drawable;  
                                Gc       : X_Gc;  
                                X        : S_Short;  
                                Y        : S_Short;  
                                Width    : U_Short;  
                                Height   : U_Short);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Drawable - Specifies the drawable (Window, Pixmap) to use.  
--  Gc       - Specifies the graphics context to use.  
--  X        - Specifies the X coordinate of the rectangle.  
--  Y        - Specifies the Y coordinate of the rectangle.  
--  Width    - Specifies the width, in pixels, of the rectangle.  
--  Height   - Specifies the height, in pixels, of the rectangle.  
--  
-- Draws a single rectangle on the drawable.  
------------------------------------------------------------------------------  

    procedure X_Fill_Rectangles (Display    : X_Display;  
                                 Drawable   : X_Drawable;  
                                 Gc         : X_Gc;  
                                 Rectangles : X_Rectangle_Array);  
------------------------------------------------------------------------------  
--  Display    - Specifies the display to use.  
--  Drawable   - Specifies the drawable (Window, Pixmap) to use.  
--  Gc         - Specifies the graphics context to use.  
--  Rectangles - Specifies the rectangles to be filled.  
--  
-- Fills a series of rectangles on a drawable.  
------------------------------------------------------------------------------  

    procedure X_Fill_Rectangle (Display  : X_Display;  
                                Drawable : X_Drawable;  
                                Gc       : X_Gc;  
                                X        : S_Short;  
                                Y        : S_Short;  
                                Width    : U_Short;  
                                Height   : U_Short);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Drawable - Specifies the drawable (Window, Pixmap) to use.  
--  Gc       - Specifies the graphics context to use.  
--  X        - Specifies the X coordinate of the rectangle.  
--  Y        - Specifies the Y coordinate of the rectangle.  
--  Width    - Specifies the width, in pixels, of the rectangle.  
--  Height   - Specifies the height, in pixels, of the rectangle.  
--  
-- Fills a single rectangle on the drawable.  
------------------------------------------------------------------------------  

end Xlbp_Graphics;  

