with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Hint3;  
use Xlbt_Hint3;  
with Xlbt_String;  
use Xlbt_String;  

package Xlbp_Hint is  
------------------------------------------------------------------------------  
-- X Library Hints - Window Manager Hints  
--  
-- Xlbp_Hint - Support for the various Window Manager properties of a window.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1987 - 1989 by Digital Equipment Corporation, Maynard, Mass.  
-- Copyright 1987 - 1989 by Massachusetts Institute of Technology,  
--                          Cambridge, Massachusetts.  
-- Copyright 1988 - 1988 by Wyse Technology, Inc., San Jose, California.  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of Digital, MIT, Rational, or  
-- Wyse not be used in advertising or publicity pertaining to distribution of  
-- the software without specific, written prior permission.  
--  
-- Digital, MIT, Rational, and Wyse disclaim all warranties with regard to this  
-- software, including all implied warranties of merchantability and fitness,  
-- in no event shall DIGITAL be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- WM_CLASS  
------------------------------------------------------------------------------  

    procedure X_Get_Class_Hint (Display    :     X_Display;  
                                Window     :     X_Window;  
                                Class_Hint : out X_Class_Hint;  
                                Status     : out X_Status);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Window           - Specifies the window to use.  
--  Class_Hint  - Receives the window class hints.  
--  Status      - Receives Successful or Failed.  
--  
-- Fetches the window class hints for the window.  
------------------------------------------------------------------------------  

    procedure X_Set_Class_Hint (Display    :     X_Display;  
                                Window     :     X_Window;  
                                Class_Hint :     X_Class_Hint;  
                                Status     : out X_Status);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Window      - Specifies the window to use.  
--  Class_Hint  - Specifies the class hints to be set for this window.  
--  Status      - Receives Successful or Failed.  
--  
-- Sets the class hints for a window.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- WM_COLORMAP_WINDOWS  
------------------------------------------------------------------------------  

    procedure X_Get_Wm_Colormap_Windows (Display :     X_Display;  
                                         Window  :     X_Window;  
                                         Windows : out X_Window_List;  
                                         Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Windows - Receives None_X_Window_List or a list of windows  
--  Status  - Receives Successful or Failed.  
--  
-- Called to fetch the list of window identifiers stored in the  
-- WM_COLORMAP_WINDOWS property on the specified window.  Returns Failed if  
-- a) the property has never been set, b) the property is set but it does not  
-- have the correct type, format, or size.  The list must be freed after use  
-- with the Free_X_Window_List routine.  
------------------------------------------------------------------------------  

    procedure X_Set_Wm_Colormap_Windows (Display :     X_Display;  
                                         Window  :     X_Window;  
                                         Windows :     X_Window_Array;  
                                         Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Windows - Specifies the window list.  
--  Status  - Receives Successful or Failed.  
--  
-- Sets the WM_COLORMAP_WINDOWS property for the specified window.  The  
-- property is set to have a type of "WINDOW" and a format of 32.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Standard Colormaps  
------------------------------------------------------------------------------  

    procedure X_Get_Rgb_Colormaps (Display   :     X_Display;  
                                   Window    :     X_Window;  
                                   Colormaps : out X_Standard_Colormap_List;  
                                   Property  :     X_Atom;  
                                   Status    : out X_Status);  
------------------------------------------------------------------------------  
--  Display   - Specifies the display to use.  
--  Window    - Specifies the window to use.  
--  Colormaps - Receives the color map(s) associated with the Property.  
--  Property  - Specifies the property to query.  
--  Status    - Receives Successful or Failed.  
--  
-- Used to obtain the color map definition(s) associated with the standard  
-- color map property specified.  
--  
-- If the property exists, is of type "RGB_COLOR_MAP", is of format 32, and is  
-- long enough to contain a colormap definition we allocate and fill in the  
-- space for the returned colormaps and returns Status = Successful.  
--  
-- If the Visual_Id of a colormap is not present then we assume the default  
-- visual for the screen.  If the Kill_Id is not present then None is used;  
-- meaning that the resources cannot be released.  
------------------------------------------------------------------------------  

    procedure X_Set_Rgb_Colormaps (Display   :     X_Display;  
                                   Window    :     X_Window;  
                                   Colormaps :     X_Standard_Colormap_Array;  
                                   Property  :     X_Atom;  
                                   Status    : out X_Status);  
------------------------------------------------------------------------------  
--  Display   - Specifies the display to use.  
--  Window    - Specifies the window to use.  
--  Colormaps - Specifies the Colormap(s) to use.  
--  Property  - Specifies the property to set.  
--  Status    - Receives Successful or Failed.  
--  
-- Used by window managers to create standard color map properties.  
--  
-- The colormap(s) are stored with type "RGB_COLOR_MAP" and a format of 32.  
--  
-- It is the caller's responsibility to honor the ICCCM restriction that only  
-- "RGB_DEFAULT_MAP" contain more than one definition.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- WM_COMMAND  
------------------------------------------------------------------------------  

    procedure X_Get_Command (Display :     X_Display;  
                             Window  :     X_Window;  
                             Arg_V   : out X_String_Pointer_List;  
                             Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Arg_V   - Receives None_X_String_Pointer_List or a list of strings  
--  Status  - Receives Successful or Failed.  
--  
-- Called to fetch the command arguments which were used in a call to  
-- X_Set_Class_Hint.  Returns Failed if a) the WM_COMMAND property  
-- has never been set, b) the property is set but it does not have the correct  
-- type, format, or size.  The command list must be freed after use with the  
-- Free_X_String_Pointer_List routine.  
------------------------------------------------------------------------------  

    procedure X_Set_Command (Display :     X_Display;  
                             Window  :     X_Window;  
                             Arg_V   :     X_String_Pointer_Array;  
                             Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Arg_V   - Specifies the command arguments.  
--  Status  - Receives Successful or Failed.  
--  
-- Tells the window manager the command arguments which were used to start the  
-- application.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- WM_ICON_SIZE  
------------------------------------------------------------------------------  

    procedure X_Get_Icon_Sizes (Display :     X_Display;  
                                Window  :     X_Window;  
                                List    : out X_Icon_Size_List;  
                                Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  List    - Specifies the full list of supported sizes.  
--  Status  - Receives Successful or Failed.  
--  
-- Returns a list of the icon sizes supported by the window manager.  Returns  
-- Failed if there is no list, the list is invalid, or there was some error.  
-- The List must be freed after use with the Free_X_Icon_Size_List routine.  
------------------------------------------------------------------------------  

    procedure X_Set_Icon_Sizes (Display :     X_Display;  
                                Window  :     X_Window;  
                                List    :     X_Icon_Size_Array;  
                                Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  List    - Specifies the full list of supported sizes.  
--  Status  - Receives Successful or Failed.  
--  
-- Sets the list of supported icon sizes for a window manager.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- WM_ICON_NAME  
------------------------------------------------------------------------------  

    procedure X_Get_Wm_Icon_Name (Display :     X_Display;  
                                  Window  :     X_Window;  
                                  Name    : out X_Text_Property;  
                                  Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Name    - Receives the result of the request  
--  Status  - Receives Successful or Failed.  
--  
-- Attempts to obtain the name of the window's icon.  If the name has not been  
-- set then Status = Failed upon return.  The data in the X_Text_Property must  
-- be freed with a Free_X_Text_Property call after use.  
------------------------------------------------------------------------------  

    procedure X_Set_Wm_Icon_Name (Display   :     X_Display;  
                                  Window    :     X_Window;  
                                  Icon_Name :     X_Text_Property;  
                                  Status    : out X_Status);  
------------------------------------------------------------------------------  
--  Display   - Specifies the display to use.  
--  Window    - Specifies the window to use.  
--  Icon_Name - Specifies the name.  
--  Status    - Receives Successful or Failed.  
--  
-- Sets the name for a window's icon.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- WM_NAME  
------------------------------------------------------------------------------  

    procedure X_Get_Wm_Name (Display :     X_Display;  
                             Window  :     X_Window;  
                             Name    : out X_Text_Property;  
                             Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Name    - Receives the result of the request  
--  Status  - Receives Successful or Failed.  
--  
-- Attempts to obtain the name of the window.  If the name has not been  
-- set then Status = Failed upon return.  The data in the X_Text_Property must  
-- be freed with a Free_X_Text_Property call after use.  
------------------------------------------------------------------------------  

    procedure X_Set_Wm_Name (Display :     X_Display;  
                             Window  :     X_Window;  
                             Name    :     X_Text_Property;  
                             Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Name    - Specifies the name.  
--  Status  - Receives Successful or Failed.  
--  
-- Sets the name for a window.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- WM_NORMAL_HINTS  
------------------------------------------------------------------------------  

    procedure X_Get_Wm_Normal_Hints (Display :     X_Display;  
                                     Window  :     X_Window;  
                                     Hints   : out X_Size_Hints;  
                                     Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Hints   - Receives the normal-window size hints.  
--  Status  - Receives Successful or Failed.  
--  
-- Fetches the current normal-window-state size hints for the window.  
------------------------------------------------------------------------------  

    procedure X_Set_Wm_Normal_Hints (Display :     X_Display;  
                                     Window  :     X_Window;  
                                     Hints   :     X_Size_Hints;  
                                     Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Hints   - Specifies the normal-size size hints.  
--  Status  - Receives Successful or Failed.  
--  
-- Sets the normal-window-state sizing hints for this window.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- WM_PROTOCOLS  
------------------------------------------------------------------------------  

    procedure X_Get_Wm_Protocols (Display   :     X_Display;  
                                  Window    :     X_Window;  
                                  Protocols : out X_Atom_List;  
                                  Status    : out X_Status);  
------------------------------------------------------------------------------  
--  Display   - Specifies the display to use.  
--  Window    - Specifies the window to use.  
--  Protocols - Receives the window protocols list.  
--  Status    - Receives Successful or Failed.  
--  
-- Fetches the protocol list for the window.  
--  
-- Called to fetch the protocol list which was used in a call to  
-- X_Set_Class_Hint.  Returns Failed if a) the WM_PROTOCOLS property  
-- has never been set, b) the property is set but it does not have the correct  
-- type, format, or size.  
--  
-- Free the list with Free_X_Atom_List after use.  
------------------------------------------------------------------------------  

    procedure X_Set_Wm_Protocols (Display   :     X_Display;  
                                  Window    :     X_Window;  
                                  Protocols :     X_Atom_Array;  
                                  Status    : out X_Status);  
------------------------------------------------------------------------------  
--  Display   - Specifies the display to use.  
--  Window    - Specifies the window to use.  
--  Protocols - Specifies the protocols to be set for this window.  
--  Status    - Receives Successful or Failed.  
--  
-- Sets the protocol list for a window.  
--  
-- The property is stored with a type of "XA_ATOM" and a format of 32.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- WM_SIZE_HINTS  
------------------------------------------------------------------------------  

    procedure X_Get_Wm_Size_Hints (Display  :     X_Display;  
                                   Window   :     X_Window;  
                                   Property :     X_Atom;  
                                   Hints    : out X_Size_Hints;  
                                   Status   : out X_Status);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Window   - Specifies the window to use.  
--  Property - Specifies the property to obtain.  
--  Hints    - Receives the size hints for the specified property.  
--  Status   - Receives Successful or Failed.  
--  
-- Returns the size hints for the specified window manager property.  
------------------------------------------------------------------------------  

    procedure X_Set_Wm_Size_Hints (Display  :     X_Display;  
                                   Window   :     X_Window;  
                                   Property :     X_Atom;  
                                   Hints    :     X_Size_Hints;  
                                   Status   : out X_Status);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Window   - Specifies the window to use.  
--  Property - Specifies the property to be set.  
--  Hints    - Specifies the new size hints.  
--  Status   - Receives Successful or Failed.  
--  
-- Sets the specified property to the Hints.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Standard Window Manager Properties  
------------------------------------------------------------------------------  

    procedure X_Set_Wm_Properties (Display     :     X_Display;  
                                   Window      :     X_Window;  
                                   Window_Name :     X_Text_Property;  
                                   Icon_Name   :     X_Text_Property;  
                                   Arg_V       :     X_String_Pointer_Array;  
                                   Normal      :     X_Size_Hints;  
                                   Wm          :     X_Wm_Hints;  
                                   Class       :     X_Class_Hint;  
                                   Status      : out X_Status);  
    procedure X_Set_Wm_Properties (Display     :     X_Display;  
                                   Window      :     X_Window;  
                                   Window_Name :     X_String;  
                                   Icon_Name   :     X_String;  
                                   Arg_V       :     X_String_Pointer_Array;  
                                   Normal      :     X_Size_Hints;  
                                   Wm          :     X_Wm_Hints;  
                                   Class       :     X_Class_Hint;  
                                   Status      : out X_Status);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Window      - Specifies the window to decorate.  
--  Window_Name - Specifies the name of the window.  
--  Icon_Name   - Specifies the name of the window's icon.  
--  Arg_V       - Specifies the command arguments used to start the application.  
--  Normal      - Specifies a series of window manager sizing hints.  
--  Wm          - Specifies a series of window manager control hints.  
--  Class       - Specifies a application name/class hint  
--  Status      - Receives Successful or Failed.  
--  
-- Provides a way to specify a minimum set of properties that describe the  
-- "quickie" application to a window manager.  
--  
-- If Window_Name = None_X_Text_Property then the WM_NAME property is not set.  
--  
-- If Icon_Name = None_X_Text_Property then the WM_ICON_Name property is not  
-- set.  
--  
-- If Arg_V'Length = 0 then the WM_COMMAND property is not set.  
--  
-- If Normal = None_X_Size_Hints then the WM_NORMAL_HINTS property  
--  
-- If Wm = None_X_Wm_Hints then the WM_HINTS property is not set.  
--  
-- If Class = None_X_Class_Hints then the WM_CLASS property is not set.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- WM_TRANSIENT_FOR  
------------------------------------------------------------------------------  

    procedure X_Get_Transient_For_Hint (Display :     X_Display;  
                                        Window  :     X_Window;  
                                        Parent  : out X_Window;  
                                        Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Parent  - Receives the window's logical parent.  
--  Status  - Receives Successful or Failed.  
--  
-- Fetches the WM_TRANSIENT_FOR property for the window.  This window is a top  
-- level window but logically it is a type of subwindow for the other window.  
------------------------------------------------------------------------------  

    procedure X_Set_Transient_For_Hint (Display :     X_Display;  
                                        Window  :     X_Window;  
                                        Parent  :     X_Window;  
                                        Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Parent  - Specifies this window's logical parent window.  
--  Status  - Receives Successful or Failed.  
--  
-- Sets the WM_TRANSIENT_FOR property for the window.  This window is a top  
-- level window but logically it is a type of subwindow for the other window.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- WM_HINTS  
------------------------------------------------------------------------------  

    procedure X_Get_Wm_Hints (Display  :     X_Display;  
                              Window   :     X_Window;  
                              Wm_Hints : out X_Wm_Hints;  
                              Status   : out X_Status);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Window   - Specifies the window to use.  
--  Wm_Hints - Receives the window manager hints.  
--  Status   - Receives Successful or Failed.  
--  
-- Obtains the current window manager hints for this window.  
------------------------------------------------------------------------------  

    procedure X_Set_Wm_Hints (Display  :     X_Display;  
                              Window   :     X_Window;  
                              Wm_Hints :     X_Wm_Hints;  
                              Status   : out X_Status);  
------------------------------------------------------------------------------  
--  Display  - Specifies the display to use.  
--  Window   - Specifies the window to use.  
--  Wm_Hints - Specifies the hints to set.  
--  Status   - Receives Successful or Failed.  
--  
-- Sets the window manager hints for this window.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- WM_CLIENT_MACHINE  
------------------------------------------------------------------------------  

    procedure X_Get_Wm_Client_Machine (Display :     X_Display;  
                                       Window  :     X_Window;  
                                       Machine : out X_Text_Property;  
                                       Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Machine - Receives the name of the client machine  
--  Status  - Receives Successful or Failed.  
--  
-- Attempts to obtain the client-machine property for the window.  If the  
-- name has not been set then Status = Failed upon return.  The data in the  
-- X_Text_Property must be freed with a Free_X_Text_Property call after use.  
------------------------------------------------------------------------------  

    procedure X_Set_Wm_Client_Machine (Display :     X_Display;  
                                       Window  :     X_Window;  
                                       Machine :     X_Text_Property;  
                                       Status  : out X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Machine - Specifies the client-machine value to set.  
--  Status  - Receives Successful or Failed.  
--  
-- Sets the client-machine property for the window.  
------------------------------------------------------------------------------  

--
end Xlbp_Hint;  

