with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Key;  
use Xlbt_Key;  
with Xlbt_Keyboard;  
use Xlbt_Keyboard;  

package Xlbp_Keyboard_Encoding is  
------------------------------------------------------------------------------  
-- X Library Keyboard Encoding  
--  
-- Xlbp_Keyboard_Encoding - Control over which key means what.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    procedure X_Change_Keyboard_Mapping (Display : X_Display;  
                                         Mapping : X_Key_Sym_Array_2d);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Mapping - Specifies the new mapping.  
--  
-- Changes the symbols for the specified key codes and modifiers.  Mapping  
-- has two dimensions.  The first dimension is by key code.  The second  
-- dimension is by key modifier.  
------------------------------------------------------------------------------  

    procedure X_Delete_Modifier_Map_Entry (Map      : in out X_Modifier_Keymap;  
                                           Key_Code :        X_Key_Code;  
                                           Modifier :        X_Key_Modifier);  
------------------------------------------------------------------------------  
--  Map       - Specifies the map to modify.  
--  Key_Code  - Specifies the key code.  
--  Modifier  - Specifies the modifier.  
--  
-- Removes the specified key code from the set that controls the specified  
-- modifier.  
------------------------------------------------------------------------------  

    procedure X_Display_Key_Codes (Display      :     X_Display;  
                                   Min_Key_Code : out X_Key_Code;  
                                   Max_Key_Code : out X_Key_Code);  
------------------------------------------------------------------------------  
--  Display         - Specifies the display to use  
--  Min_Key_Code    - Receives the minimum key code for this keyboard  
--  Max_Key_Code    - Receives the maximum key code for this keyboard  
--  
-- Called to obtain the min/max key codes that can be generated by the  
-- keyboard attached to this connection.  
------------------------------------------------------------------------------  

    procedure X_Free_Modifier_Map (Map : in out X_Modifier_Keymap)  
        renames Free_X_Modifier_Keymap;  
------------------------------------------------------------------------------  
--  Map - The map to free.  
--  
-- Frees the storage taken up by the keymap.  
------------------------------------------------------------------------------  

    function X_Get_Keyboard_Mapping  
                (Display        : X_Display;  
                 First_Key_Code : X_Key_Code;  
                 Last_Key_Code  : X_Key_Code) return X_Key_Sym_List_2d;  
------------------------------------------------------------------------------  
--  Display         - Specifies the display to use.  
--  First_Key_Code  - Specifies the first key code to be returned.  
--  Last_Key_Code   - Specifies the last key code to be returned.  
--  
-- Returns the symbols for the specified number of key codes.  The value  
-- specified for First_Key_Code must be greater than or equal to the  
-- Min_Key_Code as recorded in the X_Display.  The return value has two  
-- dimensions.  The first dimension is by key code.  The second dimension  
-- is by key modifier.  
--  
-- Free the mapping after use with the Free_X_Key_Sym_List_2d routine.  
------------------------------------------------------------------------------  

    function X_Get_Modifier_Mapping  
                (Display : X_Display) return X_Modifier_Keymap;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  
-- Returns a newly created X_Modifier_Keymap that contains the keys being  
-- used as modifiers.  The structure should be freed after use with  
-- X_Free_Modifier_Map.  
------------------------------------------------------------------------------  

    procedure X_Insert_Modifier_Map_Entry (Map      : in out X_Modifier_Keymap;  
                                           Key_Code :        X_Key_Code;  
                                           Modifier :        X_Key_Modifier);  
------------------------------------------------------------------------------  
--  Map       - Specifies the map to modify.  
--  Key_Code  - Specifies the key code.  
--  Modifier  - Specifies the modifier.  
--  
-- Adds the specified key code to the set that controls the specified  
-- modifier.  
------------------------------------------------------------------------------  

    function X_New_Modifier_Map  
                (Keys_Per_Modifier : U_Char) return X_Modifier_Keymap;  
------------------------------------------------------------------------------  
--  Keys_Per_Modifier   - Specifies the maximum number of key codes assigned to  
--                        any of the modifiers in the map.  
--  
-- Creates a new X_Modifier_Keymap structure.  
------------------------------------------------------------------------------  

    function X_Set_Modifier_Mapping  
                (Display      : X_Display;  
                 Modifier_Map : X_Modifier_Keymap) return X_Mapping_Status;  
------------------------------------------------------------------------------  
--  Display       - Specifies the display to use.  
--  Modifier_Map  - Specifies the map to set.  
--  
-- Sets the key codes of the keys, if any, that are to be used as modifiers.  
-- A None_X_Modifier_Map value indicates that no key is to be used.  No two key  
-- codes can have the same numeric value.  
------------------------------------------------------------------------------  

end Xlbp_Keyboard_Encoding;  

