with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Rm;  
use Xlbt_Rm;  
with Xlbt_String;  
use Xlbt_String;  

with Xlbp_Rm_Quark;  
use Xlbp_Rm_Quark;  

package body Xlbp_Rm_Name is  
------------------------------------------------------------------------------  
-- X Library Resource Manager - Classes, Names, Representations  
--  
-- Xlbp_Rm_Name - Manager of application/window/object resource names  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1987 - 1989 by Digital Equipment Corporation, Maynard, Mass.  
-- Copyright 1987 - 1989 by Massachusetts Institute of Technology,  
--                          Cambridge, Massachusetts.  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of Digital, MIT, or Rational  
-- not be used in advertising or publicity pertaining to distribution of  
-- the software without specific, written prior permission.  
--  
-- Digital, MIT, and Rational disclaim all warranties with regard to this  
-- software, including all implied warranties of merchantability and fitness,  
-- in no event shall Digital, MIT, or Rational be liable for any special,  
-- indirect or consequential damages or any damages whatsoever resulting from  
-- loss of use, data or profits, whether in an action of contract, negligence  
-- or other tortious action, arising out of or in connection with the use or  
-- performance of this software.  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- * Date      - /Name/ Comment  
-- *  
-- * 23-OCT-90 - /DRK/ Updated calls to X_Rm_String_To_Quark to enforce  
-- *           -  restrictions on characters allowed in component names.  
-- * 21-JAN-91 - /GEB/ Add the @_To_String_Pointer and the @_String_Pointer_To@  
-- *           -  routines for Xt.  
-- *****************************************************************************  

--
------------------------------------------------------------------------------  
-- X_Rm_Class routines  
------------------------------------------------------------------------------  

    function X_Rm_Class_To_String (Class : X_Rm_Class) return X_String is  
    begin  
        return X_Rm_Quark_To_String (X_Rm_Quark (Class));  
    end X_Rm_Class_To_String;  

    function X_Rm_String_To_Class  
                (Str      : X_String;  
                 Validate : Boolean := True) return X_Rm_Class is  
    begin  
        return X_Rm_Class (X_Rm_String_To_Quark (Str, Validate));  
    end X_Rm_String_To_Class;  

    procedure X_Rm_String_To_Class (Str      :     X_String;  
                                    Ptr      : out X_String_Pointer;  
                                    Class    : out X_Rm_Class;  
                                    Validate :     Boolean := True) is  
        Qrk : X_Rm_Quark;  
    begin  
        X_Rm_String_To_Quark (Str, Ptr, Qrk, Validate);  
        Class := X_Rm_Class (Qrk);  
    end X_Rm_String_To_Class;  

    function X_Rm_Class_To_String_Pointer  
                (Class : X_Rm_Class) return X_String_Pointer is  
    begin  
        return X_Rm_Quark_To_String_Pointer (X_Rm_Quark (Class));  
    end X_Rm_Class_To_String_Pointer;  

    procedure X_Rm_String_Pointer_To_Class (Str      : in out X_String_Pointer;  
                                            Class    : out X_Rm_Class;  
                                            Validate : Boolean := True) is  
        Qrk : X_Rm_Quark;  
    begin  
        X_Rm_String_Pointer_To_Quark (Str, Qrk, Validate);  
        Class := X_Rm_Class (Qrk);  
    end X_Rm_String_Pointer_To_Class;  

    function X_Rm_Unique_Class return X_Rm_Class is  
    begin  
        return X_Rm_Class (X_Rm_Unique_Quark);  
    end X_Rm_Unique_Class;  

--
------------------------------------------------------------------------------  
-- X_Rm_Name routines  
------------------------------------------------------------------------------  

    function X_Rm_Name_To_String (Str : X_Rm_Name) return X_String is  
    begin  
        return X_Rm_Quark_To_String (X_Rm_Quark (Str));  
    end X_Rm_Name_To_String;  

    function X_Rm_String_To_Name  
                (Str      : X_String;  
                 Validate : Boolean := True) return X_Rm_Name is  
    begin  
        return X_Rm_Name (X_Rm_String_To_Quark (Str, Validate));  
    end X_Rm_String_To_Name;  

    procedure X_Rm_String_To_Name (Str      :     X_String;  
                                   Ptr      : out X_String_Pointer;  
                                   Name     : out X_Rm_Name;  
                                   Validate :     Boolean := True) is  
        Qrk : X_Rm_Quark;  
    begin  
        X_Rm_String_To_Quark (Str, Ptr, Qrk, Validate);  
        Name := X_Rm_Name (Qrk);  
    end X_Rm_String_To_Name;  

    function X_Rm_Name_To_String_Pointer  
                (Name : X_Rm_Name) return X_String_Pointer is  
    begin  
        return X_Rm_Quark_To_String_Pointer (X_Rm_Quark (Name));  
    end X_Rm_Name_To_String_Pointer;  

    procedure X_Rm_String_Pointer_To_Name (Str      : in out X_String_Pointer;  
                                           Name     : out    X_Rm_Name;  
                                           Validate :        Boolean := True) is  
        Qrk : X_Rm_Quark;  
    begin  
        X_Rm_String_Pointer_To_Quark (Str, Qrk, Validate);  
        Name := X_Rm_Name (Qrk);  
    end X_Rm_String_Pointer_To_Name;  

    function X_Rm_Unique_Name return X_Rm_Name is  
    begin  
        return X_Rm_Name (X_Rm_Unique_Quark);  
    end X_Rm_Unique_Name;  

--
------------------------------------------------------------------------------  
-- X_Rm_Representation routines  
------------------------------------------------------------------------------  

    function X_Rm_Representation_To_String  
                (Rep : X_Rm_Representation) return X_String is  
    begin  
        return X_Rm_Quark_To_String (X_Rm_Quark (Rep));  
    end X_Rm_Representation_To_String;  

    function X_Rm_String_To_Representation  
                (Str      : X_String;  
                 Validate : Boolean := True) return X_Rm_Representation is  
    begin  
        return X_Rm_Representation (X_Rm_String_To_Quark (Str, Validate));  
    end X_Rm_String_To_Representation;  

    procedure X_Rm_String_To_Representation  
                 (Str            :     X_String;  
                  Ptr            : out X_String_Pointer;  
                  Representation : out X_Rm_Representation;  
                  Validate       :     Boolean := True) is  
        Qrk : X_Rm_Quark;  
    begin  
        X_Rm_String_To_Quark (Str, Ptr, Qrk, Validate);  
        Representation := X_Rm_Representation (Qrk);  
    end X_Rm_String_To_Representation;  

    function X_Rm_Representation_To_String_Pointer  
                (Representation : X_Rm_Representation)  
                return X_String_Pointer is  
    begin  
        return X_Rm_Quark_To_String_Pointer (X_Rm_Quark (Representation));  
    end X_Rm_Representation_To_String_Pointer;  

    procedure X_Rm_String_Pointer_To_Representation  
                 (Str            : in out X_String_Pointer;  
                  Representation : out    X_Rm_Representation;  
                  Validate       :        Boolean := True) is  
        Qrk : X_Rm_Quark;  
    begin  
        X_Rm_String_Pointer_To_Quark (Str, Qrk, Validate);  
        Representation := X_Rm_Representation (Qrk);  
    end X_Rm_String_Pointer_To_Representation;  

    function X_Rm_Unique_Representation return X_Rm_Representation is  
    begin  
        return X_Rm_Representation (X_Rm_Unique_Quark);  
    end X_Rm_Unique_Representation;  

--
end Xlbp_Rm_Name;  

