with Unchecked_Deallocation;  

with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Rm;  
use Xlbt_Rm;  
with Xlbt_String;  
use Xlbt_String;  
with Xlbt_Univ_Ptr;  
use Xlbt_Univ_Ptr;  

package Xlbt_Context_Manager3 is  
------------------------------------------------------------------------------  
-- X Library Context Manager - Full User Interface  
--  
-- Xlbt_Context_Manager3 - Types dealing with Context Management  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1987 - 1989 by Digital Equipment Corporation, Maynard, Mass.  
-- Copyright 1987 - 1989 by Massachusetts Institute of Technology,  
--                          Cambridge, Massachusetts.  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of Digital, MIT, or Rational  
-- not be used in advertising or publicity pertaining to distribution of  
-- the software without specific, written prior permission.  
--  
-- Digital, MIT, and Rational disclaim all warranties with regard to this  
-- software, including all implied warranties of merchantability and fitness,  
-- in no event shall Digital, MIT, or Rational be liable for any special,  
-- indirect or consequential damages or any damages whatsoever resulting from  
-- loss of use, data or profits, whether in an action of contract, negligence  
-- or other tortious action, arising out of or in connection with the use or  
-- performance of this software.  
------------------------------------------------------------------------------  

--/ if OLD_CONTEXT_MANAGER then  
--
  
------------------------------------------------------------------------------  
-- X_Associative_Returns  
------------------------------------------------------------------------------  
 
    type X_Associative_Returns is  
       (Xc_Success,    -- No error.  
        Xc_No_Mem,     -- Out of memory  
        Xc_No_Ent);    -- No entry in table  
 
------------------------------------------------------------------------------  
-- X_Context  
------------------------------------------------------------------------------  
 
    type X_Context is new X_Rm_Quark;  
    None_X_Context : constant X_Context := X_Context (None_X_Rm_Quark);  
 
------------------------------------------------------------------------------  
-- X_Context_Table_Entry - Context/Window Pair Cache  
------------------------------------------------------------------------------  
 
    type X_Context_Table_Entry_Rec;  
    type X_Context_Table_Entry is access X_Context_Table_Entry_Rec;  
 
--/ if Enable_Deallocation then  
    pragma Enable_Deallocation (X_Context_Table_Entry);  
--/ end if;  
 
    type X_Context_Table_Entry_Rec is  
        record  
            Window  : X_Window;  
            Context : X_Context;  
            Data    : X_Rm_Value;  
            Next    : X_Context_Table_Entry;  
        end record;  
 
    procedure Free_X_Context_Table_Entry is  
       new Unchecked_Deallocation (X_Context_Table_Entry_Rec,  
                                   X_Context_Table_Entry);  
 
------------------------------------------------------------------------------  
-- X_Context_Hash_Table - Context/Window Pair Cache  
-- Primes: 13, 31, 59, 101, 211  
------------------------------------------------------------------------------  
 
    type X_Context_Hash_Table is  
       array (S_Natural range 0 .. 30)  -- 31 is a prime number.  
           of X_Context_Table_Entry;  
 
    type X_Context_Hash_List is access X_Context_Hash_Table;  
 
--/ if Enable_Deallocation then  
    pragma Enable_Deallocation (X_Context_Hash_List);  
--/ end if;  
 
    None_X_Context_Hash_Table : constant X_Context_Hash_Table :=  
       (others => null);  
    None_X_Context_Hash_List  : constant X_Context_Hash_List  := null;  
 
    procedure Free_X_Context_Hash_List (List : in out X_Context_Hash_List);  
 
    package Univ_X_Context_Hash_List is  
       new X_Universal_Pointer_Conversion (X_Context_Hash_Table,  
                                           X_Context_Hash_List,  
                                           Free_X_Context_Hash_List);  
 
--/ end if;  
--
end Xlbt_Context_Manager3;  

