with Xlbt_Display3;  
with Xlbt_Event3;  

package Xlbt_Event is  
------------------------------------------------------------------------------  
-- X Library Protocol Event Codes and Packets - Normal User Interface  
--  
-- Xlbt_Event - Record definitions of the protocol event packets  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    package D3 renames Xlbt_Display3;  
    package E3 renames Xlbt_Event3;  

------------------------------------------------------------------------------  
-- X_Event_Code  
--  
--  X_Event_Code_8 - used in all X protocol structures  
--  X_Event_Code   - used in all user interfaces  
------------------------------------------------------------------------------  

    subtype X_Event_Code_8 is E3.X_Event_Code_8;  
    subtype X_Event_Code   is E3.X_Event_Code;  

    subtype Xec is X_Event_Code;  

    Error_Event       : constant Xec := E3.Error_Event; -- Error notice from  
                                                        --  server.  
    Reply_Event       : constant Xec := E3.Reply_Event; -- Simple reply from  
                                                        --  server  
    Key_Press         : constant Xec := E3.Key_Press;  
    Key_Release       : constant Xec := E3.Key_Release;  
    Button_Press      : constant Xec := E3.Button_Press;  
    Button_Release    : constant Xec := E3.Button_Release;  
    Motion_Notify     : constant Xec := E3.Motion_Notify;  
    Enter_Notify      : constant Xec := E3.Enter_Notify;  
    Leave_Notify      : constant Xec := E3.Leave_Notify;  
    Focus_In          : constant Xec := E3.Focus_In;  
    Focus_Out         : constant Xec := E3.Focus_Out;  
    Keymap_Notify     : constant Xec := E3.Keymap_Notify;  
    Expose            : constant Xec := E3.Expose;  
    Graphics_Expose   : constant Xec := E3.Graphics_Expose;  
    No_Expose         : constant Xec := E3.No_Expose;  
    Visibility_Notify : constant Xec := E3.Visibility_Notify;  
    Create_Notify     : constant Xec := E3.Create_Notify;  
    Destroy_Notify    : constant Xec := E3.Destroy_Notify;  
    Unmap_Notify      : constant Xec := E3.Unmap_Notify;  
    Map_Notify        : constant Xec := E3.Map_Notify;  
    Map_Request       : constant Xec := E3.Map_Request;  
    Reparent_Notify   : constant Xec := E3.Reparent_Notify;  
    Configure_Notify  : constant Xec := E3.Configure_Notify;  
    Configure_Request : constant Xec := E3.Configure_Request;  
    Gravity_Notify    : constant Xec := E3.Gravity_Notify;  
    Resize_Request    : constant Xec := E3.Resize_Request;  
    Circulate_Notify  : constant Xec := E3.Circulate_Notify;  
    Circulate_Request : constant Xec := E3.Circulate_Request;  
    Property_Notify   : constant Xec := E3.Property_Notify;  
    Selection_Clear   : constant Xec := E3.Selection_Clear;  
    Selection_Request : constant Xec := E3.Selection_Request;  
    Selection_Notify  : constant Xec := E3.Selection_Notify;  
    Colormap_Notify   : constant Xec := E3.Colormap_Notify;  
    Client_Message    : constant Xec := E3.Client_Message;  
    Mapping_Notify    : constant Xec := E3.Mapping_Notify;  
    Last_Event        : constant Xec := E3.Last_Event; -- Must be bigger than  
                                                       -- any other predefined  
                                                       -- event number  

    function "=" (A, B : X_Event_Code_8) return Boolean renames E3."=";  

    None_X_Event_Code : constant X_Event_Code := E3.None_X_Event_Code;  

------------------------------------------------------------------------------  
-- X_Event_Queuing  
--  
--      Queued_Already  
--      Queued_After_Reading  
--      Queued_After_Flush  
------------------------------------------------------------------------------  

    subtype X_Event_Queuing is E3.X_Event_Queuing;  

    Queued_Already       : constant X_Event_Queuing := E3.Queued_Already;  
    Queued_After_Reading : constant X_Event_Queuing := E3.Queued_After_Reading;  
    Queued_After_Flush   : constant X_Event_Queuing := E3.Queued_After_Flush;  

    function "=" (A, B : X_Event_Queuing) return Boolean renames E3."=";  

------------------------------------------------------------------------------  
-- This array can be used with an event type to determine the mask bits  
-- that could have generated it.  
------------------------------------------------------------------------------  

    subtype X_Event_To_Mask_Array is E3.X_Event_To_Mask_Array;  

    X_Event_To_Mask : X_Event_To_Mask_Array renames E3.X_Event_To_Mask;  

------------------------------------------------------------------------------  
-- X Protocol Specific Event Types  
------------------------------------------------------------------------------  

    subtype X_Key_Press_Event         is D3.X_Event (Key_Press);  
    subtype X_Key_Release_Event       is D3.X_Event (Key_Release);  
    subtype X_Button_Press_Event      is D3.X_Event (Button_Press);  
    subtype X_Button_Release_Event    is D3.X_Event (Button_Release);  
    subtype X_Motion_Notify_Event     is D3.X_Event (Motion_Notify);  
    subtype X_Enter_Notify_Event      is D3.X_Event (Enter_Notify);  
    subtype X_Leave_Notify_Event      is D3.X_Event (Leave_Notify);  
    subtype X_Focus_In_Event          is D3.X_Event (Focus_In);  
    subtype X_Focus_Out_Event         is D3.X_Event (Focus_Out);  
    subtype X_Keymap_Notify_Event     is D3.X_Event (Keymap_Notify);  
    subtype X_Expose_Event            is D3.X_Event (Expose);  
    subtype X_Graphics_Expose_Event   is D3.X_Event (Graphics_Expose);  
    subtype X_No_Expose_Event         is D3.X_Event (No_Expose);  
    subtype X_Visibility_Notify_Event is D3.X_Event (Visibility_Notify);  
    subtype X_Create_Notify_Event     is D3.X_Event (Create_Notify);  
    subtype X_Destroy_Notify_Event    is D3.X_Event (Destroy_Notify);  
    subtype X_Unmap_Notify_Event      is D3.X_Event (Unmap_Notify);  
    subtype X_Map_Notify_Event        is D3.X_Event (Map_Notify);  
    subtype X_Map_Request_Event       is D3.X_Event (Map_Request);  
    subtype X_Reparent_Notify_Event   is D3.X_Event (Reparent_Notify);  
    subtype X_Configure_Notify_Event  is D3.X_Event (Configure_Notify);  
    subtype X_Configure_Request_Event is D3.X_Event (Configure_Request);  
    subtype X_Gravity_Notify_Event    is D3.X_Event (Gravity_Notify);  
    subtype X_Resize_Request_Event    is D3.X_Event (Resize_Request);  
    subtype X_Circulate_Notify_Event  is D3.X_Event (Circulate_Notify);  
    subtype X_Circulate_Request_Event is D3.X_Event (Circulate_Request);  
    subtype X_Property_Notify_Event   is D3.X_Event (Property_Notify);  
    subtype X_Selection_Clear_Event   is D3.X_Event (Selection_Clear);  
    subtype X_Selection_Request_Event is D3.X_Event (Selection_Request);  
    subtype X_Selection_Notify_Event  is D3.X_Event (Selection_Notify);  
    subtype X_Colormap_Notify_Event   is D3.X_Event (Colormap_Notify);  
    subtype X_Client_Message_Event    is D3.X_Event (Client_Message);  
    subtype X_Mapping_Notify_Event    is D3.X_Event (Mapping_Notify);  

end Xlbt_Event;  

