with Unchecked_Deallocation;  

with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Error3;  
use Xlbt_Error3;  
with Xlbt_Event3;  
use Xlbt_Event3;  
with Xlbt_Extension4;  
use Xlbt_Extension4;  
with Xlbt_Proc_Var;  
use Xlbt_Proc_Var;  
with Xlbt_Request;  
use Xlbt_Request;  
with Xlbt_String;  
use Xlbt_String;  
with Xlbt_Univ_Ptr;  
use Xlbt_Univ_Ptr;  

package Xlbt_Extension3 is  
------------------------------------------------------------------------------  
-- X Library Extensions - Full Interface Part 1  
--  
-- Xlbt_Extension3 - Types dealing with extensions to the X Library and to  
-- the X Protocol.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- X_Ext_Codes - Extension Codes contains structures used by the extension  
--               mechanism.  
------------------------------------------------------------------------------  

    type X_Ext_Codes_Rec is             -- public to extension,  
        record                          --   structure cannot be changed  
            Extension    :              -- extension number  
               X_Extension_Number := 0;  
            Major_Opcode :              -- major op-code assigned by server  
               X_Request_Code     := None_X_Request_Code;  
            First_Event  :              -- first Event number for the extension  
               X_Event_Code       := None_X_Event_Code;  
            First_Error  :              -- first error number for the extension  
               X_Error_Code       := None_X_Error_Code;  
        end record;  

    type X_Ext_Codes is access X_Ext_Codes_Rec;  

--/ if Enable_Deallocation then  
    pragma Enable_Deallocation (X_Ext_Codes);  
--/ end if;  

    None_X_Ext_Codes : constant X_Ext_Codes := null;  

    procedure Free_X_Ext_Codes is new Unchecked_Deallocation (X_Ext_Codes_Rec,  
                                                              X_Ext_Codes);  

------------------------------------------------------------------------------  
-- X_Extension - This structure is private to the library.  
------------------------------------------------------------------------------  

    type X_Extension_Rec;  
    type X_Extension is access X_Extension_Rec;  

--/ if Enable_Deallocation then  
    pragma Enable_Deallocation (X_Extension);  
--/ end if;  

    type X_Extension_Rec is             -- private to extension mechanism  
        record  
            Next : X_Extension := null;  
            ----Next in list  
            Name : X_String_Pointer := None_X_String_Pointer;  
            ----Name of this extension.  
            Codes : X_Ext_Codes := None_X_Ext_Codes;  
            ----Public information,  
            Create_Gc : X_Procedure_Variable := None_X_Procedure_Variable;  
            ----X_Display_Gc_Extension.Pv  
            ----Routine to call when GC created  
            Copy_Gc : X_Procedure_Variable := None_X_Procedure_Variable;  
            ----X_Display_Gc_Extension.Pv  
            ----Routine to call when GC copied  
            Flush_Gc : X_Procedure_Variable := None_X_Procedure_Variable;  
            ----X_Display_Gc_Extension.Pv  
            ----Routine to call when GC flushed  
            Free_Gc : X_Procedure_Variable := None_X_Procedure_Variable;  
            ----X_Display_Gc_Extension.Pv  
            ----Routine to call when GC freed  
            Create_Font : X_Procedure_Variable := None_X_Procedure_Variable;  
            ----X_Display_Font_Extension.Pv  
            ----Routine to call when Font created  
            Free_Font : X_Procedure_Variable := None_X_Procedure_Variable;  
            ----X_Display_Font_Extension.Pv  
            ----Routine to call when Font freed  
            Close_Display : X_Procedure_Variable := None_X_Procedure_Variable;  
            ----X_Close_Display_Extension.Pv  
            ----Routine to call when connection closed  
            Error : X_Procedure_Variable := None_X_Procedure_Variable;  
            ----X_Error_Extension.Pv  
            ----Who to call when an error occurs  
            Error_String : X_Procedure_Variable := None_X_Procedure_Variable;  
            ----X_Error_String_Extension.Pv  
            ----Routine to supply error string  
        end record;  

    procedure Free_X_Extension is new Unchecked_Deallocation (X_Extension_Rec,  
                                                              X_Extension);  
    ----Free a single record.  

    procedure Free_X_Extension_List (List : in out X_Extension);  
    ----Free a linked list of records.  

    None_X_Extension : constant X_Extension := null;  

end Xlbt_Extension3;  

