with Unchecked_Deallocation;  

with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Proc_Var;  
use Xlbt_Proc_Var;  
with Xlbt_Univ_Ptr;  
use Xlbt_Univ_Ptr;  

package Xlbt_Extension4 is  
------------------------------------------------------------------------------  
-- X Library Extensions - Full Interface Part 2  
--  
-- Xlbt_Extension4 - Types dealing with extensions to the X Library and to  
-- the X Protocol.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- X_Extension_Number - each extension gets a unique (to this program) number  
------------------------------------------------------------------------------  

    type X_Extension_Number is new S_Long;  

--/ if Length_Clauses then  
    for X_Extension_Number'Size use 32;  
--/ end if;  

------------------------------------------------------------------------------  
-- X_Ext_Data - Extensions need a way to hang private data on some structures.  
------------------------------------------------------------------------------  

    type X_Ext_Data_Rec;  
    type X_Ext_Data is access X_Ext_Data_Rec;  

--/ if Enable_Deallocation then  
    pragma Enable_Deallocation (X_Ext_Data);  
--/ end if;  

    type X_Ext_Data_Rec is  
        record  
            Number       :        -- number returned by Register_Extension  
               X_Extension_Number   := 0;  
            Free_Private :        -- called to free private storage  
               X_Procedure_Variable := None_X_Procedure_Variable;  
            ----X_Free_Extension_Data.Pv  
            Private_Data :        -- private extension storage  
               X_Universal_Pointer := None_X_Universal_Pointer;  
            Next :                -- next item on list of data for structure  
               X_Ext_Data := null;  
        end record;  

    procedure Free_X_Ext_Data is new Unchecked_Deallocation (X_Ext_Data_Rec,  
                                                             X_Ext_Data);  
    ----Free a single X_Ext_Data record.  

    procedure Free_X_Ext_Data_List (List : in out X_Ext_Data);  
    ----Free the linked list of X_Ext_Data records.  

    None_X_Ext_Data : constant X_Ext_Data := null;  

end Xlbt_Extension4;  

