with Unchecked_Deallocation;  

with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Xlbt_Graphics3 is  
------------------------------------------------------------------------------  
-- X Library Graphics - Full Interface  
--  
-- Xlbt_Graphics3 - Draw various types of things on a window  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- X_Arc - Description of Arcs  
------------------------------------------------------------------------------  

    type X_Arc is  
        record  
            X      : S_Short;  
            Y      : S_Short;  
            Width  : U_Short;  
            Height : U_Short;  
            Angle1 : S_Short;  
            Angle2 : S_Short;  
        end record;  

--/ if Length_Clauses then  
    for X_Arc'Size use 3 * 32;  
--/ end if;  

    type X_Arc_Array is array (S_Natural range <>) of X_Arc;  
    type X_Arc_List  is access X_Arc_Array;  

--/ if Pack then  
    pragma Pack (X_Arc_Array);  
--/ end if;  
--/ if Enable_Deallocation then  
    pragma Enable_Deallocation (X_Arc_List);  
--/ end if;  

    None_X_Arc      : constant X_Arc      := (0, 0, 0, 0, 0, 0);  
    None_X_Arc_List : constant X_Arc_List := null;  

    procedure Free_X_Arc_List is  
       new Unchecked_Deallocation (X_Arc_Array,  
                                   X_Arc_List);  

------------------------------------------------------------------------------  
-- X_Coordinate_Mode for drawing routines  
------------------------------------------------------------------------------  

    type X_Coordinate_Mode is  
       (Coord_Mode_Origin,      -- relative to the origin  
        Coord_Mode_Previous);   -- relative to previous point  

--/ if Length_Clauses then  
    for X_Coordinate_Mode'Size use 8;  
--/ end if;  

    None_X_Coordinate_Mode : constant X_Coordinate_Mode :=  
       X_Coordinate_Mode'Val (0);  

------------------------------------------------------------------------------  
-- X_Point - Points in X  
------------------------------------------------------------------------------  

    type X_Point is  
        record  
            X : S_Short;  
            Y : S_Short;  
        end record;  

--/ if Length_Clauses then  
    for X_Point'Size use 32;  
--/ end if;  

    type X_Point_Array is array (S_Natural range <>) of X_Point;  
    type X_Point_List  is access X_Point_Array;  

--/ if Pack then  
    pragma Pack (X_Point_Array);  
--/ end if;  
--/ if Enable_Deallocation then  
    pragma Enable_Deallocation (X_Point_List);  
--/ end if;  

    None_X_Point      : constant X_Point      := (0, 0);  
    None_X_Point_List : constant X_Point_List := null;  

    procedure Free_X_Point_List is  
       new Unchecked_Deallocation (X_Point_Array,  
                                   X_Point_List);  

------------------------------------------------------------------------------  
-- X_Polygon_Shape  
------------------------------------------------------------------------------  

    type X_Polygon_Shape is (Complex,   -- paths may intersect  
                             Nonconvex, -- no paths intersect, but not convex  
                             Convex);   -- wholly convex  

--/ if Length_Clauses then  
    for X_Polygon_Shape'Size use 8;  
--/ end if;  

    None_X_Polygon_Shape : constant X_Polygon_Shape := X_Polygon_Shape'Val (0);  

------------------------------------------------------------------------------  
-- X_Rectangle - Rectangle  
------------------------------------------------------------------------------  

    type X_Rectangle is  
        record  
            X      : S_Short;  
            Y      : S_Short;  
            Width  : U_Short;  
            Height : U_Short;  
        end record;  

--/ if Length_Clauses then  
    for X_Rectangle'Size use 2 * 32;  
--/ end if;  

    type X_Rectangle_Array is array (S_Natural range <>) of X_Rectangle;  
    type X_Rectangle_List  is access X_Rectangle_Array;  

--/ if Pack then  
    pragma Pack (X_Rectangle_Array);  
--/ end if;  
--/ if Enable_Deallocation then  
    pragma Enable_Deallocation (X_Rectangle_List);  
--/ end if;  

    None_X_Rectangle      : constant X_Rectangle      := (0, 0, 0, 0);  
    None_X_Rectangle_List : constant X_Rectangle_List := null;  

    procedure Free_X_Rectangle_List is  
       new Unchecked_Deallocation (X_Rectangle_Array,  
                                   X_Rectangle_List);  

------------------------------------------------------------------------------  
-- X_Segment - Line Segment  
------------------------------------------------------------------------------  

    type X_Segment is  
        record  
            X1 : S_Short;  
            Y1 : S_Short;  
            X2 : S_Short;  
            Y2 : S_Short;  
        end record;  

--/ if Length_Clauses then  
    for X_Segment'Size use 2 * 32;  
--/ end if;  

    type X_Segment_Array is array (S_Natural range <>) of X_Segment;  
    type X_Segment_List  is access X_Segment_Array;  

--/ if Pack then  
    pragma Pack (X_Segment_Array);  
--/ end if;  
--/ if Enable_Deallocation then  
    pragma Enable_Deallocation (X_Segment_List);  
--/ end if;  

    None_X_Segment      : constant X_Segment      := (0, 0, 0, 0);  
    None_X_Segment_List : constant X_Segment_List := null;  

    procedure Free_X_Segment_List is  
       new Unchecked_Deallocation (X_Segment_Array,  
                                   X_Segment_List);  

end Xlbt_Graphics3;  

