with Xlbt_Key;  
use Xlbt_Key;  

package Xlbt_Key_Defs_Misc is  
------------------------------------------------------------------------------  
-- X Library Key Symbol Definitions - Misc  
--  
-- Xlbt_Key_Defs_Misc - Definitions of the various pre-defined X Key Symbols  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    Xk_Void_Symbol : constant X_Key_Sym := 16#FFFFFF#;-- void symbol  

------------------------------------------------------------------------------  
-- TTY Functions, cleverly chosen to map to ascii, for convenience of  
-- programming, but could have been arbitrary (at the cost of lookup  
-- tables in client code.  
------------------------------------------------------------------------------  

    Xk_Backspace   : constant X_Key_Sym := 16#FF08#; -- back space, back char  
    Xk_Tab         : constant X_Key_Sym := 16#FF09#;  
    Xk_Linefeed    : constant X_Key_Sym := 16#FF0A#; -- Linefeed, LF  
    Xk_Clear       : constant X_Key_Sym := 16#FF0B#;  
    Xk_Return      : constant X_Key_Sym := 16#FF0D#; -- Return, enter  
    Xk_Pause       : constant X_Key_Sym := 16#FF13#; -- Pause, hold  
    Xk_Scroll_Lock : constant X_Key_Sym := 16#FF14#; -- Scroll-Lock  
    Xk_Escape      : constant X_Key_Sym := 16#FF1B#;  
    Xk_Delete      : constant X_Key_Sym := 16#FFFF#; -- Delete, rubout  

------------------------------------------------------------------------------  
---- International & multi-key character composition  
------------------------------------------------------------------------------  

    Xk_Multi_Key   : constant X_Key_Sym := 16#FF20#; -- Multi-key char compose  
    Xk_Kanji       : constant X_Key_Sym := 16#FF21#; -- Kanji, Kanji convert  
    Xk_Muhenkan    : constant X_Key_Sym := 16#FF22#; -- Cancel Conversion  
    Xk_Henkan_Mode : constant X_Key_Sym := 16#FF23#; -- Start/Stop Conversion  
    Xk_Henkan      : constant X_Key_Sym := 16#FF23#; -- Alias for Henkan_Mode  
    Xk_Romaji      : constant X_Key_Sym := 16#FF24#; -- to Romaji  
    Xk_Hiragana    : constant X_Key_Sym := 16#FF25#; -- to Hiragana  
    Xk_Katakana    : constant X_Key_Sym := 16#FF26#; -- to Katakana  
    Xk_Hiragana_Katakana  
        : constant X_Key_Sym            := 16#FF27#; -- Hiragan/Katakana toggle  
    Xk_Zenkaku     : constant X_Key_Sym := 16#FF28#; -- to Zenkaku  
    Xk_Hankaku     : constant X_Key_Sym := 16#FF29#; -- to Hankaku  
    Xk_Zenkaku_Hankaku  
        : constant X_Key_Sym            := 16#FF2A#; -- Zenkaku/Hankaku toggle  
    Xk_Touroku     : constant X_Key_Sym := 16#FF2B#; -- Add to Dictionary  
    Xk_Massyo      : constant X_Key_Sym := 16#FF2C#; -- Delete from Dictionary  
    Xk_Kana_Lock   : constant X_Key_Sym := 16#FF2D#; -- Kana toggle/Shift-Lock  
    Xk_Kana_Shift  : constant X_Key_Sym := 16#FF2E#; -- Kana Shift  
    Xk_Eisu_Lock   : constant X_Key_Sym := 16#FF2F#; -- Alpha-num. Shift-Lock  
    Xk_Eisu_Toggle : constant X_Key_Sym := 16#FF30#; -- Alphanumeric toggle  

------------------------------------------------------------------------------  
---- Cursor control & motion  
------------------------------------------------------------------------------  

    Xk_Home  : constant X_Key_Sym := 16#FF50#;  
    Xk_Left  : constant X_Key_Sym := 16#FF51#;  -- Move left, left arrow  
    Xk_Up    : constant X_Key_Sym := 16#FF52#;  -- Move up, up arrow  
    Xk_Right : constant X_Key_Sym := 16#FF53#;  -- Move right, right arrow  
    Xk_Down  : constant X_Key_Sym := 16#FF54#;  -- Move down, down arrow  
    Xk_Prior : constant X_Key_Sym := 16#FF55#;  -- Prior, previous  
    Xk_Next  : constant X_Key_Sym := 16#FF56#;  -- Next  
    Xk_End   : constant X_Key_Sym := 16#FF57#;  -- EOL  
    Xk_Begin : constant X_Key_Sym := 16#FF58#;  -- BOL  

------------------------------------------------------------------------------  
---- Misc Functions  
------------------------------------------------------------------------------  

    Xk_Select        : constant X_Key_Sym := 16#FF60#; -- Select, mark  
    Xk_Print         : constant X_Key_Sym := 16#FF61#;  
    Xk_Execute       : constant X_Key_Sym := 16#FF62#; -- Execute, run, do  
    Xk_Insert        : constant X_Key_Sym := 16#FF63#; -- Insert,insert here  
    Xk_Undo          : constant X_Key_Sym := 16#FF65#; -- Undo, oops  
    Xk_Redo          : constant X_Key_Sym := 16#FF66#; -- redo, again  
    Xk_Menu          : constant X_Key_Sym := 16#FF67#;  
    Xk_Find          : constant X_Key_Sym := 16#FF68#; -- Find, search  
    Xk_Cancel        : constant X_Key_Sym := 16#FF69#; -- Cancel,stop,abort  
    Xk_Help          : constant X_Key_Sym := 16#FF6A#; -- Help, ?  
    Xk_Break         : constant X_Key_Sym := 16#FF6B#;  
    Xk_Mode_Switch   : constant X_Key_Sym := 16#FF7E#; -- Char set switch  
    Xk_Script_Switch : constant X_Key_Sym := 16#FF7E#; -- Alias for mode_sw  
    Xk_Num_Lock      : constant X_Key_Sym := 16#FF7F#;  

------------------------------------------------------------------------------  
---- Keypad Functions, keypad numbers cleverly chosen to map to ascii  
------------------------------------------------------------------------------  

    Xk_Kp_Space     : constant X_Key_Sym := 16#FF80#;  -- space  
    Xk_Kp_Tab       : constant X_Key_Sym := 16#FF89#;  
    Xk_Kp_Enter     : constant X_Key_Sym := 16#FF8D#;  -- enter  
    Xk_Kp_F1        : constant X_Key_Sym := 16#FF91#;  -- PF1, KP_A, ...  
    Xk_Kp_F2        : constant X_Key_Sym := 16#FF92#;  
    Xk_Kp_F3        : constant X_Key_Sym := 16#FF93#;  
    Xk_Kp_F4        : constant X_Key_Sym := 16#FF94#;  
    Xk_Kp_Equal     : constant X_Key_Sym := 16#FFBD#;  -- equals  
    Xk_Kp_Multiply  : constant X_Key_Sym := 16#FFAA#;  
    Xk_Kp_Add       : constant X_Key_Sym := 16#FFAB#;  
    Xk_Kp_Separator : constant X_Key_Sym := 16#FFAC#; -- separator,often ','  
    Xk_Kp_Subtract  : constant X_Key_Sym := 16#FFAD#;  
    Xk_Kp_Decimal   : constant X_Key_Sym := 16#FFAE#;  
    Xk_Kp_Divide    : constant X_Key_Sym := 16#FFAF#;  
    Xk_Kp_0         : constant X_Key_Sym := 16#FFB0#;  
    Xk_Kp_1         : constant X_Key_Sym := 16#FFB1#;  
    Xk_Kp_2         : constant X_Key_Sym := 16#FFB2#;  
    Xk_Kp_3         : constant X_Key_Sym := 16#FFB3#;  
    Xk_Kp_4         : constant X_Key_Sym := 16#FFB4#;  
    Xk_Kp_5         : constant X_Key_Sym := 16#FFB5#;  
    Xk_Kp_6         : constant X_Key_Sym := 16#FFB6#;  
    Xk_Kp_7         : constant X_Key_Sym := 16#FFB7#;  
    Xk_Kp_8         : constant X_Key_Sym := 16#FFB8#;  
    Xk_Kp_9         : constant X_Key_Sym := 16#FFB9#;  


------------------------------------------------------------------------------  
-- Auxiliary Functions; note the duplicate definitions for left and right  
-- function keys;  Sun keyboards and a few other manufactures have such  
-- function key groups on the left and/or right sides of the keyboard.  
-- We've not found a keyboard with more than 35 function keys total.  
------------------------------------------------------------------------------  

    Xk_F1  : constant X_Key_Sym := 16#FFBE#;  
    Xk_F2  : constant X_Key_Sym := 16#FFBF#;  
    Xk_F3  : constant X_Key_Sym := 16#FFC0#;  
    Xk_F4  : constant X_Key_Sym := 16#FFC1#;  
    Xk_F5  : constant X_Key_Sym := 16#FFC2#;  
    Xk_F6  : constant X_Key_Sym := 16#FFC3#;  
    Xk_F7  : constant X_Key_Sym := 16#FFC4#;  
    Xk_F8  : constant X_Key_Sym := 16#FFC5#;  
    Xk_F9  : constant X_Key_Sym := 16#FFC6#;  
    Xk_F10 : constant X_Key_Sym := 16#FFC7#;  
    Xk_F11 : constant X_Key_Sym := 16#FFC8#;  
    Xk_L1  : constant X_Key_Sym := 16#FFC8#;  
    Xk_F12 : constant X_Key_Sym := 16#FFC9#;  
    Xk_L2  : constant X_Key_Sym := 16#FFC9#;  
    Xk_F13 : constant X_Key_Sym := 16#FFCA#;  
    Xk_L3  : constant X_Key_Sym := 16#FFCA#;  
    Xk_F14 : constant X_Key_Sym := 16#FFCB#;  
    Xk_L4  : constant X_Key_Sym := 16#FFCB#;  
    Xk_F15 : constant X_Key_Sym := 16#FFCC#;  
    Xk_L5  : constant X_Key_Sym := 16#FFCC#;  
    Xk_F16 : constant X_Key_Sym := 16#FFCD#;  
    Xk_L6  : constant X_Key_Sym := 16#FFCD#;  
    Xk_F17 : constant X_Key_Sym := 16#FFCE#;  
    Xk_L7  : constant X_Key_Sym := 16#FFCE#;  
    Xk_F18 : constant X_Key_Sym := 16#FFCF#;  
    Xk_L8  : constant X_Key_Sym := 16#FFCF#;  
    Xk_F19 : constant X_Key_Sym := 16#FFD0#;  
    Xk_L9  : constant X_Key_Sym := 16#FFD0#;  
    Xk_F20 : constant X_Key_Sym := 16#FFD1#;  
    Xk_L10 : constant X_Key_Sym := 16#FFD1#;  
    Xk_F21 : constant X_Key_Sym := 16#FFD2#;  
    Xk_R1  : constant X_Key_Sym := 16#FFD2#;  
    Xk_F22 : constant X_Key_Sym := 16#FFD3#;  
    Xk_R2  : constant X_Key_Sym := 16#FFD3#;  
    Xk_F23 : constant X_Key_Sym := 16#FFD4#;  
    Xk_R3  : constant X_Key_Sym := 16#FFD4#;  
    Xk_F24 : constant X_Key_Sym := 16#FFD5#;  
    Xk_R4  : constant X_Key_Sym := 16#FFD5#;  
    Xk_F25 : constant X_Key_Sym := 16#FFD6#;  
    Xk_R5  : constant X_Key_Sym := 16#FFD6#;  
    Xk_F26 : constant X_Key_Sym := 16#FFD7#;  
    Xk_R6  : constant X_Key_Sym := 16#FFD7#;  
    Xk_F27 : constant X_Key_Sym := 16#FFD8#;  
    Xk_R7  : constant X_Key_Sym := 16#FFD8#;  
    Xk_F28 : constant X_Key_Sym := 16#FFD9#;  
    Xk_R8  : constant X_Key_Sym := 16#FFD9#;  
    Xk_F29 : constant X_Key_Sym := 16#FFDA#;  
    Xk_R9  : constant X_Key_Sym := 16#FFDA#;  
    Xk_F30 : constant X_Key_Sym := 16#FFDB#;  
    Xk_R10 : constant X_Key_Sym := 16#FFDB#;  
    Xk_F31 : constant X_Key_Sym := 16#FFDC#;  
    Xk_R11 : constant X_Key_Sym := 16#FFDC#;  
    Xk_F32 : constant X_Key_Sym := 16#FFDD#;  
    Xk_R12 : constant X_Key_Sym := 16#FFDD#;  
    Xk_R13 : constant X_Key_Sym := 16#FFDE#;  
    Xk_F33 : constant X_Key_Sym := 16#FFDE#;  
    Xk_F34 : constant X_Key_Sym := 16#FFDF#;  
    Xk_R14 : constant X_Key_Sym := 16#FFDF#;  
    Xk_F35 : constant X_Key_Sym := 16#FFE0#;  
    Xk_R15 : constant X_Key_Sym := 16#FFE0#;  

------------------------------------------------------------------------------  
---- Modifiers  
------------------------------------------------------------------------------  

    Xk_Shift_L    : constant X_Key_Sym := 16#FFE1#;  -- Left shift  
    Xk_Shift_R    : constant X_Key_Sym := 16#FFE2#;  -- Right shift  
    Xk_Control_L  : constant X_Key_Sym := 16#FFE3#;  -- Left control  
    Xk_Control_R  : constant X_Key_Sym := 16#FFE4#;  -- Right control  
    Xk_Caps_Lock  : constant X_Key_Sym := 16#FFE5#;  -- Caps lock  
    Xk_Shift_Lock : constant X_Key_Sym := 16#FFE6#;  -- Shift lock  
    Xk_Meta_L     : constant X_Key_Sym := 16#FFE7#;  -- Left meta  
    Xk_Meta_R     : constant X_Key_Sym := 16#FFE8#;  -- Right meta  
    Xk_Alt_L      : constant X_Key_Sym := 16#FFE9#;  -- Left alt  
    Xk_Alt_R      : constant X_Key_Sym := 16#FFEA#;  -- Right alt  
    Xk_Super_L    : constant X_Key_Sym := 16#FFEB#;  -- Left super  
    Xk_Super_R    : constant X_Key_Sym := 16#FFEC#;  -- Right super  
    Xk_Hyper_L    : constant X_Key_Sym := 16#FFED#;  -- Left hyper  
    Xk_Hyper_R    : constant X_Key_Sym := 16#FFEE#;  -- Right hyper  

end Xlbt_Key_Defs_Misc;  

