with Xlbt_Key3;  

package Xlbt_Key is  
------------------------------------------------------------------------------  
-- X Library Key Codes, Symbols, and Translations - Normal User Interface  
--  
-- Xlbt_Key2 - Types and structures used to describe and keep track of keys  
-- on the keyboard.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    package K3 renames Xlbt_Key3;  

------------------------------------------------------------------------------  
-- X_Compose_Status - Compose sequence status structure, used w/X_Lookup_String.  
------------------------------------------------------------------------------  

    subtype X_Compose_Status is K3.X_Compose_Status;  

    None_X_Compose_Status : X_Compose_Status renames K3.None_X_Compose_Status;  

------------------------------------------------------------------------------  
-- X_Key_Button_Mask  
--  
--      Any_Modifier  
--      Button_5_Mask  
--      Button_4_Mask  
--      Button_3_Mask  
--      Button_2_Mask  
--      Button_1_Mask  
--      Mod_5_Mask  
--      Mod_4_Mask  
--      Mod_3_Mask  
--      Mod_2_Mask  
--      Mod_1_Mask  
--      Control_Mask  
--      Lock_Mask  
--      Shift_Mask  
------------------------------------------------------------------------------  

    subtype X_Key_Button_Mask_Index is K3.X_Key_Button_Mask_Index;  
    subtype X_Key_Button_Mask       is K3.X_Key_Button_Mask;  

    Any_Modifier  : constant X_Key_Button_Mask_Index := K3.Any_Modifier;  
    Button_5_Mask : constant X_Key_Button_Mask_Index := K3.Button_5_Mask;  
    Button_4_Mask : constant X_Key_Button_Mask_Index := K3.Button_4_Mask;  
    Button_3_Mask : constant X_Key_Button_Mask_Index := K3.Button_3_Mask;  
    Button_2_Mask : constant X_Key_Button_Mask_Index := K3.Button_2_Mask;  
    Button_1_Mask : constant X_Key_Button_Mask_Index := K3.Button_1_Mask;  
    Mod_5_Mask    : constant X_Key_Button_Mask_Index := K3.Mod_5_Mask;  
    Mod_4_Mask    : constant X_Key_Button_Mask_Index := K3.Mod_4_Mask;  
    Mod_3_Mask    : constant X_Key_Button_Mask_Index := K3.Mod_3_Mask;  
    Mod_2_Mask    : constant X_Key_Button_Mask_Index := K3.Mod_2_Mask;  
    Mod_1_Mask    : constant X_Key_Button_Mask_Index := K3.Mod_1_Mask;  
    Control_Mask  : constant X_Key_Button_Mask_Index := K3.Control_Mask;  
    Lock_Mask     : constant X_Key_Button_Mask_Index := K3.Lock_Mask;  
    Shift_Mask    : constant X_Key_Button_Mask_Index := K3.Shift_Mask;  

    None_X_Key_Button_Mask : X_Key_Button_Mask  
        renames K3.None_X_Key_Button_Mask;  

    function "=" (A, B : X_Key_Button_Mask_Index) return Boolean renames K3."=";  
    function "=" (A, B : X_Key_Button_Mask) return Boolean renames K3."=";  
    function "and" (A, B : X_Key_Button_Mask) return X_Key_Button_Mask  
        renames K3."and";  
    function "or" (A, B : X_Key_Button_Mask) return X_Key_Button_Mask  
        renames K3."or";  
    function "xor" (A, B : X_Key_Button_Mask) return X_Key_Button_Mask  
        renames K3."xor";  
    function "not" (A : X_Key_Button_Mask) return X_Key_Button_Mask  
        renames K3."not";  

------------------------------------------------------------------------------  
-- X_Key_Code - simple "numeric" index of 8 bits  
--  
--  Any_Key - Special X_Key_Code passed to Grab_Key  
------------------------------------------------------------------------------  

    subtype X_Key_Code       is K3.X_Key_Code;  
    subtype X_Key_Code_Array is K3.X_Key_Code_Array;  
    subtype X_Key_Code_List  is K3.X_Key_Code_List;  

    None_X_Key_Code      : X_Key_Code      renames K3.None_X_Key_Code;  
    None_X_Key_Code_List : X_Key_Code_List renames K3.None_X_Key_Code_List;  
    Any_Key              : X_Key_Code      renames K3.Any_Key;  

    function "=" (A, B : X_Key_Code)      return Boolean    renames K3."=";  
    function "=" (A, B : X_Key_Code_List) return Boolean    renames K3."=";  
    function "+" (A, B : X_Key_Code)      return X_Key_Code renames K3."+";  
    function "-" (A, B : X_Key_Code)      return X_Key_Code renames K3."-";  
    function "-" (A : X_Key_Code)         return X_Key_Code renames K3."-";  

    procedure Free_X_Key_Code_List (List : in out X_Key_Code_List)  
        renames K3.Free_X_Key_Code_List;  

------------------------------------------------------------------------------  
-- X_Key_Modifier names.  Used to build X_Set_Modifier_Mapping requests or  
-- to read X_Get_Modifier_Mapping requests.  These correspond to the  
-- masks defined above.  
------------------------------------------------------------------------------  

    subtype X_Key_Modifier is K3.X_Key_Modifier;  

    Shift_Map_Index   : constant X_Key_Modifier := K3.Shift_Map_Index;  
    Lock_Map_Index    : constant X_Key_Modifier := K3.Lock_Map_Index;  
    Control_Map_Index : constant X_Key_Modifier := K3.Control_Map_Index;  
    Mod_1_Map_Index   : constant X_Key_Modifier := K3.Mod_1_Map_Index;  
    Mod_2_Map_Index   : constant X_Key_Modifier := K3.Mod_2_Map_Index;  
    Mod_3_Map_Index   : constant X_Key_Modifier := K3.Mod_3_Map_Index;  
    Mod_4_Map_Index   : constant X_Key_Modifier := K3.Mod_4_Map_Index;  
    Mod_5_Map_Index   : constant X_Key_Modifier := K3.Mod_5_Map_Index;  

------------------------------------------------------------------------------  
-- X_Key_Sym  
------------------------------------------------------------------------------  

    subtype X_Key_Sym is K3.X_Key_Sym;  

    None_X_Key_Sym : X_Key_Sym renames K3.None_X_Key_Sym;  

    No_Symbol : X_Key_Sym renames K3.No_Symbol;  
    ----Special key symbol meaning "this key does nothing"  

    subtype X_Key_Sym_Array    is K3.X_Key_Sym_Array;  
    subtype X_Key_Sym_Array_2d is K3.X_Key_Sym_Array_2d;  
    subtype X_Key_Sym_List_2d  is K3.X_Key_Sym_List_2d;  

    None_X_Key_Sym_List_2d : X_Key_Sym_List_2d  
        renames K3.None_X_Key_Sym_List_2d;  

    function "=" (A, B : X_Key_Sym_List_2d) return Boolean renames K3."=";  


    function "="  (A, B : X_Key_Sym) return Boolean renames K3."=";  
    function "<"  (A, B : X_Key_Sym) return Boolean renames K3."<";  
    function "<=" (A, B : X_Key_Sym) return Boolean renames K3."<=";  
    function ">"  (A, B : X_Key_Sym) return Boolean renames K3.">";  
    function ">=" (A, B : X_Key_Sym) return Boolean renames K3.">=";  

    function  "+" (A, B : X_Key_Sym) return X_Key_Sym renames K3."+";  
    function  "-" (A, B : X_Key_Sym) return X_Key_Sym renames K3."-";  
    function  "-" (A : X_Key_Sym) return X_Key_Sym renames K3."-";  
    function  "*" (A, B : X_Key_Sym) return X_Key_Sym renames K3."*";  
    function  "/" (A, B : X_Key_Sym) return X_Key_Sym renames K3."/";  
    function  "rem" (A, B : X_Key_Sym) return X_Key_Sym renames K3."rem";  
    function  "mod" (A, B : X_Key_Sym) return X_Key_Sym renames K3."mod";  
    procedure Free_X_Key_Sym_List_2d (List : in out X_Key_Sym_List_2d)  
        renames K3.Free_X_Key_Sym_List_2d;  

------------------------------------------------------------------------------  
-- X_Modifier_Keymap  
------------------------------------------------------------------------------  

    subtype X_Modifier_Keymap is K3.X_Modifier_Keymap;  

    None_X_Modifier_Keymap : X_Modifier_Keymap  
        renames K3.None_X_Modifier_Keymap;  

    function "=" (A, B : X_Modifier_Keymap) return Boolean renames K3."=";  

    procedure Free_X_Modifier_Keymap (Map : in out X_Modifier_Keymap)  
        renames K3.Free_X_Modifier_Keymap;  

end Xlbt_Key;  

