with Unchecked_Deallocation;  

with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Xlbt_Region3 is  
------------------------------------------------------------------------------  
-- X Library Regions - Full Interface  
--  
-- Xlbt_Region3 - Structures for working with regions of images/screens/windows  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1987 - 1989 by Digital Equipment Corporation, Maynard, Mass.  
-- Copyright 1987 - 1989 by Massachusetts Institute of Technology,  
--                          Cambridge, Massachusetts.  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of Digital, MIT, or Rational  
-- not be used in advertising or publicity pertaining to distribution of  
-- the software without specific, written prior permission.  
--  
-- Digital, MIT, and Rational disclaim all warranties with regard to this  
-- software, including all implied warranties of merchantability and fitness,  
-- in no event shall Digital, MIT, or Rational be liable for any special,  
-- indirect or consequential damages or any damages whatsoever resulting from  
-- loss of use, data or profits, whether in an action of contract, negligence  
-- or other tortious action, arising out of or in connection with the use or  
-- performance of this software.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- X_Box_Rec  
------------------------------------------------------------------------------  

    type X_Box_Rec is  
        record  
            X1 : S_Short := 0;  
            X2 : S_Short := 0;  
            Y1 : S_Short := 0;  
            Y2 : S_Short := 0;  
        end record;  

    type X_Box_Array is array (S_Natural range <>) of X_Box_Rec;  
    type X_Box_List  is access X_Box_Array;  

--/ if Pack then  
    pragma Pack (X_Box_Array);  
--/ end if;  
--/ if Enable_Deallocation then  
    pragma Enable_Deallocation (X_Box_List);  
--/ end if;  

    None_X_Box_Rec  : constant X_Box_Rec  := (0, 0, 0, 0);  
    None_X_Box_List : constant X_Box_List := null;  

    procedure Free_X_Box_List is new Unchecked_Deallocation (X_Box_Array,  
                                                             X_Box_List);  

------------------------------------------------------------------------------  
-- Return values from X_Rect_In_Region()  
------------------------------------------------------------------------------  

    type X_Rect_In_Region_Returns is (Rectangle_Out,  
                                      Rectangle_In,  
                                      Rectangle_Part);  

------------------------------------------------------------------------------  
-- X_Region - clip regions  
------------------------------------------------------------------------------  

    type X_Region_Rec is  
        record  
            Size      : S_Natural  := 0;  
            Num_Rects : S_Natural  := 0;  
            Rects     : X_Box_List := null;  
            Extents   : X_Box_Rec;  
        end record;  

    type X_Region is access X_Region_Rec;  

    None_X_Region : constant X_Region := null;  

    procedure Free_X_Region (Region : in out X_Region);  

--/ if Enable_Deallocation then  
    pragma Enable_Deallocation (X_Region);  
--/ end if;  

end Xlbt_Region3;  

