with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic3;  
use Xlbt_Basic3;  
with Xlbt_Display3;  
use Xlbt_Display3;  
with Xlbt_Visual3;  
use Xlbt_Visual3;  
with Xlbt_Window4;  
use Xlbt_Window4;  

package Xlbt_Window3 is  
------------------------------------------------------------------------------  
-- X Library Windows - Full Interface - Records and Arrays  
--  
-- Xlbt_Window3 - Types dealing with windows and window state  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- X_Set_Window_Attributes - used with X_New_Window_Attributes to set/change  
-- a window's attributes.  
------------------------------------------------------------------------------  

    type X_Set_Window_Attributes is  
        record  
            Background_Pixmap : X_Pixmap := None_X_Pixmap;  
            ----Background, None, Parent_Relative  
            Background_Pixel : X_Pixel := None_X_Pixel;  
            ----Background pixel  
            Border_Pixmap : X_Pixmap := None_X_Pixmap;  
            ----Border of the window  
            Border_Pixel : X_Pixel := None_X_Pixel;  
            ----Border pixel value  
            Bit_Gravity : X_Bit_Gravity := None_X_Bit_Gravity;  
            ----One of bit gravity values  
            Win_Gravity : X_Window_Gravity := None_X_Window_Gravity;  
            ----One of the window gravity values  
            Backing_Store : X_Backing_Store_Hint := None_X_Backing_Store_Hint;  
            ----Not_Useful, When_Mapped, Always  
            Backing_Planes : X_Plane_Mask := No_Planes;  
            ----Planes to be preserved if possible  
            Backing_Pixel : X_Pixel := None_X_Pixel;  
            ----Value to use in restoring planes  
            Override_Redirect : Boolean := False;  
            ----Boolean for override-redirect  
            Save_Under : Boolean := False;  
            ----Should bits under be saved?  (popups)  
            Event_Mask : X_Event_Mask := None_X_Event_Mask;  
            ----Set of events that should be saved  
            Do_Not_Propagate_Mask : X_Event_Mask := None_X_Event_Mask;  
            ----Set of events that should not propagate  
            Colormap : X_Colormap := None_X_Colormap;  
            ----Color map to be associated with window  
            Cursor : X_Cursor := None_X_Cursor;  
            ----Cursor to be displayed (or None)  
        end record;  

------------------------------------------------------------------------------  
-- X_Window_Attributes - returned in response to a query about window attributes  
------------------------------------------------------------------------------  

    type X_Window_Attributes is  
        record  
            X : S_Short := 0;  
            Y : S_Short := 0;  
            ----Location of window relative to its parent's origin  
            Width : U_Short := 0;  
            ----Width of the window  
            Height : U_Short := 0;  
            ----Height of the window  
            Border_Width : U_Short := 0;  
            ----Width of window border  
            Depth : U_Char := 0;  
            ----Depth of window  
            Visual : X_Visual;  
            ----The associated visual structure  
            Root : X_Window := None_X_Window;  
            ----Root of screen containing window  
            Class : X_Window_Class := None_X_Window_Class;  
            ----Input_Output, Input_Only  
            Bit_Gravity : X_Bit_Gravity := None_X_Bit_Gravity;  
            ----One of bit gravity values  
            Win_Gravity : X_Window_Gravity := None_X_Window_Gravity;  
            ----One of the window gravity values  
            Backing_Store : X_Backing_Store_Hint := None_X_Backing_Store_Hint;  
            ----Not_Useful, When_Mapped, Always  
            Backing_Planes : X_Plane_Mask := No_Planes;  
            ----Planes to be preserved if possible  
            Backing_Pixel : X_Pixel := None_X_Pixel;  
            ----Value to use when restoring planes  
            Save_Under : Boolean := False;  
            ----Should bits under be saved?  
            Colormap : X_Colormap := None_X_Colormap;  
            ----Color map to associate with window  
            Map_Installed : Boolean := False;  
            ----Is color map currently installed  
            Map_State : X_Map_State := None_X_Map_State;  
            ----Is_Unmapped/Unviewable/Viewable  
            All_Event_Masks : X_Event_Mask := None_X_Event_Mask;  
            ----Set of events all people have interest in  
            Your_Event_Mask : X_Event_Mask := None_X_Event_Mask;  
            ----My event mask  
            Do_Not_Propagate_Mask : X_Event_Mask := None_X_Event_Mask;  
            ----Set of events that should not propagate  
            Override_Redirect : Boolean := False;  
            ----Value for override-redirect  
            Screen : X_Screen := None_X_Screen;  
            ----Back pointer to correct screen  
        end record;  

------------------------------------------------------------------------------  
-- X_Window_Changes - used to make several changes at once  
------------------------------------------------------------------------------  

    type X_Window_Changes is  
        record  
            X : S_Short := 0;  
            Y : S_Short := 0;  
            ----Position of upper left corner relative to parent origin  
            Width : U_Short_Positive := 1;  
            ----Width of window, must be non-zero  
            Height : U_Short_Positive := 1;  
            ----Height of window, must be non-zero  
            Border_Width : U_Short := 0;  
            ----Border width of window.  
            Sibling : X_Window := None_X_Window;  
            ----Stack relative to this sibling.  
            Stack_Mode : X_Window_Stacking := None_X_Window_Stacking;  
            ----Stacking order relative to Sibling  
        end record;  

end Xlbt_Window3;  

